@@ -1,5 +1,5 @@
 /* Copyright (C) 2003, 2004, 2005, 2006, 2008, 2009 Dean Beeler, Jerome Fisher
- * Copyright (C) 2011 Dean Beeler, Jerome Fisher, Sergey V. Mikayev
+ * Copyright (C) 2011, 2012, 2013, 2014 Dean Beeler, Jerome Fisher, Sergey V. Mikayev
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -26,6 +26,8 @@ class TableInitialiser;
 class Partial;
 class PartialManager;
 class Part;
+class ROMImage;
+class BReverbModel;
 
 /**
  * Methods for emulating the connection between the LA32 and the DAC, which involves
@@ -43,6 +45,7 @@ enum DACInputMode {
 	// * Much less likely to overdrive than any other mode.
 	// * Half the volume of any of the other modes, meaning its volume relative to the reverb
 	//   output when mixed together directly will sound wrong.
+	// * Output gain is ignored for both LA32 and reverb output.
 	// * Perfect for developers while debugging :)
 	DACInputMode_PURE,
 
@@ -57,73 +60,18 @@ enum DACInputMode {
 	DACInputMode_GENERATION2
 };
 
-enum ReportType {
-	// Errors
-	ReportType_errorControlROM = 1,
-	ReportType_errorPCMROM,
-	ReportType_errorSampleRate,
+enum MIDIDelayMode {
+	// Process incoming MIDI events immediately.
+	MIDIDelayMode_IMMEDIATE,
 
-	// Progress
-	ReportType_progressInit,
+	// Delay incoming short MIDI messages as if they where transferred via a MIDI cable to a real hardware unit and immediate sysex processing.
+	// This ensures more accurate timing of simultaneous NoteOn messages.
+	MIDIDelayMode_DELAY_SHORT_MESSAGES_ONLY,
 
-	// HW spec
-	ReportType_availableSSE,
-	ReportType_available3DNow,
-	ReportType_usingSSE,
-	ReportType_using3DNow,
-
-	// General info
-	ReportType_lcdMessage,
-	ReportType_devReset,
-	ReportType_devReconfig,
-	ReportType_newReverbMode,
-	ReportType_newReverbTime,
-	ReportType_newReverbLevel
+	// Delay all incoming MIDI events as if they where transferred via a MIDI cable to a real hardware unit.
+	MIDIDelayMode_DELAY_ALL
 };
 
-enum LoadResult {
-	LoadResult_OK,
-	LoadResult_NotFound,
-	LoadResult_Unreadable,
-	LoadResult_Invalid
-};
-
-struct SynthProperties {
-	// Sample rate to use in mixing
-	unsigned int sampleRate;
-
-	// Deprecated - ignored. Use Synth::setReverbEnabled() instead.
-	bool useReverb;
-	// Deprecated - ignored. Use Synth::setReverbOverridden() instead.
-	bool useDefaultReverb;
-	// Deprecated - ignored. Use Synth::playSysex*() to configure reverb instead.
-	unsigned char reverbType;
-	// Deprecated - ignored. Use Synth::playSysex*() to configure reverb instead.
-	unsigned char reverbTime;
-	// Deprecated - ignored. Use Synth::playSysex*() to configure reverb instead.
-	unsigned char reverbLevel;
-	// The name of the directory in which the ROM and data files are stored (with trailing slash/backslash)
-	// Not used if "openFile" is set. May be NULL in any case.
-	const char *baseDir;
-	// This is used as the first argument to all callbacks
-	void *userData;
-	// Callback for reporting various errors and information. May be NULL
-	int (*report)(void *userData, ReportType type, const void *reportData);
-	// Callback for debug messages, in vprintf() format
-	void (*printDebug)(void *userData, const char *fmt, va_list list);
-	// Callback for providing an implementation of File, opened and ready for use
-	// May be NULL, in which case a default implementation will be used.
-	Common::File *(*openFile)(void *userData, const char *filename);
-	// Callback for closing a File. May be NULL, in which case the File will automatically be close()d/deleted.
-	void (*closeFile)(void *userData, Common::File *file);
-};
-
-// This is the specification of the Callback routine used when calling the RecalcWaveforms
-// function
-typedef void (*recalcStatusCallback)(int percDone);
-
-typedef void (*FloatToBit16sFunc)(Bit16s *target, const float *source, Bit32u len, float outputGain);
-
 const Bit8u SYSEX_MANUFACTURER_ROLAND = 0x41;
 
 const Bit8u SYSEX_MDL_MT32 = 0x16;
@@ -179,6 +127,13 @@ enum MemoryRegionType {
 	MR_PatchTemp, MR_RhythmTemp, MR_TimbreTemp, MR_Patches, MR_Timbres, MR_System, MR_Display, MR_Reset
 };
 
+enum ReverbMode {
+	REVERB_MODE_ROOM,
+	REVERB_MODE_HALL,
+	REVERB_MODE_PLATE,
+	REVERB_MODE_TAP_DELAY
+};
+
 class MemoryRegion {
 private:
 	Synth *synth;
@@ -274,16 +229,70 @@ public:
 	ResetMemoryRegion(Synth *useSynth) : MemoryRegion(useSynth, NULL, NULL, MR_Reset, MT32EMU_MEMADDR(0x7F0000), 0x3FFF, 1) {}
 };
 
-class ReverbModel {
+class ReportHandler {
+friend class Synth;
+
 public:
-	virtual ~ReverbModel() {}
-	// After construction or a close(), open() will be called at least once before any other call (with the exception of close()).
-	virtual void open(unsigned int sampleRate) = 0;
-	// May be called multiple times without an open() in between.
-	virtual void close() = 0;
-	virtual void setParameters(Bit8u time, Bit8u level) = 0;
-	virtual void process(const float *inLeft, const float *inRight, float *outLeft, float *outRight, unsigned long numSamples) = 0;
-	virtual bool isActive() const = 0;
+	virtual ~ReportHandler() {}
+
+protected:
+
+	// Callback for debug messages, in vprintf() format
+	virtual void printDebug(const char *fmt, va_list list);
+
+	// Callbacks for reporting various errors and information
+	virtual void onErrorControlROM() {}
+	virtual void onErrorPCMROM() {}
+	virtual void showLCDMessage(const char *message);
+	virtual void onMIDIMessagePlayed() {}
+	virtual void onDeviceReset() {}
+	virtual void onDeviceReconfig() {}
+	virtual void onNewReverbMode(Bit8u /* mode */) {}
+	virtual void onNewReverbTime(Bit8u /* time */) {}
+	virtual void onNewReverbLevel(Bit8u /* level */) {}
+	virtual void onPolyStateChanged(int /* partNum */) {}
+	virtual void onProgramChanged(int /* partNum */, int /* bankNum */, const char * /* patchName */) {}
+};
+
+/**
+ * Used to safely store timestamped MIDI events in a local queue.
+ */
+struct MidiEvent {
+	Bit32u shortMessageData;
+	const Bit8u *sysexData;
+	Bit32u sysexLength;
+	Bit32u timestamp;
+
+	~MidiEvent();
+	void setShortMessage(Bit32u shortMessageData, Bit32u timestamp);
+	void setSysex(const Bit8u *sysexData, Bit32u sysexLength, Bit32u timestamp);
+};
+
+/**
+ * Simple queue implementation using a ring buffer to store incoming MIDI event before the synth actually processes it.
+ * It is intended to:
+ * - get rid of prerenderer while retaining graceful partial abortion
+ * - add fair emulation of the MIDI interface delays
+ * - extend the synth interface with the default implementation of a typical rendering loop.
+ * THREAD SAFETY:
+ * It is safe to use either in a single thread environment or when there are only two threads - one performs only reading
+ * and one performs only writing. More complicated usage requires external synchronisation.
+ */
+class MidiEventQueue {
+private:
+	MidiEvent *ringBuffer;
+	Bit32u ringBufferSize;
+	volatile Bit32u startPosition;
+	volatile Bit32u endPosition;
+
+public:
+	MidiEventQueue(Bit32u ringBufferSize = DEFAULT_MIDI_EVENT_QUEUE_SIZE);
+	~MidiEventQueue();
+	void reset();
+	bool pushShortMessage(Bit32u shortMessageData, Bit32u timestamp);
+	bool pushSysex(const Bit8u *sysexData, Bit32u sysexLength, Bit32u timestamp);
+	const MidiEvent *peekMidiEvent();
+	void dropMidiEvent();
 };
 
 class Synth {
@@ -291,6 +300,7 @@ friend class Part;
 friend class RhythmPart;
 friend class Poly;
 friend class Partial;
+friend class PartialManager;
 friend class Tables;
 friend class MemoryRegion;
 friend class TVA;
@@ -312,71 +322,60 @@ private:
 
 	PCMWaveEntry *pcmWaves; // Array
 
+	const ControlROMFeatureSet *controlROMFeatures;
 	const ControlROMMap *controlROMMap;
 	Bit8u controlROMData[CONTROL_ROM_SIZE];
-	float *pcmROMData;
-	int pcmROMSize; // This is in 16-bit samples, therefore half the number of bytes in the ROM
+	Bit16s *pcmROMData;
+	size_t pcmROMSize; // This is in 16-bit samples, therefore half the number of bytes in the ROM
 
-	Bit8s chantable[32];
+	unsigned int partialCount;
+	Bit8s chantable[32]; // FIXME: Need explanation why 32 is set, obviously it should be 16
 
-	Bit32u renderedSampleCount;
-
-	Tables tables;
+	MidiEventQueue *midiQueue;
+	volatile Bit32u lastReceivedMIDIEventTimestamp;
+	volatile Bit32u renderedSampleCount;
 
 	MemParams mt32ram, mt32default;
 
-	ReverbModel *reverbModels[4];
-	ReverbModel *reverbModel;
-	bool reverbEnabled;
+	BReverbModel *reverbModels[4];
+	BReverbModel *reverbModel;
 	bool reverbOverridden;
 
-	FloatToBit16sFunc la32FloatToBit16sFunc;
-	FloatToBit16sFunc reverbFloatToBit16sFunc;
+	MIDIDelayMode midiDelayMode;
+	DACInputMode dacInputMode;
+
 	float outputGain;
 	float reverbOutputGain;
+#if MT32EMU_USE_FLOAT_SAMPLES
+	float effectiveReverbOutputGain;
+#else
+	int effectiveOutputGain;
+	int effectiveReverbOutputGain;
+#endif
+
+	bool reversedStereoEnabled;
 
 	bool isOpen;
 
+	bool isDefaultReportHandler;
+	ReportHandler *reportHandler;
+
 	PartialManager *partialManager;
 	Part *parts[9];
 
-	// FIXME: We can reorganise things so that we don't need all these separate tmpBuf, tmp and prerender buffers.
-	// This should be rationalised when things have stabilised a bit (if prerender buffers don't die in the mean time).
-
-	float tmpBufPartialLeft[MAX_SAMPLES_PER_RUN];
-	float tmpBufPartialRight[MAX_SAMPLES_PER_RUN];
-	float tmpBufMixLeft[MAX_SAMPLES_PER_RUN];
-	float tmpBufMixRight[MAX_SAMPLES_PER_RUN];
-	float tmpBufReverbOutLeft[MAX_SAMPLES_PER_RUN];
-	float tmpBufReverbOutRight[MAX_SAMPLES_PER_RUN];
-
-	Bit16s tmpNonReverbLeft[MAX_SAMPLES_PER_RUN];
-	Bit16s tmpNonReverbRight[MAX_SAMPLES_PER_RUN];
-	Bit16s tmpReverbDryLeft[MAX_SAMPLES_PER_RUN];
-	Bit16s tmpReverbDryRight[MAX_SAMPLES_PER_RUN];
-	Bit16s tmpReverbWetLeft[MAX_SAMPLES_PER_RUN];
-	Bit16s tmpReverbWetRight[MAX_SAMPLES_PER_RUN];
-
-	// These ring buffers are only used to simulate delays present on the real device.
-	// In particular, when a partial needs to be aborted to free it up for use by a new Poly,
+	// When a partial needs to be aborted to free it up for use by a new Poly,
 	// the controller will busy-loop waiting for the sound to finish.
-	Bit16s prerenderNonReverbLeft[MAX_PRERENDER_SAMPLES];
-	Bit16s prerenderNonReverbRight[MAX_PRERENDER_SAMPLES];
-	Bit16s prerenderReverbDryLeft[MAX_PRERENDER_SAMPLES];
-	Bit16s prerenderReverbDryRight[MAX_PRERENDER_SAMPLES];
-	Bit16s prerenderReverbWetLeft[MAX_PRERENDER_SAMPLES];
-	Bit16s prerenderReverbWetRight[MAX_PRERENDER_SAMPLES];
-	int prerenderReadIx;
-	int prerenderWriteIx;
+	// We emulate this by delaying new MIDI events processing until abortion finishes.
+	Poly *abortingPoly;
 
-	SynthProperties myProp;
+	Bit32u getShortMessageLength(Bit32u msg);
+	Bit32u addMIDIInterfaceDelay(Bit32u len, Bit32u timestamp);
 
-	bool prerender();
-	void copyPrerender(Bit16s *nonReverbLeft, Bit16s *nonReverbRight, Bit16s *reverbDryLeft, Bit16s *reverbDryRight, Bit16s *reverbWetLeft, Bit16s *reverbWetRight, Bit32u pos, Bit32u len);
-	void checkPrerender(Bit16s *nonReverbLeft, Bit16s *nonReverbRight, Bit16s *reverbDryLeft, Bit16s *reverbDryRight, Bit16s *reverbWetLeft, Bit16s *reverbWetRight, Bit32u &pos, Bit32u &len);
-	void doRenderStreams(Bit16s *nonReverbLeft, Bit16s *nonReverbRight, Bit16s *reverbDryLeft, Bit16s *reverbDryRight, Bit16s *reverbWetLeft, Bit16s *reverbWetRight, Bit32u len);
+	void produceLA32Output(Sample *buffer, Bit32u len);
+	void convertSamplesToOutput(Sample *buffer, Bit32u len, bool reverb);
+	bool isAbortingPoly() const;
+	void doRenderStreams(Sample *nonReverbLeft, Sample *nonReverbRight, Sample *reverbDryLeft, Sample *reverbDryRight, Sample *reverbWetLeft, Sample *reverbWetRight, Bit32u len);
 
-	void playAddressedSysex(unsigned char channel, const Bit8u *sysex, Bit32u len);
 	void readSysex(unsigned char channel, const Bit8u *sysex, Bit32u len) const;
 	void initMemoryRegions();
 	void deleteMemoryRegions();
@@ -384,8 +383,8 @@ private:
 	void writeMemoryRegion(const MemoryRegion *region, Bit32u addr, Bit32u len, const Bit8u *data);
 	void readMemoryRegion(const MemoryRegion *region, Bit32u addr, Bit32u len, Bit8u *data);
 
-	LoadResult loadControlROM(const char *filename);
-	LoadResult loadPCMROM(const char *filename);
+	bool loadControlROM(const ROMImage &controlROMImage);
+	bool loadPCMROM(const ROMImage &pcmROMImage);
 
 	bool initPCMList(Bit16u mapAddress, Bit16u count);
 	bool initTimbres(Bit16u mapAddress, Bit16u offset, int timbreCount, int startTimbre, bool compressed);
@@ -399,58 +398,127 @@ private:
 	void refreshSystem();
 	void reset();
 
-	unsigned int getSampleRate() const;
-
 	void printPartialUsage(unsigned long sampleOffset = 0);
-protected:
-	int report(ReportType type, const void *reportData);
-	Common::File *openFile(const char *filename);
-	void closeFile(Common::File *file);
+
+	void polyStateChanged(int partNum);
+	void newTimbreSet(int partNum, Bit8u timbreGroup, const char patchName[]);
 	void printDebug(const char *fmt, ...);
 
 public:
+	static inline Bit16s clipBit16s(Bit32s sample) {
+		// Clamp values above 32767 to 32767, and values below -32768 to -32768
+		// FIXME: Do we really need this stuff? I think these branches are very well predicted. Instead, this introduces a chain.
+		// The version below is actually a bit faster on my system...
+		//return ((sample + 0x8000) & ~0xFFFF) ? (sample >> 31) ^ 0x7FFF : (Bit16s)sample;
+		return ((-0x8000 <= sample) && (sample <= 0x7FFF)) ? (Bit16s)sample : (sample >> 31) ^ 0x7FFF;
+	}
+
+	static inline void muteSampleBuffer(Sample *buffer, Bit32u len) {
+		if (buffer == NULL) return;
+
+#if MT32EMU_USE_FLOAT_SAMPLES
+		// FIXME: Use memset() where compatibility is guaranteed (if this turns out to be a win)
+		while (len--) {
+			*(buffer++) = 0.0f;
+		}
+#else
+		memset(buffer, 0, len * sizeof(Sample));
+#endif
+	}
+
 	static Bit8u calcSysexChecksum(const Bit8u *data, Bit32u len, Bit8u checksum);
 
-	Synth();
+	// Optionally sets callbacks for reporting various errors, information and debug messages
+	Synth(ReportHandler *useReportHandler = NULL);
 	~Synth();
 
 	// Used to initialise the MT-32. Must be called before any other function.
 	// Returns true if initialization was sucessful, otherwise returns false.
-	bool open(SynthProperties &useProp);
+	// controlROMImage and pcmROMImage represent Control and PCM ROM images for use by synth.
+	// usePartialCount sets the maximum number of partials playing simultaneously for this session.
+	bool open(const ROMImage &controlROMImage, const ROMImage &pcmROMImage, unsigned int usePartialCount = DEFAULT_MAX_PARTIALS);
 
 	// Closes the MT-32 and deallocates any memory used by the synthesizer
-	void close(void);
+	void close(bool forced = false);
 
-	// Sends a 4-byte MIDI message to the MT-32 for immediate playback
-	void playMsg(Bit32u msg);
+	// All the enqueued events are processed by the synth immediately.
+	void flushMIDIQueue();
+
+	// Sets size of the internal MIDI event queue.
+	// The queue is flushed before reallocation.
+	void setMIDIEventQueueSize(Bit32u);
+
+	// Enqueues a MIDI event for subsequent playback.
+	// The minimum delay involves the delay introduced while the event is transferred via MIDI interface
+	// and emulation of the MCU busy-loop while it frees partials for use by a new Poly.
+	// Calls from multiple threads must be synchronised, although,
+	// no synchronisation is required with the rendering thread.
+
+	// The MIDI event will be processed not before the specified timestamp.
+	// The timestamp is measured as the global rendered sample count since the synth was created.
+	bool playMsg(Bit32u msg, Bit32u timestamp);
+	bool playSysex(const Bit8u *sysex, Bit32u len, Bit32u timestamp);
+	// The MIDI event will be processed ASAP.
+	bool playMsg(Bit32u msg);
+	bool playSysex(const Bit8u *sysex, Bit32u len);
+
+	// WARNING:
+	// The methods below don't ensure minimum 1-sample delay between sequential MIDI events,
+	// and a sequence of NoteOn and immediately succeeding NoteOff messages is always silent.
+
+	// Sends a 4-byte MIDI message to the MT-32 for immediate playback.
+	void playMsgNow(Bit32u msg);
 	void playMsgOnPart(unsigned char part, unsigned char code, unsigned char note, unsigned char velocity);
 
 	// Sends a string of Sysex commands to the MT-32 for immediate interpretation
 	// The length is in bytes
-	void playSysex(const Bit8u *sysex, Bit32u len);
+	void playSysexNow(const Bit8u *sysex, Bit32u len);
 	void playSysexWithoutFraming(const Bit8u *sysex, Bit32u len);
 	void playSysexWithoutHeader(unsigned char device, unsigned char command, const Bit8u *sysex, Bit32u len);
 	void writeSysex(unsigned char channel, const Bit8u *sysex, Bit32u len);
 
 	void setReverbEnabled(bool reverbEnabled);
 	bool isReverbEnabled() const;
+	// Sets override reverb mode. In this mode, emulation ignores sysexes (or the related part of them) which control the reverb parameters.
+	// This mode is in effect until it is turned off. When the synth is re-opened, the override mode is unchanged but the state
+	// of the reverb model is reset to default.
 	void setReverbOverridden(bool reverbOverridden);
 	bool isReverbOverridden() const;
+	// Forces reverb model compatibility mode. By default, the compatibility mode corresponds to the used control ROM version.
+	// Invoking this method with the argument set to true forces emulation of old MT-32 reverb circuit.
+	// When the argument is false, emulation of the reverb circuit used in new generation of MT-32 compatible modules is enforced
+	// (these include CM-32L and LAPC-I).
+	void setReverbCompatibilityMode(bool mt32CompatibleMode);
+	bool isMT32ReverbCompatibilityMode() const;
 	void setDACInputMode(DACInputMode mode);
+	DACInputMode getDACInputMode() const;
+	void setMIDIDelayMode(MIDIDelayMode mode);
+	MIDIDelayMode getMIDIDelayMode() const;
 
 	// Sets output gain factor. Applied to all output samples and unrelated with the synth's Master volume.
+	// Ignored in DACInputMode_PURE
 	void setOutputGain(float);
+	float getOutputGain() const;
 
 	// Sets output gain factor for the reverb wet output. setOutputGain() doesn't change reverb output gain.
+	// Note: We're currently emulate CM-32L/CM-64 reverb quite accurately and the reverb output level closely
+	// corresponds to the level of digital capture. Although, according to the CM-64 PCB schematic,
+	// there is a difference in the reverb analogue circuit, and the resulting output gain is 0.68
+	// of that for LA32 analogue output. This factor is applied to the reverb output gain.
+	// Ignored in DACInputMode_PURE
 	void setReverbOutputGain(float);
+	float getReverbOutputGain() const;
+
+	void setReversedStereoEnabled(bool enabled);
+	bool isReversedStereoEnabled();
 
 	// Renders samples to the specified output stream.
 	// The length is in frames, not bytes (in 16-bit stereo,
 	// one frame is 4 bytes).
-	void render(Bit16s *stream, Bit32u len);
+	void render(Sample *stream, Bit32u len);
 
 	// Renders samples to the specified output streams (any or all of which may be NULL).
-	void renderStreams(Bit16s *nonReverbLeft, Bit16s *nonReverbRight, Bit16s *reverbDryLeft, Bit16s *reverbDryRight, Bit16s *reverbWetLeft, Bit16s *reverbWetRight, Bit32u len);
+	void renderStreams(Sample *nonReverbLeft, Sample *nonReverbRight, Sample *reverbDryLeft, Sample *reverbDryRight, Sample *reverbWetLeft, Sample *reverbWetRight, Bit32u len);
 
 	// Returns true when there is at least one active partial, otherwise false.
 	bool hasActivePartials() const;
@@ -460,6 +528,9 @@ public:
 
 	const Partial *getPartial(unsigned int partialNum) const;
 
+	// Returns the maximum number of partials playing simultaneously.
+	unsigned int getPartialCount() const;
+
 	void readMemory(Bit32u addr, Bit32u len, Bit8u *data);
 
 	// partNum should be 0..7 for Part 1..8, or 8 for Rhythm
