@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -101,6 +101,7 @@ int SdlEventSource::mapKey(SDLKey key, SDLMod mod, Uint16 unicode) {
 	return key;
 }
 
+// ResidualVM specific relMouse x,y
 void SdlEventSource::processMouseEvent(Common::Event &event, int x, int y, int relx, int rely) {
 	event.mouse.x = x;
 	event.mouse.y = y;
@@ -118,7 +119,9 @@ void SdlEventSource::processMouseEvent(Common::Event &event, int x, int y, int r
 }
 
 void SdlEventSource::handleKbdMouse() {
-	uint32 curTime = g_system->getMillis();
+	// Skip recording of these events
+	uint32 curTime = g_system->getMillis(true);
+
 	if (curTime >= _km.last_time + _km.delay_time) {
 		_km.last_time = curTime;
 		if (_km.x_down_count == 1) {
@@ -204,6 +207,8 @@ void SdlEventSource::SDLModToOSystemKeyFlags(SDLMod mod, Common::Event &event) {
 #endif
 	if (mod & KMOD_CTRL)
 		event.kbd.flags |= Common::KBD_CTRL;
+	if (mod & KMOD_META)
+		event.kbd.flags |= Common::KBD_META;
 
 	// Sticky flags
 	if (mod & KMOD_NUM)
@@ -400,8 +405,17 @@ bool SdlEventSource::dispatchSDLEvent(SDL_Event &ev, Common::Event &event) {
 		return false;
 
 	case SDL_VIDEORESIZE:
-		if (_graphicsManager)
+		if (_graphicsManager) {
 			_graphicsManager->notifyResize(ev.resize.w, ev.resize.h);
+
+			// If the screen changed, send an Common::EVENT_SCREEN_CHANGED
+			int screenID = ((OSystem_SDL *)g_system)->getGraphicsManager()->getScreenChangeID();
+			if (screenID != _lastScreenID) {
+				_lastScreenID = screenID;
+				event.type = Common::EVENT_SCREEN_CHANGED;
+				return true;
+			}
+		}
 		return false;
 
 	case SDL_QUIT:
@@ -528,7 +542,7 @@ bool SdlEventSource::handleKeyUp(SDL_Event &ev, Common::Event &event) {
 
 bool SdlEventSource::handleMouseMotion(SDL_Event &ev, Common::Event &event) {
 	event.type = Common::EVENT_MOUSEMOVE;
-	processMouseEvent(event, ev.motion.x, ev.motion.y, ev.motion.xrel, ev.motion.yrel);
+	processMouseEvent(event, ev.motion.x, ev.motion.y, ev.motion.xrel, ev.motion.yrel); // ResidualVM xrel,yrel
 
 	return true;
 }
