@@ -27,8 +27,12 @@
 #define PROJECT_NAME "residualvm"         // Used for folders, icons, resources and project/solution name
 #define LIBS_DEFINE "RESIDUALVM_LIBS"     // Name of the include environment variable
 #define REVISION_DEFINE "SCUMMVM_INTERNAL_REVISION"
+#define FIRST_ENGINE "grim"               // Name of the engine which should be sorted as first element
 
-#define ADDITIONAL_LIBRARY "glu32"
-#define NEEDS_RTTI 0
+#define ENABLE_LANGUAGE_EXTENSIONS "grim,myst3"    // Comma separated list of projects that need language extensions
+#define DISABLE_EDIT_AND_CONTINUE ""     // Comma separated list of projects that need Edit&Continue to be disabled for co-routine support (the main project is automatically added)
+
+//#define ADDITIONAL_LIBRARY ""            // Add a single library to the list of externally linked libraries
+#define NEEDS_RTTI 1                     // Enable RTTI globally
 
 #endif // TOOLS_CREATE_PROJECT_CONFIG_H
