@@ -29,7 +29,7 @@
 namespace CreateProjectTool {
 
 //////////////////////////////////////////////////////////////////////////
-// Visual Studio Provider (Visual Studio 2005 & 2008)
+// Visual Studio Provider (Visual Studio 2008)
 //////////////////////////////////////////////////////////////////////////
 
 VisualStudioProvider::VisualStudioProvider(StringList &global_warnings, std::map<std::string, StringList> &project_warnings, const int version)
@@ -48,9 +48,6 @@ int VisualStudioProvider::getVisualStudioVersion() {
 	if (_version == 9)
 		return 2008;
 
-	if (_version == 8)
-		return 2005;
-
 	error("Unsupported version passed to getVisualStudioVersion");
 }
 
@@ -70,8 +67,7 @@ void VisualStudioProvider::createProjectFile(const std::string &name, const std:
 	           "\tRootNamespace=\"" << name << "\"\n"
 	           "\tKeyword=\"Win32Proj\"\n";
 
-	if (_version >= 9)
-		project << "\tTargetFrameworkVersion=\"131072\"\n";
+	project << "\tTargetFrameworkVersion=\"131072\"\n";
 
 	project << "\t>\n"
 	           "\t<Platforms>\n"
@@ -83,7 +79,7 @@ void VisualStudioProvider::createProjectFile(const std::string &name, const std:
 	// Check for project-specific warnings:
 	std::map< std::string, std::list<std::string> >::iterator warningsIterator = _projectWarnings.find(name);
 
-	if (setup.devTools || name == setup.projectName) {
+	if (setup.devTools || setup.tests || name == setup.projectName) {
 		std::string libraries;
 
 		for (StringList::const_iterator i = setup.libraries.begin(); i != setup.libraries.end(); ++i)
@@ -92,6 +88,7 @@ void VisualStudioProvider::createProjectFile(const std::string &name, const std:
 		// Win32
 		outputConfiguration(project, setup, libraries, "Debug", "Win32", "", true);
 		outputConfiguration(project, setup, libraries, "Analysis", "Win32", "", true);
+		outputConfiguration(project, setup, libraries, "LLVM", "Win32", "", true);
 		outputConfiguration(project, setup, libraries, "Release", "Win32", "", true);
 
 		// x64
@@ -100,9 +97,13 @@ void VisualStudioProvider::createProjectFile(const std::string &name, const std:
 		// libraries list created for IA-32. If that changes in the future, we need to adjust this part!
 		outputConfiguration(project, setup, libraries, "Debug", "x64", "64", false);
 		outputConfiguration(project, setup, libraries, "Analysis", "x64", "64", false);
+		outputConfiguration(project, setup, libraries, "LLVM", "Win32", "64", false);
 		outputConfiguration(project, setup, libraries, "Release", "x64", "64", false);
 
 	} else {
+		bool enableLanguageExtensions = find(_enableLanguageExtensions.begin(), _enableLanguageExtensions.end(), name) != _enableLanguageExtensions.end();
+		bool disableEditAndContinue = find(_disableEditAndContinue.begin(), _disableEditAndContinue.end(), name) != _disableEditAndContinue.end();
+
 		std::string warnings = "";
 		if (warningsIterator != _projectWarnings.end())
 			for (StringList::const_iterator i = warningsIterator->second.begin(); i != warningsIterator->second.end(); ++i)
@@ -110,17 +111,17 @@ void VisualStudioProvider::createProjectFile(const std::string &name, const std:
 
 		std::string toolConfig;
 		toolConfig  = (!warnings.empty() ? "DisableSpecificWarnings=\"" + warnings + "\"" : "");
-		toolConfig += (name == "tinsel" ? "DebugInformationFormat=\"3\" " : "");
-		toolConfig += (name == "sword25" ? "DisableLanguageExtensions=\"false\" " : "");
-		toolConfig += (name == "grim" ? "DisableLanguageExtensions=\"false\" " : "");
-		toolConfig += (name == "myst3" ? "DisableLanguageExtensions=\"false\" " : "");
+		toolConfig += (disableEditAndContinue   ? "DebugInformationFormat=\"3\" " : "");
+		toolConfig += (enableLanguageExtensions ? "DisableLanguageExtensions=\"false\" " : "");
 
 		// Win32
 		outputConfiguration(setup, project, toolConfig, "Debug", "Win32", "");
 		outputConfiguration(setup, project, toolConfig, "Analysis", "Win32", "");
+		outputConfiguration(setup, project, toolConfig, "LLVM", "Win32", "");
 		outputConfiguration(setup, project, toolConfig, "Release", "Win32", "");
 		outputConfiguration(setup, project, toolConfig, "Debug", "x64", "64");
 		outputConfiguration(setup, project, toolConfig, "Analysis", "x64", "64");
+		outputConfiguration(setup, project, toolConfig, "LLVM", "x64", "64");
 		outputConfiguration(setup, project, toolConfig, "Release", "x64", "64");
 	}
 
@@ -139,13 +140,18 @@ void VisualStudioProvider::createProjectFile(const std::string &name, const std:
 	else
 		addFilesToProject(moduleDir, project, includeList, excludeList, setup.filePrefix);
 
+	// Output auto-generated test runner
+	if (setup.tests) {
+		project << "\t\t<File RelativePath=\"test_runner.cpp\" />\n";
+	}
+
 	project << "\t</Files>\n"
 	           "</VisualStudioProject>\n";
 }
 
 void VisualStudioProvider::outputConfiguration(std::ostream &project, const BuildSetup &setup, const std::string &libraries, const std::string &config, const std::string &platform, const std::string &props, const bool isWin32) {
 	project << "\t\t<Configuration Name=\"" << config << "|" << platform << "\" ConfigurationType=\"1\" InheritedPropertySheets=\".\\" << setup.projectDescription << "_" << config << props << ".vsprops\">\n"
-	           "\t\t\t<Tool\tName=\"VCCLCompilerTool\" DisableLanguageExtensions=\"false\" />\n"
+	           "\t\t\t<Tool\tName=\"VCCLCompilerTool\" DisableLanguageExtensions=\"false\" DebugInformationFormat=\"3\" />\n"
 	           "\t\t\t<Tool\tName=\"VCLinkerTool\" OutputFile=\"$(OutDir)/" << setup.projectName << ".exe\"\n"
 	           "\t\t\t\tAdditionalDependencies=\"" << libraries << "\"\n"
 	           "\t\t\t/>\n";
@@ -160,7 +166,7 @@ void VisualStudioProvider::outputConfiguration(const BuildSetup &setup, std::ost
 }
 
 void VisualStudioProvider::outputBuildEvents(std::ostream &project, const BuildSetup &setup, const bool isWin32) {
-	if (!setup.devTools && setup.runBuildEvents) {
+	if (!setup.devTools && !setup.tests && setup.runBuildEvents) {
 		project << "\t\t\t<Tool\tName=\"VCPreBuildEventTool\"\n"
 		           "\t\t\t\tCommandLine=\"" << getPreBuildEvent() << "\"\n"
 		           "\t\t\t/>\n"
@@ -168,6 +174,17 @@ void VisualStudioProvider::outputBuildEvents(std::ostream &project, const BuildS
 		           "\t\t\t\tCommandLine=\"" << getPostBuildEvent(isWin32, setup.createInstaller) << "\"\n"
 		           "\t\t\t/>\n";
 	}
+
+	// Generate runner file before build for tests
+	if (setup.tests) {
+		project << "\t\t\t<Tool\tName=\"VCPreBuildEventTool\"\n"
+			"\t\t\t\tCommandLine=\"" << getTestPreBuildEvent(setup) << "\"\n"
+			"\t\t\t/>\n";
+
+		project << "\t\t\t<Tool\tName=\"VCPostBuildEventTool\"\n"
+			"\t\t\t\tCommandLine=\"$(TargetPath)\" IgnoreExitCode=\"true\"\n"
+			"\t\t\t/>\n";
+	}
 }
 
 void VisualStudioProvider::writeReferences(const BuildSetup &setup, std::ofstream &output) {
@@ -211,9 +228,9 @@ void VisualStudioProvider::outputGlobalPropFile(const BuildSetup &setup, std::of
 	              "\t\tName=\"VCCLCompilerTool\"\n"
 	              "\t\tDisableLanguageExtensions=\"" << (setup.devTools ? "false" : "true") << "\"\n"
 	              "\t\tDisableSpecificWarnings=\"" << warnings << "\"\n"
-	              "\t\tAdditionalIncludeDirectories=\"" << prefix << ";" << prefix << "\\engines;$(" << LIBS_DEFINE << ")\\include;$(TargetDir)\"\n"
+	              "\t\tAdditionalIncludeDirectories=\".\\;" << prefix << ";" << prefix << "\\engines;$(" << LIBS_DEFINE << ")\\include;$(" << LIBS_DEFINE << ")\\include\\SDL;" << (setup.tests ? prefix + "\\test\\cxxtest;" : "") << "$(TargetDir)\"\n"
 	              "\t\tPreprocessorDefinitions=\"" << definesList << "\"\n"
-	              "\t\tExceptionHandling=\"" << (setup.devTools ? "1" : "0") << "\"\n";
+	              "\t\tExceptionHandling=\"" << ((setup.devTools || setup.tests) ? "1" : "0") << "\"\n";
 
 #if NEEDS_RTTI
 	properties << "\t\tRuntimeTypeInfo=\"true\"\n";
@@ -234,7 +251,7 @@ void VisualStudioProvider::outputGlobalPropFile(const BuildSetup &setup, std::of
 	              "\t\tIgnoreDefaultLibraryNames=\"\"\n"
 	              "\t\tSubSystem=\"1\"\n";
 
-	if (!setup.devTools)
+	if (!setup.devTools && !setup.tests)
 		properties << "\t\tEntryPointSymbol=\"WinMainCRTStartup\"\n";
 
 	properties << "\t\tAdditionalLibraryDirectories=\"$(" << LIBS_DEFINE << ")\\lib\\" << ((bits == 32) ? "x86" : "x64") << "\"\n"
@@ -248,19 +265,18 @@ void VisualStudioProvider::outputGlobalPropFile(const BuildSetup &setup, std::of
 	properties.flush();
 }
 
-void VisualStudioProvider::createBuildProp(const BuildSetup &setup, bool isRelease, bool isWin32, bool enableAnalysis) {
-	const std::string outputType = (enableAnalysis ? "Analysis" : (isRelease ? "Release" : "Debug"));
+void VisualStudioProvider::createBuildProp(const BuildSetup &setup, bool isRelease, bool isWin32, std::string configuration) {
 	const std::string outputBitness = (isWin32 ? "32" : "64");
 
-	std::ofstream properties((setup.outputDir + '/' + setup.projectDescription + "_" + outputType + (isWin32 ? "" : "64") + getPropertiesExtension()).c_str());
+	std::ofstream properties((setup.outputDir + '/' + setup.projectDescription + "_" + configuration + (isWin32 ? "" : "64") + getPropertiesExtension()).c_str());
 	if (!properties)
-		error("Could not open \"" + setup.outputDir + '/' + setup.projectDescription + "_" + outputType + (isWin32 ? "" : "64") + getPropertiesExtension() + "\" for writing");
+		error("Could not open \"" + setup.outputDir + '/' + setup.projectDescription + "_" + configuration + (isWin32 ? "" : "64") + getPropertiesExtension() + "\" for writing");
 
 	properties << "<?xml version=\"1.0\" encoding=\"Windows-1252\"?>\n"
 	              "<VisualStudioPropertySheet\n"
 	              "\tProjectType=\"Visual C++\"\n"
 	              "\tVersion=\"8.00\"\n"
-	              "\tName=\"" << setup.projectDescription << "_" << outputType << outputBitness << "\"\n"
+	              "\tName=\"" << setup.projectDescription << "_" << configuration << outputBitness << "\"\n"
 	              "\tInheritedPropertySheets=\".\\" << setup.projectDescription << "_Global" << (isWin32 ? "" : "64") << ".vsprops\"\n"
 	              "\t>\n"
 	              "\t<Tool\n"
@@ -274,7 +290,7 @@ void VisualStudioProvider::createBuildProp(const BuildSetup &setup, bool isRelea
 		              "\t\tBufferSecurityCheck=\"false\"\n"
 		              "\t\tDebugInformationFormat=\"0\"\n"
 		              "\t\tRuntimeLibrary=\"0\"\n"
-		              "\t\tAdditionalOption=\"" << (enableAnalysis ? "/analyze" : "") << "\"\n"
+		              "\t\tAdditionalOption=\"" << (configuration == "Analysis" ? "/analyze" : "") << "\"\n"
 		              "\t/>\n"
 		              "\t<Tool\n"
 		              "\t\tName=\"VCLinkerTool\"\n"
@@ -290,7 +306,7 @@ void VisualStudioProvider::createBuildProp(const BuildSetup &setup, bool isRelea
 		              "\t\tEnableFunctionLevelLinking=\"true\"\n"
 		              "\t\tWarnAsError=\"false\"\n"
 		              "\t\tDebugInformationFormat=\"" << (isWin32 ? "4" : "3") << "\"\n" // For x64 format "4" (Edit and continue) is not supported, thus we default to "3"
-		              "\t\tAdditionalOption=\"" << (enableAnalysis ? "/analyze" : "") << "\"\n"
+		              "\t\tAdditionalOption=\"" << (configuration == "Analysis" ? "/analyze" : "") << "\"\n"
 		              "\t/>\n"
 		              "\t<Tool\n"
 		              "\t\tName=\"VCLinkerTool\"\n"
@@ -337,9 +353,9 @@ void VisualStudioProvider::writeFileListToProject(const FileNode &dir, std::ofst
 					            << indentString << "\t<FileConfiguration Name=\"Debug|Win32\">\n"
 					            << toolLine
 					            << indentString << "\t</FileConfiguration>\n"
-								<< indentString << "\t<FileConfiguration Name=\"Analysis|Win32\">\n"
-								<< toolLine
-								<< indentString << "\t</FileConfiguration>\n"
+					            << indentString << "\t<FileConfiguration Name=\"Analysis|Win32\">\n"
+					            << toolLine
+					            << indentString << "\t</FileConfiguration>\n"
 					            << indentString << "\t<FileConfiguration Name=\"Release|Win32\">\n"
 					            << toolLine
 					            << indentString << "\t</FileConfiguration>\n"
@@ -352,18 +368,18 @@ void VisualStudioProvider::writeFileListToProject(const FileNode &dir, std::ofst
 						            << indentString << "\t<FileConfiguration Name=\"Debug|Win32\">\n"
 						            << toolLine
 						            << indentString << "\t</FileConfiguration>\n"
-									<< indentString << "\t<FileConfiguration Name=\"Analysis|Win32\">\n"
-									<< toolLine
-									<< indentString << "\t</FileConfiguration>\n"
+						            << indentString << "\t<FileConfiguration Name=\"Analysis|Win32\">\n"
+						            << toolLine
+						            << indentString << "\t</FileConfiguration>\n"
 						            << indentString << "\t<FileConfiguration Name=\"Release|Win32\">\n"
 						            << toolLine
 						            << indentString << "\t</FileConfiguration>\n"
-									<< indentString << "\t<FileConfiguration Name=\"Debug|x64\">\n"
-									<< toolLine
-									<< indentString << "\t</FileConfiguration>\n"
-									<< indentString << "\t<FileConfiguration Name=\"Analysis|x64\">\n"
-									<< toolLine
-									<< indentString << "\t</FileConfiguration>\n"
+						            << indentString << "\t<FileConfiguration Name=\"Debug|x64\">\n"
+						            << toolLine
+						            << indentString << "\t</FileConfiguration>\n"
+						            << indentString << "\t<FileConfiguration Name=\"Analysis|x64\">\n"
+						            << toolLine
+						            << indentString << "\t</FileConfiguration>\n"
 						            << indentString << "\t<FileConfiguration Name=\"Release|x64\">\n"
 						            << toolLine
 						            << indentString << "\t</FileConfiguration>\n"
