@@ -26,15 +26,6 @@
 #include <fstream>
 #include <algorithm>
 
-#if defined(_WIN32) || defined(WIN32)
-#include <windows.h>
-#else
-#include <sys/param.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#include <errno.h>
-#endif
-
 namespace CreateProjectTool {
 
 #define DEBUG_XCODE_HASH 0
@@ -88,27 +79,7 @@ XCodeProvider::XCodeProvider(StringList &global_warnings, std::map<std::string,
 void XCodeProvider::createWorkspace(const BuildSetup &setup) {
 	// Create project folder
 	std::string workspace = setup.outputDir + '/' + PROJECT_NAME ".xcodeproj";
-
-#if defined(_WIN32) || defined(WIN32)
-	if (!CreateDirectory(workspace.c_str(), NULL))
-		if (GetLastError() != ERROR_ALREADY_EXISTS)
-			error("Could not create folder \"" + setup.outputDir + '/' + PROJECT_NAME ".xcodeproj\"");
-#else
-	if (mkdir(workspace.c_str(), 0777) == -1) {
-		if (errno == EEXIST) {
-			// Try to open as a folder (might be a file / symbolic link)
-			DIR *dirp = opendir(workspace.c_str());
-			if (dirp == NULL) {
-				error("Could not create folder \"" + setup.outputDir + '/' + PROJECT_NAME ".xcodeproj\"");
-			} else {
-				// The folder exists, just close the stream and return
-				closedir(dirp);
-			}
-		} else {
-			error("Could not create folder \"" + setup.outputDir + '/' + PROJECT_NAME ".xcodeproj\"");
-		}
-	}
-#endif
+	createDirectory(workspace);
 
 	// Setup global objects
 	setupDefines(setup);
@@ -202,18 +173,38 @@ void XCodeProvider::writeFileListToProject(const FileNode &dir, std::ofstream &p
 
 	// Init root group
 	_groups.comment = "PBXGroup";
-	Object *group = new Object(this, "PBXGroup", "PBXGroup", "PBXGroup", "", "");
 
-	//Property children;
-	//children.flags = SettingsAsList;
-	//group->properties["children"] = children;
-	group->addProperty("children", "", "", SettingsNoValue|SettingsAsList);
+	// Create group
+	std::string name = getLastPathComponent(dir.name);
+	Object *group = new Object(this, "PBXGroup_" + name , "PBXGroup", "PBXGroup", "", name);
 
+	// List of children
+	Property children;
+	children.hasOrder = true;
+	children.flags = SettingsAsList;
+
+	group->addProperty("name", name, "", SettingsNoValue|SettingsQuoteVariable);
 	group->addProperty("sourceTree", "<group>", "", SettingsNoValue|SettingsQuoteVariable);
 
-	_groups.add(group);
+	int order = 0;
+	for (FileNode::NodeList::const_iterator i = dir.children.begin(); i != dir.children.end(); ++i) {
+		const FileNode *node = *i;
 
-	// TODO Add files
+		std::string id = "FileReference_" + node->name;
+		FileProperty property = FileProperty(node->name, node->name, node->name, "<group>");
+
+		ADD_SETTING_ORDER_NOVALUE(children, getHash(id), node->name, order++);
+		ADD_BUILD_FILE(id, node->name, node->name + " in Sources");
+		ADD_FILE_REFERENCE(node->name, property);
+
+		// Process child nodes
+		if (!node->children.empty())
+			writeFileListToProject(*node, projectFile, indentation + 1, duplicate, objPrefix + node->name + '_', filePrefix + node->name + '/');
+	}
+
+	group->properties["children"] = children;
+
+	_groups.add(group);
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -717,6 +708,7 @@ void XCodeProvider::setupBuildConfiguration() {
 	ADD_SETTING_QUOTE(scummvmSimulator_Debug, "FRAMEWORK_SEARCH_PATHS", "$(inherited)");
 	ADD_SETTING_LIST(scummvmSimulator_Debug, "GCC_PREPROCESSOR_DEFINITIONS", scummvm_defines, SettingsNoQuote|SettingsAsList, 5);
 	ADD_SETTING(scummvmSimulator_Debug, "SDKROOT", "iphonesimulator3.2");
+	ADD_SETTING_QUOTE(scummvmSimulator_Debug, "VALID_ARCHS", "i386 x86_64");
 	REMOVE_SETTING(scummvmSimulator_Debug, "TARGETED_DEVICE_FAMILY");
 
 	scummvmSimulator_Debug_Object->addProperty("name", "Debug", "", SettingsNoValue);
@@ -726,6 +718,7 @@ void XCodeProvider::setupBuildConfiguration() {
 	Object *scummvmSimulator_Release_Object = new Object(this, "XCBuildConfiguration_" PROJECT_DESCRIPTION "-Simulator_Release", _targets[2] /* ScummVM-Simulator */, "XCBuildConfiguration", "PBXNativeTarget", "Release");
 	Property scummvmSimulator_Release(scummvmSimulator_Debug);
 	ADD_SETTING(scummvmSimulator_Release, "COPY_PHASE_STRIP", "YES");
+	ADD_SETTING(scummvmSimulator_Release, "GCC_OPTIMIZATION_LEVEL", "3");
 	REMOVE_SETTING(scummvmSimulator_Release, "GCC_DYNAMIC_NO_PIC");
 	ADD_SETTING(scummvmSimulator_Release, "WRAPPER_EXTENSION", "app");
 
@@ -871,7 +864,9 @@ std::string XCodeProvider::writeProperty(const std::string &variable, Property &
 std::string XCodeProvider::writeSetting(const std::string &variable, std::string value, std::string comment, int flags, int indent) const {
 	return writeSetting(variable, Setting(value, comment, flags, indent));
 }
-// Heavily modified (not in a good way) function, imported from QMake XCode project generator (licensed under the QT license)
+
+// Heavily modified (not in a good way) function, imported from the QMake
+// XCode project generator pbuilder_pbx.cpp, writeSettings() (under LGPL 2.1)
 std::string XCodeProvider::writeSetting(const std::string &variable, const Setting &setting) const {
 	std::string output;
 	const std::string quote = (setting.flags & SettingsNoQuote) ? "" : "\"";
@@ -893,7 +888,7 @@ std::string XCodeProvider::writeSetting(const std::string &variable, const Setti
 		for (unsigned int i = 0, count = 0; i < setting.entries.size(); ++i) {
 
 			std::string value = setting.entries.at(i).value;
-			if(!value.empty()) {
+			if (!value.empty()) {
 				if (count++ > 0)
 					output += "," + newline;
 
