@@ -48,7 +48,7 @@ if ($mode eq "") {
 $Text::Wrap::unexpand = 0;
 if ($mode eq "TEXT") {
 	$Text::Wrap::columns = 78;
-	$max_name_width = 21; # The maximal width of a name.
+	$max_name_width = 28; # The maximal width of a name.
 } elsif ($mode eq "CPP") {
 	$Text::Wrap::columns = 48;	# Approx.
 }
@@ -60,26 +60,34 @@ sub html_entities_to_ascii {
 	# For now we hardcode these mappings
 	# &aacute;  -> a
 	# &eacute;  -> e
+	# &iacute;  -> i
 	# &igrave;  -> i
 	# &oacute;  -> o
 	# &oslash;  -> o
 	# &ouml;    -> o / oe
 	# &auml;    -> a
+	# &euml;    -> e
 	# &uuml;    -> ue
 	# &aring;   -> aa
 	# &amp;     -> &
 	# &#322;    -> l
+	# &#347;    -> s
 	# &Scaron;  -> S
+	# &ntilde;  -> n
 	$text =~ s/&aacute;/a/g;
 	$text =~ s/&eacute;/e/g;
+	$text =~ s/&iacute;/i/g;
 	$text =~ s/&igrave;/i/g;
 	$text =~ s/&oacute;/o/g;
 	$text =~ s/&oslash;/o/g;
 	$text =~ s/&#322;/l/g;
+	$text =~ s/&#347;/s/g;
 	$text =~ s/&Scaron;/S/g;
 	$text =~ s/&aring;/aa/g;
+	$text =~ s/&ntilde;/n/g;
 
 	$text =~ s/&auml;/a/g;
+	$text =~ s/&euml;/e/g;
 	$text =~ s/&uuml;/ue/g;
 	# HACK: Torbj*o*rn but G*oe*ffringmann and R*oe*ver and J*oe*rg
 	$text =~ s/Torbj&ouml;rn/Torbjorn/g;
@@ -97,14 +105,18 @@ sub html_entities_to_cpp {
 	# The numerical values are octal!
 	$text =~ s/&aacute;/\\341/g;
 	$text =~ s/&eacute;/\\351/g;
+	$text =~ s/&iacute;/\\355/g;
 	$text =~ s/&igrave;/\\354/g;
 	$text =~ s/&oacute;/\\363/g;
 	$text =~ s/&oslash;/\\370/g;
 	$text =~ s/&#322;/l/g;
+	$text =~ s/&#347;/s/g;
 	$text =~ s/&Scaron;/S/g;
 	$text =~ s/&aring;/\\345/g;
+	$text =~ s/&ntilde;/\\361/g;
 
 	$text =~ s/&auml;/\\344/g;
+	$text =~ s/&euml;/\\353/g;
 	$text =~ s/&ouml;/\\366/g;
 	$text =~ s/&uuml;/\\374/g;
 
@@ -120,6 +132,7 @@ sub html_entities_to_rtf {
 
 	$text =~ s/&aacute;/\\'87/g;
 	$text =~ s/&eacute;/\\'8e/g;
+	$text =~ s/&iacute;/\\'92/g;
 	$text =~ s/&igrave;/\\'93/g;
 	$text =~ s/&oacute;/\\'97/g;
 	$text =~ s/&oslash;/\\'bf/g;
@@ -129,7 +142,10 @@ sub html_entities_to_rtf {
 	$text =~ s/&Scaron;/\\uc0\\u540 /g;
 
 	# Back to hex numbers
+	$text =~ s/&ntilde;/\\'96/g;
+
 	$text =~ s/&auml;/\\'8a/g;
+	$text =~ s/&euml;/\\'eb/g;
 	$text =~ s/&ouml;/\\'9a/g;
 	$text =~ s/&uuml;/\\'9f/g;
 
@@ -144,15 +160,18 @@ sub html_entities_to_tex {
 
 	$text =~ s/&aacute;/\\'a/g;
 	$text =~ s/&eacute;/\\'e/g;
+	$text =~ s/&iacute;/\\'i/g;
 	$text =~ s/&igrave;/\\`\\i/g;
 	$text =~ s/&oacute;/\\'o/g;
 	$text =~ s/&oslash;/{\\o}/g;
 	$text =~ s/&aring;/\\aa /g;
 	$text =~ s/&#322;/{\\l}/g;
 	$text =~ s/&Scaron;/{\\v S}/g;
+	$text =~ s/&ntilde;/\\˜n/g;
 
 	$text =~ s/&auml;/\\"a/g;
 	$text =~ s/&ouml;/\\"o/g;
+	$text =~ s/&euml;/\\"e/g;
 	$text =~ s/&uuml;/\\"u/g;
 
 	$text =~ s/&amp;/\\&/g;
@@ -264,14 +283,22 @@ sub begin_section {
 		print '\f1\b0\fs24 \cf0 \\' . "\n";
 	} elsif ($mode eq "CPP") {
 		if ($section_level eq 0) {
-		  # TODO: Would be nice to have a 'fat' or 'large' mode for
-		  # headlines...
-		  $title = html_entities_to_cpp($title);
-		  print '"C1""'.$title.'",' . "\n";
-		  print '"",' . "\n";
+			# TODO: Would be nice to have a 'fat' or 'large' mode for
+			# headlines...
+			my $ascii_title = html_entities_to_ascii($title);
+			$title = html_entities_to_cpp($title);
+			if ($ascii_title ne $title) {	
+				print '"A1""'.$ascii_title.'",' . "\n";
+			}
+			print '"C1""'.$title.'",' . "\n";
+			print '"",' . "\n";
 		} else {
-		  $title = html_entities_to_cpp($title);
-		  print '"C1""'.$title.'",' . "\n";
+			my $ascii_title = html_entities_to_ascii($title);
+			$title = html_entities_to_cpp($title);
+			if ($ascii_title ne $title) {	
+				print '"A1""'.$ascii_title.'",' . "\n";
+			}
+			print '"C1""'.$title.'",' . "\n";
 		}
 	} elsif ($mode eq "XML-DOC") {
 		print "  <row><entry namest='start' nameend='job'>";
@@ -356,6 +383,9 @@ sub add_person {
 		my $min_name_width = length $desc > 0 ? $max_name_width : 0;
 		$name = $nick if $name eq "";
 		$name = html_entities_to_ascii($name);
+		if (length $name > $max_name_width) {
+			print STDERR "Warning: max_name_width is too small (" . $max_name_width . " < " . (length $name) . " for \"" . $name. "\")\n";
+		}
 		$desc = html_entities_to_ascii($desc);
 
 		$tab = " " x ($section_level * 2 + 1);
@@ -386,13 +416,21 @@ sub add_person {
 		}
 	} elsif ($mode eq "CPP") {
 		$name = $nick if $name eq "";
+		my $ascii_name = html_entities_to_ascii($name);
 		$name = html_entities_to_cpp($name);
 
+		if ($ascii_name ne $name) {
+			print '"A0""'.$ascii_name.'",' . "\n";
+		}
 		print '"C0""'.$name.'",' . "\n";
 
 		# Print desc wrapped
 		if (length $desc > 0) {
+			my $ascii_desc = html_entities_to_ascii($desc);
 			$desc = html_entities_to_cpp($desc);
+			if ($ascii_desc ne $desc) {	
+				print '"A2""'.$ascii_desc.'",' . "\n";
+			}
 			print '"C2""'.$desc.'",' . "\n";
 		}
 	} elsif ($mode eq "XML-DOC") {
@@ -454,11 +492,14 @@ begin_credits("Credits");
 
 	begin_section("Teams");
 	    begin_section("Grim Team");
-		add_person("James Brown", "ender", "Core developer (retired)");
-		add_person("Giulio Camuffo", "giucam", "Core developer");
+		add_person("James Brown", "ender", "Grim developer (retired)");
+		add_person("Giulio Camuffo", "giucam", "Grim developer");
+		add_person("Dries Harnie", "Botje", "EMI, Android, OpenGL ES developer");
 		add_person("Pawe&#322; Ko&#322;odziejski", "aquadran", "Core developer (suspended)");
-		add_person("Einar Johan T. S&oslash;m&aring;en", "somaen", "EMI support, various code fixes and improvements");
-		add_person("Joel Teichroeb ", "klusark", "EMI support, various engine fixes and improvements");
+		add_person("Christian Krause", "chkr", "EMI developer");
+		add_person("Einar Johan T. S&oslash;m&aring;en", "somaen", "Grim, EMI developer");
+		add_person("Joel Teichroeb ", "klusark", "EMI, Android developer");
+		add_person("Joni V&auml;h&auml;m&auml;ki", "Akz", "EMI developer");
 	    end_section();
 
 	    begin_section("Myst 3 Team");
@@ -479,32 +520,41 @@ begin_credits("Credits");
 		add_person("Marcus Comstedt", "marcus_c", "Initial Dreamcast port");
 		add_person("Andrea Corna", "Yak Bizzarro", "Patcher module, various engine improvements");
 		add_person("Jonathan Gray", "khalek", "Various code fixes");
+		add_person("Tobias Gunkel", "tobigun", "Initial Android port, few engines fixes");
+		add_person("Azamat H. Hackimov", "winterheart", "Configure fix");
 		add_person("Vincent Hamm", "yazoo", "Various engine code fixes and improvements");
 		add_person("Sven Hesse", "DrMcCoy", "Various compilation fixes");
 		add_person("Matthew Hoops", "clone2727", "Some engine improvements, some work on EMI support");
 		add_person("Erich Hoover", "Compholio", "x86-64 fixes, various code fixes and improvements");
-		add_person("Max Horn", "fingolfin", "Minor code fixes");
+		add_person("Max Horn", "fingolfin", "Few code fixes");
 		add_person("Travis Howell", "Kirben", "Various code fixes, Windows port");
+		add_person("Joseph Jezak", "JoseJX", "A lot of engine improvements and fixes");
 		add_person("Guillem Jover", "guillemj", "Few code improvements");
 		add_person("Filippos Karapetis", "bluegr", "Compilation fixes");
+		add_person("Ingo van Lil", "inguin", "Various fixes and improvements for EMI");
+		add_person("Vincent Pelletier", "vpelletier", "Various engine improvements and code fixes");
 		add_person("Joost Peters", "joostp", "Various code fixes");
-		add_person("George Macon", "gmacon", "Compilation fixes");
+		add_person("George Macon", "gmacon", "Few fixes");
 		add_person("Josh Matthews", "jdm", "Few fixes to engine");
 		add_person("Matthieu Milan", "usineur", "Various engine improvements");
+		add_person("Gregory Montoir", "cyx", "Few fixes to engine");
+		add_person("Stefano Musumeci", "subr3v", "TinyGL backend and engine driver improvements");
 		add_person("Christian Neumair", "mannythegnome", "Various optimisation patches");
-		add_person("Vincent Pelletier", "vpelletier", "Various engine improvements and code fixes");
-		add_person("Daniel Schepler", "", "Initial engine codebase, LUA support");
+		add_person("Daniel Schepler", "", "Initial grim engine contributor, LUA support");
+		add_person("Dmitry Smirnov", "onlyjob", "Minor spelling corrections");
 		add_person("Yaron Tausky", "yaront", "Fixes to subtitles");
 		add_person("Julien Templier", "Littleboy", "create_project tool");
 		add_person("Pino Toscano", "pinotree", "Debian GNU/Linux package files");
 		add_person("Lionel Ulmer", "bbrox", "OpenGL optimisations");
-		add_person("Joni V&auml;h&auml;m&auml;ki", "Akz", "A lot of engine improvements and fixes");
+		add_person("", "cmayer0087", "Various engine code fixes");
 		add_person("", "JenniBee", "Compilation fixes");
-		add_person("", "tobigun", "Few fixes to engine");
+		add_person("", "karjonas", "Various engine code fixes");
+		add_person("", "sietschie", "Few fixes to engine");
 	    end_section();
 
 	    begin_section("Myst 3");
 		add_person("Matthew Hoops", "clone2727", "Various engine improvements and code fixes");
+		add_person("Stefano Musumeci", "subr3v", "TinyGL engine driver support");
 	    end_section();
 	end_section();
 
