@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -22,7 +22,6 @@
 
 #include "common/debug.h"
 #include "common/util.h"
-#include "common/hash-str.h"
 #include "common/file.h"
 #include "common/macresman.h"
 #include "common/md5.h"
@@ -30,7 +29,7 @@
 #include "common/system.h"
 #include "common/textconsole.h"
 #include "common/translation.h"
-
+#include "gui/EventRecorder.h"
 #include "engines/advancedDetector.h"
 #include "engines/obsolete.h"
 
@@ -79,7 +78,7 @@ static Common::String generatePreferredTarget(const Common::String &id, const AD
 		res = res + "-cd";
 	}
 
-	if (desc->platform != Common::kPlatformPC && desc->platform != Common::kPlatformUnknown) {
+	if (desc->platform != Common::kPlatformDOS && desc->platform != Common::kPlatformUnknown && !(desc->flags & ADGF_DROPPLATFORM)) {
 		res = res + "-" + getPlatformAbbrev(desc->platform);
 	}
 
@@ -123,7 +122,7 @@ bool cleanupPirated(ADGameDescList &matched) {
 
 		// We ruled out all variants and now have nothing
 		if (matched.empty()) {
-			warning("Illegitimate game copy detected. We give no support in such cases %d", matched.size());
+			warning("Illegitimate game copy detected. We provide no support in such cases");
 			return true;
 		}
 	}
@@ -302,20 +301,14 @@ Common::Error AdvancedMetaEngine::createInstance(OSystem *syst, Engine **engine)
 		return Common::kUserCanceled;
 
 	debug(2, "Running %s", gameDescriptor.description().c_str());
+	initSubSystems(agdDesc);
 	if (!createInstance(syst, engine, agdDesc))
 		return Common::kNoGameDataFoundError;
 	else
 		return Common::kNoError;
 }
 
-struct SizeMD5 {
-	int size;
-	Common::String md5;
-};
-
-typedef Common::HashMap<Common::String, SizeMD5, Common::IgnoreCase_Hash, Common::IgnoreCase_EqualTo> SizeMD5Map;
-
-static void reportUnknown(const Common::FSNode &path, const SizeMD5Map &filesSizeMD5) {
+void AdvancedMetaEngine::reportUnknown(const Common::FSNode &path, const ADFilePropertiesMap &filesProps) const {
 	// TODO: This message should be cleaned up / made more specific.
 	// For example, we should specify at least which engine triggered this.
 	//
@@ -327,7 +320,7 @@ static void reportUnknown(const Common::FSNode &path, const SizeMD5Map &filesSiz
 	report += _("of the game you tried to add and its version/language/etc.:");
 	report += "\n";
 
-	for (SizeMD5Map::const_iterator file = filesSizeMD5.begin(); file != filesSizeMD5.end(); ++file)
+	for (ADFilePropertiesMap::const_iterator file = filesProps.begin(); file != filesProps.end(); ++file)
 		report += Common::String::format("  {\"%s\", 0, \"%s\", %d},\n", file->_key.c_str(), file->_value.md5.c_str(), file->_value.size);
 
 	report += "\n";
@@ -375,8 +368,36 @@ void AdvancedMetaEngine::composeFileHashMap(FileMap &allFiles, const Common::FSL
 	}
 }
 
+bool AdvancedMetaEngine::getFileProperties(const Common::FSNode &parent, const FileMap &allFiles, const ADGameDescription &game, const Common::String fname, ADFileProperties &fileProps) const {
+	// FIXME/TODO: We don't handle the case that a file is listed as a regular
+	// file and as one with resource fork.
+
+	if (game.flags & ADGF_MACRESFORK) {
+		Common::MacResManager macResMan;
+
+		if (!macResMan.open(parent, fname))
+			return false;
+
+		fileProps.md5 = macResMan.computeResForkMD5AsString(_md5Bytes);
+		fileProps.size = macResMan.getResForkDataSize();
+		return true;
+	}
+
+	if (!allFiles.contains(fname))
+		return false;
+
+	Common::File testFile;
+
+	if (!testFile.open(allFiles[fname]))
+		return false;
+
+	fileProps.size = (int32)testFile.size();
+	fileProps.md5 = Common::computeStreamMD5AsString(testFile, _md5Bytes);
+	return true;
+}
+
 ADGameDescList AdvancedMetaEngine::detectGame(const Common::FSNode &parent, const FileMap &allFiles, Common::Language language, Common::Platform platform, const Common::String &extra) const {
-	SizeMD5Map filesSizeMD5;
+	ADFilePropertiesMap filesProps;
 
 	const ADGameFileDescription *fileDesc;
 	const ADGameDescription *g;
@@ -391,39 +412,14 @@ ADGameDescList AdvancedMetaEngine::detectGame(const Common::FSNode &parent, cons
 
 		for (fileDesc = g->filesDescriptions; fileDesc->fileName; fileDesc++) {
 			Common::String fname = fileDesc->fileName;
-			SizeMD5 tmp;
+			ADFileProperties tmp;
 
-			if (filesSizeMD5.contains(fname))
+			if (filesProps.contains(fname))
 				continue;
 
-			// FIXME/TODO: We don't handle the case that a file is listed as a regular
-			// file and as one with resource fork.
-
-			if (g->flags & ADGF_MACRESFORK) {
-				Common::MacResManager macResMan;
-
-				if (macResMan.open(parent, fname)) {
-					tmp.md5 = macResMan.computeResForkMD5AsString(_md5Bytes);
-					tmp.size = macResMan.getResForkDataSize();
-					debug(3, "> '%s': '%s'", fname.c_str(), tmp.md5.c_str());
-					filesSizeMD5[fname] = tmp;
-				}
-			} else {
-				if (allFiles.contains(fname)) {
-					debug(3, "+ %s", fname.c_str());
-
-					Common::File testFile;
-
-					if (testFile.open(allFiles[fname])) {
-						tmp.size = (int32)testFile.size();
-						tmp.md5 = Common::computeStreamMD5AsString(testFile, _md5Bytes);
-					} else {
-						tmp.size = -1;
-					}
-
-					debug(3, "> '%s': '%s'", fname.c_str(), tmp.md5.c_str());
-					filesSizeMD5[fname] = tmp;
-				}
+			if (getFileProperties(parent, allFiles, *g, fname, tmp)) {
+				debug(3, "> '%s': '%s'", fname.c_str(), tmp.md5.c_str());
+				filesProps[fname] = tmp;
 			}
 		}
 	}
@@ -456,19 +452,19 @@ ADGameDescList AdvancedMetaEngine::detectGame(const Common::FSNode &parent, cons
 		for (fileDesc = g->filesDescriptions; fileDesc->fileName; fileDesc++) {
 			Common::String tstr = fileDesc->fileName;
 
-			if (!filesSizeMD5.contains(tstr)) {
+			if (!filesProps.contains(tstr)) {
 				fileMissing = true;
 				allFilesPresent = false;
 				break;
 			}
 
-			if (fileDesc->md5 != NULL && fileDesc->md5 != filesSizeMD5[tstr].md5) {
-				debug(3, "MD5 Mismatch. Skipping (%s) (%s)", fileDesc->md5, filesSizeMD5[tstr].md5.c_str());
+			if (fileDesc->md5 != NULL && fileDesc->md5 != filesProps[tstr].md5) {
+				debug(3, "MD5 Mismatch. Skipping (%s) (%s)", fileDesc->md5, filesProps[tstr].md5.c_str());
 				fileMissing = true;
 				break;
 			}
 
-			if (fileDesc->fileSize != -1 && fileDesc->fileSize != filesSizeMD5[tstr].size) {
+			if (fileDesc->fileSize != -1 && fileDesc->fileSize != filesProps[tstr].size) {
 				debug(3, "Size Mismatch. Skipping");
 				fileMissing = true;
 				break;
@@ -514,8 +510,8 @@ ADGameDescList AdvancedMetaEngine::detectGame(const Common::FSNode &parent, cons
 
 	// We didn't find a match
 	if (matched.empty()) {
-		if (!filesSizeMD5.empty() && gotAnyMatchesWithAllFiles) {
-			reportUnknown(parent, filesSizeMD5);
+		if (!filesProps.empty() && gotAnyMatchesWithAllFiles) {
+			reportUnknown(parent, filesProps);
 		}
 
 		// Filename based fallback
@@ -524,7 +520,7 @@ ADGameDescList AdvancedMetaEngine::detectGame(const Common::FSNode &parent, cons
 	return matched;
 }
 
-const ADGameDescription *AdvancedMetaEngine::detectGameFilebased(const FileMap &allFiles, const ADFileBasedFallback *fileBasedFallback) const {
+const ADGameDescription *AdvancedMetaEngine::detectGameFilebased(const FileMap &allFiles, const Common::FSList &fslist, const ADFileBasedFallback *fileBasedFallback, ADFilePropertiesMap *filesProps) const {
 	const ADFileBasedFallback *ptr;
 	const char* const* filenames;
 
@@ -554,6 +550,16 @@ const ADGameDescription *AdvancedMetaEngine::detectGameFilebased(const FileMap &
 				maxNumMatchedFiles = numMatchedFiles;
 
 				debug(4, "and overridden");
+
+				if (filesProps) {
+					for (filenames = ptr->filenames; *filenames; ++filenames) {
+						ADFileProperties tmp;
+
+						if (getFileProperties(fslist.begin()->getParent(), allFiles, *agdesc, *filenames, tmp))
+							(*filesProps)[*filenames] = tmp;
+					}
+				}
+
 			}
 		}
 	}
@@ -601,3 +607,11 @@ AdvancedMetaEngine::AdvancedMetaEngine(const void *descs, uint descItemSize, con
 	_maxScanDepth = 1;
 	_directoryGlobs = NULL;
 }
+
+void AdvancedMetaEngine::initSubSystems(const ADGameDescription *gameDesc) const {
+#ifdef ENABLE_EVENTRECORDER
+	if (gameDesc) {
+		g_eventRec.processGameDescription(gameDesc);
+	}
+#endif
+}
