@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -106,16 +106,15 @@ namespace Grim {
 // marked OBJSTATE_OVERLAY.  So the BitmapComponent just needs to pass
 // along setKey requests to the actual bitmap object.
 
-Costume::Costume(const Common::String &fname, Costume *prevCost) :
-		Object(), _head(new Head()), _chores(NULL) {
-
-	_fname = fname;
+Costume::Costume(const Common::String &fname, Actor *owner, Costume *prevCost) :
+		Object(), _head(nullptr), _chores(nullptr), _components(nullptr),
+		_numComponents(0), _numChores(0), _fname(fname), _owner(owner) {
 	_lookAtRate = 200;
 	_prevCostume = prevCost;
 }
 
 void Costume::load(Common::SeekableReadStream *data) {
-	TextSplitter ts(data);
+	TextSplitter ts(_fname, data);
 	ts.expectString("costume v0.1");
 	ts.expectString("section tags");
 	int numTags;
@@ -140,7 +139,7 @@ void Costume::load(Common::SeekableReadStream *data) {
 	for (int i = 0; i < _numComponents; i++) {
 		int id, tagID, hash, parentID, namePos;
 		const char *line = ts.getCurrentLine();
-		Component *prevComponent = NULL;
+		Component *prevComponent = nullptr;
 
 		if (sscanf(line, " %d %d %d %d %n", &id, &tagID, &hash, &parentID, &namePos) < 4)
 			error("Bad component specification line: `%s'", line);
@@ -158,7 +157,7 @@ void Costume::load(Common::SeekableReadStream *data) {
 				prevComponent = _prevCostume->_components[0];
 				// Make sure that the component is valid
 				if (!prevComponent->isComponentType('M','M','D','L'))
-					prevComponent = NULL;
+					prevComponent = nullptr;
 			} else if (id > 0) {
 				// Use the MainModelComponent of this costume as prevComponent,
 				// so that the component can use its colormap.
@@ -166,7 +165,7 @@ void Costume::load(Common::SeekableReadStream *data) {
 			}
 		}
 		// Actually load the appropriate component
-		_components[id] = loadComponent(tags[tagID], parentID < 0 ? NULL : _components[parentID], parentID, line + namePos, prevComponent);
+		_components[id] = loadComponent(tags[tagID], parentID < 0 ? nullptr : _components[parentID], parentID, line + namePos, prevComponent);
 		_components[id]->setCostume(this);
 	}
 
@@ -184,10 +183,7 @@ void Costume::load(Common::SeekableReadStream *data) {
 		int id, length, tracks;
 		char name[32];
 		ts.scanString(" %d %d %d %32s", 4, &id, &length, &tracks, name);
-		_chores[id] = new Chore();
-		_chores[id]->_length = length;
-		_chores[id]->_numTracks = tracks;
-		memcpy(_chores[id]->_name, name, 32);
+		_chores[id] = new Chore(name, i, this, length, tracks);
 		Debug::debug(Debug::Chores, "Loaded chore: %s\n", name);
 	}
 
@@ -195,25 +191,23 @@ void Costume::load(Common::SeekableReadStream *data) {
 	for (int i = 0; i < _numChores; i++) {
 		int which;
 		ts.scanString("chore %d", 1, &which);
-		_chores[which]->load(i, this, ts);
+		_chores[which]->load(ts);
 	}
+
+	_head = new Head();
 }
 
 Costume::~Costume() {
-	if (_chores) {
-		stopChores();
-		for (int i = _numComponents - 1; i >= 0; i--) {
-			// The "Sprite" component can be NULL
-			if (_components[i])
-				delete _components[i];
-		}
-		delete[] _components;
-
-		for (int i = 0; i < _numChores; ++i) {
-			delete _chores[i];
-		}
-		delete[] _chores;
+	stopChores();
+	for (int i = _numComponents - 1; i >= 0; i--) {
+		delete _components[i];
 	}
+	delete[] _components;
+
+	for (int i = 0; i < _numChores; ++i) {
+		delete _chores[i];
+	}
+	delete[] _chores;
 	delete _head;
 }
 
@@ -244,17 +238,17 @@ Component *Costume::loadComponent (tag32 tag, Component *parent, int parentID, c
 	char t[4];
 	memcpy(t, &tag, sizeof(tag32));
 	warning("loadComponent: Unknown tag '%c%c%c%c', name '%s'", t[0], t[1], t[2], t[3], name);
-	return NULL;
+	return nullptr;
 }
 
 ModelComponent *Costume::getMainModelComponent() const {
 	for (int i = 0; i < _numComponents; i++) {
 		// Needs to handle Main Models (pigeons) and normal Models
 		// (when Manny climbs the rope)
-		if (_components[i]->isComponentType('M','M','D','L'))
+		if (_components[i] && _components[i]->isComponentType('M','M','D','L'))
 			return static_cast<ModelComponent *>(_components[i]);
 	}
-	return NULL;
+	return nullptr;
 }
 
 ModelNode *Costume::getModelNodes() {
@@ -262,7 +256,7 @@ ModelNode *Costume::getModelNodes() {
 	if (comp) {
 		return comp->getHierarchy();
 	}
-	return NULL;
+	return nullptr;
 }
 
 Model *Costume::getModel() {
@@ -270,7 +264,7 @@ Model *Costume::getModel() {
 	if (comp) {
 		return comp->getModel();
 	}
-	return NULL;
+	return nullptr;
 }
 
 void Costume::setChoreLastFrame(int num) {
@@ -289,38 +283,53 @@ void Costume::setChoreLooping(int num, bool val) {
 	_chores[num]->setLooping(val);
 }
 
-void Costume::playChoreLooping(int num) {
+void Costume::playChoreLooping(const char *name, uint msecs) {
+	for (int i = 0; i < _numChores; ++i) {
+		if (strcmp(_chores[i]->getName(), name) == 0) {
+			playChoreLooping(i, msecs);
+			return;
+		}
+	}
+	warning("Costume::playChoreLooping: Could not find chore: %s", name);
+	return;
+}
+
+void Costume::playChoreLooping(int num, uint msecs) {
 	if (num < 0 || num >= _numChores) {
 		Debug::warning(Debug::Chores, "Requested chore number %d is outside the range of chores (0-%d)", num, _numChores);
 		return;
 	}
-	_chores[num]->playLooping();
+
+	_chores[num]->playLooping(msecs);
 	if (Common::find(_playingChores.begin(), _playingChores.end(), _chores[num]) == _playingChores.end())
 		_playingChores.push_back(_chores[num]);
 }
 
 Chore *Costume::getChore(const char *name) {
 	for (int i = 0; i < _numChores; ++i) {
-		if (strcmp(_chores[i]->_name, name) == 0) {
+		if (strcmp(_chores[i]->getName(), name) == 0) {
 			return _chores[i];
 		}
 	}
-	return 0;
+	return nullptr;
 }
 
 int Costume::getChoreId(const char *name) {
+	if (name == nullptr) {
+		return -1;
+	}
 	for (int i = 0; i < _numChores; ++i) {
-		if (strcmp(_chores[i]->_name, name) == 0) {
+		if (strcmp(_chores[i]->getName(), name) == 0) {
 			return i;
 		}
 	}
 	return -1;
 }
 
-void Costume::playChore(const char *name) {
+void Costume::playChore(const char *name, uint msecs) {
 	for (int i = 0; i < _numChores; ++i) {
-			if (strcmp(_chores[i]->_name, name) == 0) {
-			playChore(i);
+		if (strcmp(_chores[i]->getName(), name) == 0) {
+			playChore(i, msecs);
 			return;
 		}
 	}
@@ -328,23 +337,22 @@ void Costume::playChore(const char *name) {
 	return;
 }
 
-void Costume::playChore(int num) {
+void Costume::playChore(int num, uint msecs) {
 	if (num < 0 || num >= _numChores) {
 		Debug::warning(Debug::Chores, "Requested chore number %d is outside the range of chores (0-%d)", num, _numChores);
 		return;
 	}
-	_chores[num]->play();
+	_chores[num]->play(msecs);
 	if (Common::find(_playingChores.begin(), _playingChores.end(), _chores[num]) == _playingChores.end())
 		_playingChores.push_back(_chores[num]);
 }
 
-void Costume::stopChore(int num) {
+void Costume::stopChore(int num, uint msecs) {
 	if (num < 0 || num >= _numChores) {
 		Debug::warning(Debug::Chores, "Requested chore number %d is outside the range of chores (0-%d)", num, _numChores);
 		return;
 	}
-	_chores[num]->stop();
-	_playingChores.remove(_chores[num]);
+	_chores[num]->stop(msecs);
 }
 
 void Costume::setColormap(const Common::String &map) {
@@ -354,17 +362,20 @@ void Costume::setColormap(const Common::String &map) {
 		return;
 	_cmap = g_resourceloader->getColormap(map);
 	for (int i = 0; i < _numComponents; i++)
-		_components[i]->setColormap(NULL);
+		if (_components[i])
+			_components[i]->setColormap(nullptr);
 }
 
-void Costume::stopChores() {
+void Costume::stopChores(bool ignoreLoopingChores, int msecs) {
 	for (int i = 0; i < _numChores; i++) {
-		_chores[i]->stop();
-		_playingChores.remove(_chores[i]);
+		if (ignoreLoopingChores && _chores[i]->isLooping()) {
+			continue;
+		}
+		_chores[i]->stop(msecs);
 	}
 }
 
-void Costume::fadeChoreIn(int chore, int msecs) {
+void Costume::fadeChoreIn(int chore, uint msecs) {
 	if (chore < 0 || chore >= _numChores) {
 		Debug::warning(Debug::Chores, "Requested chore number %d is outside the range of chores (0-%d)", chore, _numChores);
 		return;
@@ -374,7 +385,7 @@ void Costume::fadeChoreIn(int chore, int msecs) {
 		_playingChores.push_back(_chores[chore]);
 }
 
-void Costume::fadeChoreOut(int chore, int msecs) {
+void Costume::fadeChoreOut(int chore, uint msecs) {
 	if (chore < 0 || chore >= _numChores) {
 		Debug::warning(Debug::Chores, "Requested chore number %d is outside the range of chores (0-%d)", chore, _numChores);
 		return;
@@ -384,7 +395,7 @@ void Costume::fadeChoreOut(int chore, int msecs) {
 
 int Costume::isChoring(const char *name, bool excludeLooping) {
 	for (int i = 0; i < _numChores; i++) {
-		if (!strcmp(_chores[i]->_name, name) && _chores[i]->_playing && !(excludeLooping && _chores[i]->_looping))
+		if (!strcmp(_chores[i]->getName(), name) && _chores[i]->isPlaying() && !(excludeLooping && _chores[i]->isLooping()))
 			return i;
 	}
 	return -1;
@@ -395,7 +406,7 @@ int Costume::isChoring(int num, bool excludeLooping) {
 		Debug::warning(Debug::Chores, "Requested chore number %d is outside the range of chores (0-%d)", num, _numChores);
 		return -1;
 	}
-	if (_chores[num]->_playing && !(excludeLooping && _chores[num]->_looping))
+	if (_chores[num]->isPlaying() && !(excludeLooping && _chores[num]->isLooping()))
 		return num;
 	else
 		return -1;
@@ -403,7 +414,7 @@ int Costume::isChoring(int num, bool excludeLooping) {
 
 int Costume::isChoring(bool excludeLooping) {
 	for (int i = 0; i < _numChores; i++) {
-		if (_chores[i]->_playing && !(excludeLooping && _chores[i]->_looping))
+		if (_chores[i]->isPlaying() && !(excludeLooping && _chores[i]->isLooping()))
 			return i;
 	}
 	return -1;
@@ -428,13 +439,18 @@ void Costume::getBoundingBox(int *x1, int *y1, int *x2, int *y2) {
 			ModelComponent *c = static_cast<ModelComponent *>(_components[i]);
 			c->getBoundingBox(x1, y1, x2, y2);
 		}
+
+		if (_components[i] &&(_components[i]->isComponentType('m','e','s','h'))) {
+			EMIMeshComponent *c = static_cast<EMIMeshComponent *>(_components[i]);
+			c->getBoundingBox(x1, y1, x2, y2);
+		}
 	}
 }
 
 int Costume::update(uint time) {
 	for (Common::List<Chore*>::iterator i = _playingChores.begin(); i != _playingChores.end(); ++i) {
 		(*i)->update(time);
-		if (!(*i)->_playing) {
+		if (!(*i)->isPlaying()) {
 			i = _playingChores.erase(i);
 			--i;
 		}
@@ -466,10 +482,15 @@ void Costume::moveHead(bool entering, const Math::Vector3d &lookAt) {
 	_head->lookAt(entering, lookAt, _lookAtRate, _matrix);
 }
 
+int Costume::getHeadJoint() const {
+	return static_cast<Head *>(_head)->getJoint3();
+}
+
 void Costume::setHead(int joint1, int joint2, int joint3, float maxRoll, float maxPitch, float maxYaw) {
-	_head->setJoints(joint1, joint2, joint3);
-	_head->loadJoints(getModelNodes());
-	_head->setMaxAngles(maxPitch, maxYaw, maxRoll);
+	Head *head = static_cast<Head *>(_head);
+	head->setJoints(joint1, joint2, joint3);
+	head->loadJoints(getModelNodes());
+	head->setMaxAngles(maxPitch, maxYaw, maxRoll);
 }
 
 void Costume::setLookAtRate(float rate) {
@@ -483,7 +504,7 @@ float Costume::getLookAtRate() const {
 void Costume::setPosRotate(const Math::Vector3d &pos, const Math::Angle &pitch,
 						   const Math::Angle &yaw, const Math::Angle &roll) {
 	_matrix.setPosition(pos);
-	_matrix.buildFromPitchYawRoll(pitch, yaw, roll);
+	_matrix.buildFromEuler(yaw, pitch, roll, Math::EO_ZXY);
 }
 
 Math::Matrix4 Costume::getMatrix() const {
@@ -503,12 +524,7 @@ void Costume::saveState(SaveGame *state) const {
 	}
 
 	for (int i = 0; i < _numChores; ++i) {
-		Chore *c = _chores[i];
-
-		state->writeBool(c->_hasPlayed);
-		state->writeBool(c->_playing);
-		state->writeBool(c->_looping);
-		state->writeLESint32(c->_currTime);
+		_chores[i]->saveState(state);
 	}
 
 	for (int i = 0; i < _numComponents; ++i) {
@@ -516,14 +532,13 @@ void Costume::saveState(SaveGame *state) const {
 
 		if (c) {
 			state->writeBool(c->_visible);
-			state->writeVector3d(c->_matrix.getPosition());
 			c->saveState(state);
 		}
 	}
 
 	state->writeLEUint32(_playingChores.size());
 	for (Common::List<Chore*>::const_iterator i = _playingChores.begin(); i != _playingChores.end(); ++i) {
-		state->writeLESint32((*i)->getId());
+		state->writeLESint32((*i)->getChoreId());
 	}
 
 	state->writeFloat(_lookAtRate);
@@ -537,19 +552,18 @@ bool Costume::restoreState(SaveGame *state) {
 	}
 
 	for (int i = 0; i < _numChores; ++i) {
-		Chore *c = _chores[i];
-
-		c->_hasPlayed = state->readBool();
-		c->_playing = state->readBool();
-		c->_looping = state->readBool();
-		c->_currTime = state->readLESint32();
+		_chores[i]->restoreState(state);
 	}
+
 	for (int i = 0; i < _numComponents; ++i) {
 		Component *c = _components[i];
 
 		if (c) {
 			c->_visible = state->readBool();
-			c->_matrix.setPosition(state->readVector3d());
+			if (state->saveMinorVersion() < 14) {
+				// skip the old _matrix vector
+				state->readVector3d();
+			}
 			c->restoreState(state);
 		}
 	}
