@@ -8,26 +8,20 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
-#define FORBIDDEN_SYMBOL_EXCEPTION_setjmp
-#define FORBIDDEN_SYMBOL_EXCEPTION_longjmp
 #define FORBIDDEN_SYMBOL_EXCEPTION_fprintf
 #define FORBIDDEN_SYMBOL_EXCEPTION_fgetc
-#define FORBIDDEN_SYMBOL_EXCEPTION_chdir
-#define FORBIDDEN_SYMBOL_EXCEPTION_getcwd
-#define FORBIDDEN_SYMBOL_EXCEPTION_getwd
-#define FORBIDDEN_SYMBOL_EXCEPTION_unlink
 #define FORBIDDEN_SYMBOL_EXCEPTION_stderr
 #define FORBIDDEN_SYMBOL_EXCEPTION_stdin
 
@@ -42,14 +36,18 @@
 
 #include "gui/error.h"
 #include "gui/gui-manager.h"
+#include "gui/message.h"
 
 #include "engines/engine.h"
 
+#include "engines/grim/md5check.h"
+#include "engines/grim/md5checkdialog.h"
 #include "engines/grim/debug.h"
 #include "engines/grim/grim.h"
 #include "engines/grim/lua.h"
 #include "engines/grim/lua_v1.h"
-#include "engines/grim/emi/lua_v2.h"
+#include "engines/grim/emi/poolsound.h"
+#include "engines/grim/emi/layer.h"
 #include "engines/grim/actor.h"
 #include "engines/grim/movie/movie.h"
 #include "engines/grim/savegame.h"
@@ -64,33 +62,48 @@
 #include "engines/grim/set.h"
 #include "engines/grim/sound.h"
 #include "engines/grim/stuffit.h"
+#include "engines/grim/debugger.h"
 
 #include "engines/grim/imuse/imuse.h"
+#include "engines/grim/emi/sound/emisound.h"
 
 #include "engines/grim/lua/lua.h"
 
 namespace Grim {
 
-GrimEngine *g_grim = NULL;
-GfxBase *g_driver = NULL;
+GrimEngine *g_grim = nullptr;
+GfxBase *g_driver = nullptr;
 int g_imuseState = -1;
 
 GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, Common::Platform platform, Common::Language language) :
-		Engine(syst), _currSet(NULL), _selectedActor(NULL) {
+		Engine(syst), _currSet(nullptr), _selectedActor(nullptr), _pauseStartTime(0) {
 	g_grim = this;
 
+	_debugger = new Debugger();
 	_gameType = gameType;
 	_gameFlags = gameFlags;
 	_gamePlatform = platform;
 	_gameLanguage = language;
 
-	g_registry = new Registry();
-	g_resourceloader = NULL;
-	g_localizer = NULL;
-	g_movie = NULL;
-	g_imuse = NULL;
+	if (getGameType() == GType_GRIM)
+		g_registry = new Registry();
+	else
+		g_registry = nullptr;
+
+	g_resourceloader = nullptr;
+	g_localizer = nullptr;
+	g_movie = nullptr;
+	g_imuse = nullptr;
+
+	//Set default settings
+	ConfMan.registerDefault("soft_renderer", false);
+	ConfMan.registerDefault("engine_speed", 60);
+	ConfMan.registerDefault("fullscreen", false);
+	ConfMan.registerDefault("show_fps", false);
+	ConfMan.registerDefault("use_arb_shaders", true);
+
+	_showFps = ConfMan.getBool("show_fps");
 
-	_showFps = g_registry->getBool("show_fps");
 	_softRenderer = true;
 
 	_mixer->setVolumeForSoundType(Audio::Mixer::kPlainSoundType, 192);
@@ -98,8 +111,8 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, ConfMan.getInt("speech_volume"));
 	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
 
-	_currSet = NULL;
-	_selectedActor = NULL;
+	_currSet = nullptr;
+	_selectedActor = nullptr;
 	_controlsEnabled = new bool[KEYCODE_EXTRA_LAST];
 	_controlsState = new bool[KEYCODE_EXTRA_LAST];
 	for (int i = 0; i < KEYCODE_EXTRA_LAST; i++) {
@@ -110,17 +123,14 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	_textSpeed = 7;
 	_mode = _previousMode = NormalMode;
 	_flipEnable = true;
-	int speed = g_registry->getInt("engine_speed");
+	int speed = ConfMan.getInt("engine_speed");
 	if (speed <= 0 || speed > 100)
 		_speedLimitMs = 1000 / 60;
 	else
 		_speedLimitMs = 1000 / speed;
-	char buf[20];
-	sprintf(buf, "%d", 1000 / _speedLimitMs);
-	g_registry->setString("engine_speed", buf);
-	_refreshDrawNeeded = true;
-	_listFilesIter = NULL;
-	_savedState = NULL;
+	ConfMan.setInt("engine_speed", 1000 / _speedLimitMs);
+	_listFilesIter = nullptr;
+	_savedState = nullptr;
 	_fps[0] = 0;
 	_iris = new Iris();
 	_buildActiveActorsList = false;
@@ -132,7 +142,7 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	_printLineDefaults.setWidth(0);
 	_printLineDefaults.setHeight(0);
 	_printLineDefaults.setFGColor(c);
-	_printLineDefaults.setFont(NULL);
+	_printLineDefaults.setFont(nullptr);
 	_printLineDefaults.setJustify(TextObject::LJUSTIFY);
 
 	_sayLineDefaults.setX(0);
@@ -140,7 +150,7 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	_sayLineDefaults.setWidth(0);
 	_sayLineDefaults.setHeight(0);
 	_sayLineDefaults.setFGColor(c);
-	_sayLineDefaults.setFont(NULL);
+	_sayLineDefaults.setFont(nullptr);
 	_sayLineDefaults.setJustify(TextObject::CENTER);
 
 	_blastTextDefaults.setX(0);
@@ -148,7 +158,7 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	_blastTextDefaults.setWidth(0);
 	_blastTextDefaults.setHeight(0);
 	_blastTextDefaults.setFGColor(c);
-	_blastTextDefaults.setFont(NULL);
+	_blastTextDefaults.setFont(nullptr);
 	_blastTextDefaults.setJustify(TextObject::LJUSTIFY);
 
 	const Common::FSNode gameDataDir(ConfMan.get("path"));
@@ -168,23 +178,29 @@ GrimEngine::~GrimEngine() {
 	if (g_registry) {
 		g_registry->save();
 		delete g_registry;
-		g_registry = NULL;
+		g_registry = nullptr;
 	}
 	delete g_movie;
-	g_movie = NULL;
+	g_movie = nullptr;
 	delete g_imuse;
-	g_imuse = NULL;
+	g_imuse = nullptr;
+	delete g_emiSound;
+	g_emiSound = nullptr;
 	delete g_sound;
-	g_sound = NULL;
+	g_sound = nullptr;
 	delete g_localizer;
-	g_localizer = NULL;
+	g_localizer = nullptr;
 	delete g_resourceloader;
-	g_resourceloader = NULL;
+	g_resourceloader = nullptr;
 	delete g_driver;
-	g_driver = NULL;
+	g_driver = nullptr;
 	delete _iris;
+	delete _debugger;
 
+	ConfMan.flushToDisk();
 	DebugMan.clearAllDebugChannels();
+
+	g_grim = nullptr;
 }
 
 void GrimEngine::clearPools() {
@@ -196,15 +212,19 @@ void GrimEngine::clearPools() {
 	Font::getPool().deleteObjects();
 	ObjectState::getPool().deleteObjects();
 
-	_currSet = NULL;
+	_currSet = nullptr;
+}
+
+LuaBase *GrimEngine::createLua() {
+	return new Lua_V1();
 }
 
 void GrimEngine::createRenderer() {
 #ifdef USE_OPENGL
-	_softRenderer = g_registry->getBool("soft_renderer");
+	_softRenderer = ConfMan.getBool("soft_renderer");
 #endif
 
-	if (!_softRenderer && !g_system->hasFeature(OSystem::kFeatureOpenGL)){
+	if (!_softRenderer && !g_system->hasFeature(OSystem::kFeatureOpenGL)) {
 		warning("gfx backend doesn't support hardware rendering");
 		_softRenderer = true;
 	}
@@ -218,16 +238,52 @@ void GrimEngine::createRenderer() {
 	}
 }
 
+const char *GrimEngine::getUpdateFilename() {
+	if (!(getGameFlags() & ADGF_DEMO))
+		return "gfupd101.exe";
+	else
+		return nullptr;
+}
+
 Common::Error GrimEngine::run() {
 	// Try to see if we have the EMI Mac installer present
 	// Currently, this requires the data fork to be standalone
-	if (getGameType() == GType_MONKEY4 && SearchMan.hasFile("Monkey Island 4 Installer")) {
-		StuffItArchive *archive = new StuffItArchive();
+	if (getGameType() == GType_MONKEY4) {
+		if (SearchMan.hasFile("Monkey Island 4 Installer")) {
+			StuffItArchive *archive = new StuffItArchive();
 
-		if (archive->open("Monkey Island 4 Installer"))
-			SearchMan.add("Monkey Island 4 Installer", archive, 0, true);
-		else
-			delete archive;
+			if (archive->open("Monkey Island 4 Installer"))
+				SearchMan.add("Monkey Island 4 Installer", archive, 0, true);
+			else
+				delete archive;
+		}
+		if (SearchMan.hasFile("EFMI Installer")) {
+			StuffItArchive *archive = new StuffItArchive();
+
+			if (archive->open("EFMI Installer"))
+				SearchMan.add("EFMI Installer", archive, 0, true);
+			else
+				delete archive;
+
+		}
+	}
+
+	ConfMan.registerDefault("check_gamedata", true);
+	if (ConfMan.getBool("check_gamedata")) {
+		MD5CheckDialog d;
+		if (!d.runModal()) {
+			Common::String confirmString("ResidualVM found some problems with your game data files.\n"
+										 "Running ResidualVM nevertheless may cause game bugs or even crashes.\n"
+										 "Do you still want to run ");
+			confirmString += (GType_MONKEY4 == getGameType() ? "Escape From Monkey Island?" : "Grim Fandango?");
+			GUI::MessageDialog msg(confirmString, "Yes", "No");
+			if (!msg.runModal()) {
+				return Common::kUserCanceled;
+			}
+		}
+
+		ConfMan.setBool("check_gamedata", false);
+		ConfMan.flushToDisk();
 	}
 
 	g_resourceloader = new ResourceLoader();
@@ -240,19 +296,25 @@ Common::Error GrimEngine::run() {
 		else
 			g_movie = CreateBinkPlayer(demo);
 	}
-	g_imuse = new Imuse(20, demo);
+	if (getGameType() == GType_GRIM) {
+		g_imuse = new Imuse(20, demo);
+		g_emiSound = nullptr;
+	} else if (getGameType() == GType_MONKEY4) {
+		g_emiSound = new EMISound(20);
+		g_imuse = nullptr;
+	}
 	g_sound = new SoundPlayer();
 
-	bool fullscreen = g_registry->getBool("fullscreen");
+	bool fullscreen = ConfMan.getBool("fullscreen");
 	createRenderer();
 	g_driver->setupScreen(640, 480, fullscreen);
 
 	if (getGameType() == GType_MONKEY4 && SearchMan.hasFile("AMWI.m4b")) {
-		// TODO: Play EMI Mac Aspyr logo
-		warning("TODO: Play Aspyr logo");
+		// Play EMI Mac Aspyr logo
+		playAspyrLogo();
 	}
 
-	Bitmap *splash_bm = NULL;
+	Bitmap *splash_bm = nullptr;
 	if (!(_gameFlags & ADGF_DEMO) && getGameType() == GType_GRIM)
 		splash_bm = Bitmap::create("splash.bm");
 	else if ((_gameFlags & ADGF_DEMO) && getGameType() == GType_MONKEY4)
@@ -262,7 +324,7 @@ Common::Error GrimEngine::run() {
 
 	g_driver->clearScreen();
 
-	if (splash_bm != NULL)
+	if (splash_bm != nullptr)
 		splash_bm->draw();
 
 	// This flipBuffer() may make the OpenGL renderer show garbage instead of the splash,
@@ -270,21 +332,14 @@ Common::Error GrimEngine::run() {
 	if (_softRenderer)
 		g_driver->flipBuffer();
 
-	LuaBase *lua = NULL;
-	if (getGameType() == GType_GRIM) {
-		lua = new Lua_V1();
-	} else {
-		lua = new Lua_V2();
-	}
+	LuaBase *lua = createLua();
 
 	lua->registerOpcodes();
 	lua->registerLua();
 
-	lua->loadSystemScript();
-
-	//Make sure that the localizer is initialized after the version is checked
+	//Initialize Localizer first. In system-script are already localizeable Strings
 	g_localizer = new Localizer();
-
+	lua->loadSystemScript();
 	lua->boot();
 
 	_savegameLoadRequest = false;
@@ -296,16 +351,65 @@ Common::Error GrimEngine::run() {
 	}
 
 	g_grim->setMode(NormalMode);
-	if (splash_bm)
-		delete splash_bm;
+	delete splash_bm;
 	g_grim->mainLoop();
 
 	return Common::kNoError;
 }
 
+void GrimEngine::playAspyrLogo() {
+	// A trimmed down version of the code found in mainloop
+	// for the purpose of playing the Aspyr-logo. 
+	// The reason for this, is that the logo needs a different
+	// codec than all the other videos (which are Bink).
+	// Code is provided to keep within the fps-limit, as well as to
+	// allow for pressing ESC to skip the movie.
+	MoviePlayer *defaultPlayer = g_movie;
+	g_movie = CreateQuickTimePlayer();
+	g_movie->play("AMWI.m4b", false, 0, 0);
+	setMode(SmushMode);
+	while (g_movie->isPlaying()) {
+		_doFlip = true;
+		uint32 startTime = g_system->getMillis();
+
+		updateDisplayScene();
+		if (_doFlip) {
+			doFlip();
+		}
+		// Process events to allow the user to skip the logo.
+		Common::Event event;
+		while (g_system->getEventManager()->pollEvent(event)) {
+			// Ignore everything but ESC when movies are playing
+			Common::EventType type = event.type;
+			if (type == Common::EVENT_KEYDOWN && event.kbd.keycode == Common::KEYCODE_ESCAPE) {
+				g_movie->stop();
+				break;
+			}
+		}
+
+		uint32 endTime = g_system->getMillis();
+		if (startTime > endTime)
+			continue;
+		uint32 diffTime = endTime - startTime;
+		if (_speedLimitMs == 0)
+			continue;
+		if (diffTime < _speedLimitMs) {
+			uint32 delayTime = _speedLimitMs - diffTime;
+			g_system->delayMillis(delayTime);
+		}
+	}
+	delete g_movie;
+	setMode(NormalMode);
+	g_movie = defaultPlayer;
+}
+
 Common::Error GrimEngine::loadGameState(int slot) {
 	assert(slot >= 0);
-	_savegameFileName = Common::String::format("grim%02d.gsv", slot);
+	if (getGameType() == GType_MONKEY4) {
+		_savegameFileName = Common::String::format("efmi%03d.gsv", slot);
+	} else {
+		_savegameFileName = Common::String::format("grim%02d.gsv", slot);
+	}
 	_savegameLoadRequest = true;
 	return Common::kNoError;
 }
@@ -348,9 +452,9 @@ void GrimEngine::savegameCallback() {
 }
 
 void GrimEngine::handleDebugLoadResource() {
-	void *resource = NULL;
+	void *resource = nullptr;
 	int c, i = 0;
-	char buf[512];
+	char buf[513];
 
 	// Tool for debugging the loading of a particular resource without
 	// having to actually make it all the way to it in the game
@@ -366,18 +470,20 @@ void GrimEngine::handleDebugLoadResource() {
 	else if (strstr(buf, ".cmp"))
 		resource = (void *)g_resourceloader->loadColormap(buf);
 	else if (strstr(buf, ".cos"))
-		resource = (void *)g_resourceloader->loadCostume(buf, NULL);
+		resource = (void *)g_resourceloader->loadCostume(buf, nullptr, nullptr);
 	else if (strstr(buf, ".lip"))
 		resource = (void *)g_resourceloader->loadLipSync(buf);
 	else if (strstr(buf, ".snm"))
 		resource = (void *)g_movie->play(buf, false, 0, 0);
 	else if (strstr(buf, ".wav") || strstr(buf, ".imu")) {
-		g_imuse->startSfx(buf);
+		if (g_imuse)
+			g_imuse->startSfx(buf);
 		resource = (void *)1;
 	} else if (strstr(buf, ".mat")) {
 		CMap *cmap = g_resourceloader->loadColormap("item.cmp");
 		warning("Default colormap applied to resources loaded in this fashion");
-		resource = (void *)g_resourceloader->loadMaterial(buf, cmap);
+		// Default to repeating the texture as in GRIM
+		resource = (void *)g_resourceloader->loadMaterial(buf, cmap, false);
 	} else {
 		warning("Resource type not understood");
 	}
@@ -385,18 +491,9 @@ void GrimEngine::handleDebugLoadResource() {
 		warning("Requested resouce (%s) not found", buf);
 }
 
-void GrimEngine::drawPrimitives() {
-	_iris->draw();
-
-	// Draw text
-	if (_mode == SmushMode) {
-		if (_movieSubtitle) {
-			_movieSubtitle->draw();
-		}
-	} else {
-		foreach (TextObject *t, TextObject::getPool()) {
-			t->draw();
-		}
+void GrimEngine::drawTextObjects() {
+	foreach (TextObject *t, TextObject::getPool()) {
+		t->draw();
 	}
 }
 
@@ -466,7 +563,7 @@ void GrimEngine::updateDisplayScene() {
 				if (frame != _prevSmushFrame) {
 					_prevSmushFrame = g_movie->getFrame();
 					g_driver->drawMovieFrame(g_movie->getX(), g_movie->getY());
-					if (_showFps)
+					if (_showFps && _mode != DrawMode)
 						g_driver->drawEmergString(550, 25, _fps, Color(255, 255, 255));
 				} else
 					_doFlip = false;
@@ -474,90 +571,104 @@ void GrimEngine::updateDisplayScene() {
 				g_driver->releaseMovieFrame();
 		}
 		// Draw Primitives
-		foreach (PrimitiveObject *p, PrimitiveObject::getPool()) {
-			p->draw();
+		_iris->draw();
+		if (_movieSubtitle) {
+			_movieSubtitle->draw();
 		}
-		drawPrimitives();
 	} else if (_mode == NormalMode || _mode == OverworldMode) {
-		if (!_currSet)
-			return;
-
-		g_driver->clearScreen();
-
-		_prevSmushFrame = 0;
-		_movieTime = 0;
-
-		_currSet->drawBackground();
-
-		// Draw underlying scene components
-		// Background objects are drawn underneath everything except the background
-		// There are a bunch of these, especially in the tube-switcher room
-		_currSet->drawBitmaps(ObjectState::OBJSTATE_BACKGROUND);
-
-		// State objects are drawn on top of other things, such as the flag
-		// on Manny's message tube
-		_currSet->drawBitmaps(ObjectState::OBJSTATE_STATE);
-
-		// Play SMUSH Animations
-		// This should occur on top of all underlying scene objects,
-		// a good example is the tube switcher room where some state objects
-		// need to render underneath the animation or you can't see what's going on
-		// This should not occur on top of everything though or Manny gets covered
-		// up when he's next to Glottis's service room
-		if (g_movie->isPlaying() && _movieSetup == _currSet->getCurrSetup()->_name) {
-			_movieTime = g_movie->getMovieTime();
-			if (g_movie->isUpdateNeeded()) {
-				g_driver->prepareMovieFrame(g_movie->getDstSurface());
-				g_movie->clearUpdateNeeded();
-			}
-			if (g_movie->getFrame() >= 0)
-				g_driver->drawMovieFrame(g_movie->getX(), g_movie->getY());
-			else
-				g_driver->releaseMovieFrame();
-		}
-
-		// Underlay objects must be drawn on top of movies
-		// Otherwise the lighthouse door will always be open as the underlay for
-		// the closed door will be overdrawn by a movie used as background image.
-		_currSet->drawBitmaps(ObjectState::OBJSTATE_UNDERLAY);
-
-		// Draw Primitives
-		foreach (PrimitiveObject *p, PrimitiveObject::getPool()) {
-			p->draw();
-		}
-
-		_currSet->setupCamera();
-
-		g_driver->set3DMode();
-
-		if (_setupChanged) {
-			cameraPostChangeHandle(_currSet->getSetup());
-			_setupChanged = false;
-		}
-
-		// Draw actors
-		buildActiveActorsList();
-		foreach (Actor *a, _activeActors) {
-			if (a->isVisible())
-				a->draw();
-		}
-
-		flagRefreshShadowMask(false);
-
-		// Draw overlying scene components
-		// The overlay objects should be drawn on top of everything else,
-		// including 3D objects such as Manny and the message tube
-		_currSet->drawBitmaps(ObjectState::OBJSTATE_OVERLAY);
-
-		g_driver->drawBuffers();
-		drawPrimitives();
+		updateNormalMode();
 	} else if (_mode == DrawMode) {
-		_doFlip = false;
-		_prevSmushFrame = 0;
-		_movieTime = 0;
+		updateDrawMode();
 	}
 }
 
+void GrimEngine::updateNormalMode() {
+	if (!_currSet)
+		return;
+
+	g_driver->clearScreen();
+
+	drawNormalMode();
+
+	g_driver->drawBuffers();
+
+	_iris->draw();
+	drawTextObjects();
+}
+
+void GrimEngine::updateDrawMode() {
+	_doFlip = false;
+	_prevSmushFrame = 0;
+	_movieTime = 0;
+}
+
+void GrimEngine::drawNormalMode() {
+	_prevSmushFrame = 0;
+	_movieTime = 0;
+
+	_currSet->drawBackground();
+
+	// Draw underlying scene components
+	// Background objects are drawn underneath everything except the background
+	// There are a bunch of these, especially in the tube-switcher room
+	_currSet->drawBitmaps(ObjectState::OBJSTATE_BACKGROUND);
+
+	// State objects are drawn on top of other things, such as the flag
+	// on Manny's message tube
+	_currSet->drawBitmaps(ObjectState::OBJSTATE_STATE);
+
+	// Play SMUSH Animations
+	// This should occur on top of all underlying scene objects,
+	// a good example is the tube switcher room where some state objects
+	// need to render underneath the animation or you can't see what's going on
+	// This should not occur on top of everything though or Manny gets covered
+	// up when he's next to Glottis's service room
+	if (g_movie->isPlaying() && _movieSetup == _currSet->getCurrSetup()->_name) {
+		_movieTime = g_movie->getMovieTime();
+		if (g_movie->isUpdateNeeded()) {
+			g_driver->prepareMovieFrame(g_movie->getDstSurface());
+			g_movie->clearUpdateNeeded();
+		}
+		if (g_movie->getFrame() >= 0)
+			g_driver->drawMovieFrame(g_movie->getX(), g_movie->getY());
+		else
+			g_driver->releaseMovieFrame();
+	}
+
+	// Underlay objects must be drawn on top of movies
+	// Otherwise the lighthouse door will always be open as the underlay for
+	// the closed door will be overdrawn by a movie used as background image.
+	_currSet->drawBitmaps(ObjectState::OBJSTATE_UNDERLAY);
+
+	// Draw Primitives
+	foreach (PrimitiveObject *p, PrimitiveObject::getPool()) {
+		p->draw();
+	}
+
+	_currSet->setupCamera();
+
+	g_driver->set3DMode();
+
+	if (_setupChanged) {
+		cameraPostChangeHandle(_currSet->getSetup());
+		_setupChanged = false;
+	}
+
+	// Draw actors
+	buildActiveActorsList();
+	foreach (Actor *a, _activeActors) {
+		if (a->isVisible())
+			a->draw();
+	}
+
+	flagRefreshShadowMask(false);
+
+	// Draw overlying scene components
+	// The overlay objects should be drawn on top of everything else,
+	// including 3D objects such as Manny and the message tube
+	_currSet->drawBitmaps(ObjectState::OBJSTATE_OVERLAY);
+}
+
 void GrimEngine::doFlip() {
 	_frameCounter++;
 	if (!_doFlip) {
@@ -574,7 +685,7 @@ void GrimEngine::doFlip() {
 		unsigned int currentTime = g_system->getMillis();
 		unsigned int delta = currentTime - _lastFrameTime;
 		if (delta > 500) {
-			sprintf(_fps, "%7.2f", (double)(_frameCounter * 1000) / (double)delta );
+			sprintf(_fps, "%7.2f", (double)(_frameCounter * 1000) / (double)delta);
 			_frameCounter = 0;
 			_lastFrameTime = currentTime;
 		}
@@ -622,7 +733,7 @@ void GrimEngine::mainLoop() {
 				fullscreen = !fullscreen;
 			}
 			g_system->setFeatureState(OSystem::kFeatureFullscreenMode, fullscreen);
-			g_registry->setBool("fullscreen", fullscreen);
+			ConfMan.setBool("fullscreen", fullscreen);
 
 			uint screenWidth = g_driver->getScreenWidth();
 			uint screenHeight = g_driver->getScreenHeight();
@@ -648,8 +759,12 @@ void GrimEngine::mainLoop() {
 			_changeFullscreenState = false;
 		}
 
-		g_imuse->flushTracks();
-		g_imuse->refreshScripts();
+		g_sound->flushTracks();
+		if (g_imuse) {
+			g_imuse->refreshScripts();
+		}
+
+		_debugger->onFrame();
 
 		// Process events
 		Common::Event event;
@@ -658,6 +773,17 @@ void GrimEngine::mainLoop() {
 			Common::EventType type = event.type;
 			if (type == Common::EVENT_KEYDOWN || type == Common::EVENT_KEYUP) {
 				if (type == Common::EVENT_KEYDOWN) {
+					// Ignore everything but ESC when movies are playing
+					// This matches the retail and demo versions of EMI
+					// This also allows the PS2 version to skip movies
+					if (_mode == SmushMode && g_grim->getGameType() == GType_MONKEY4) {
+						if (event.kbd.keycode == Common::KEYCODE_ESCAPE) {
+							g_movie->stop();
+							break;
+						}
+						continue;
+					}
+
 					if (_mode != DrawMode && _mode != SmushMode && (event.kbd.ascii == 'q')) {
 						handleExit();
 						break;
@@ -676,19 +802,33 @@ void GrimEngine::mainLoop() {
 				// if the button is not kept pressed the KEYUP will arrive just after the KEYDOWN
 				// and it will break the lua scripts that checks for the state of the button
 				// with GetControlState()
-				luaUpdate();
+
+				// We do not want the scripts to update while a movie is playing in the PS2-version.
+				if (!(getGamePlatform() == Common::kPlatformPS2 && _mode == SmushMode)) {
+					luaUpdate();
+				}
 			}
-			if (type == Common::EVENT_SCREEN_CHANGED)
-				_refreshDrawNeeded = true;
 		}
 
-		luaUpdate();
+		if (_mode != PauseMode) {
+			// Draw the display scene before doing the luaUpdate.
+			// This give a large performance boost as OpenGL stores commands
+			// in a queue on the gpu to be rendered later. When doFlip is
+			// called the cpu must wait for the gpu to finish its queue.
+			// Now, it will queue all the OpenGL commands and draw them on the
+			// GPU while the CPU is busy updating the game world.
+			updateDisplayScene();
+		}
 
 		if (_mode != PauseMode) {
-			updateDisplayScene();
 			doFlip();
 		}
 
+		// We do not want the scripts to update while a movie is playing in the PS2-version.
+		if (!(getGamePlatform() == Common::kPlatformPS2 && _mode == SmushMode)) {
+			luaUpdate();
+		}
+
 		if (g_imuseState != -1) {
 			g_sound->setMusicState(g_imuseState);
 			g_imuseState = -1;
@@ -733,55 +873,67 @@ void GrimEngine::savegameRestore() {
 	_savedState = SaveGame::openForLoading(filename);
 	if (!_savedState || !_savedState->isCompatible())
 		return;
-	g_imuse->stopAllSounds();
-	g_imuse->resetState();
+	if (g_imuse) {
+		g_imuse->stopAllSounds();
+		g_imuse->resetState();
+	}
 	g_movie->stop();
-	g_imuse->pause(true);
+	if (g_imuse)
+		g_imuse->pause(true);
 	g_movie->pause(true);
-	g_registry->save();
+	if (g_registry)
+	    g_registry->save();
 
-	_selectedActor = NULL;
+	_selectedActor = nullptr;
 	delete _currSet;
-	_currSet = NULL;
+	_currSet = nullptr;
 
 	Bitmap::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "Bitmaps restored succesfully.");
+	Debug::debug(Debug::Engine, "Bitmaps restored successfully.");
 
 	Font::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "Fonts restored succesfully.");
+	Debug::debug(Debug::Engine, "Fonts restored successfully.");
 
 	ObjectState::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "ObjectStates restored succesfully.");
+	Debug::debug(Debug::Engine, "ObjectStates restored successfully.");
 
 	Set::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "Sets restored succesfully.");
+	Debug::debug(Debug::Engine, "Sets restored successfully.");
 
 	TextObject::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "TextObjects restored succesfully.");
+	Debug::debug(Debug::Engine, "TextObjects restored successfully.");
 
 	PrimitiveObject::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "PrimitiveObjects restored succesfully.");
+	Debug::debug(Debug::Engine, "PrimitiveObjects restored successfully.");
 
 	Actor::getPool().restoreObjects(_savedState);
-	Debug::debug(Debug::Engine, "Actors restored succesfully.");
+	Debug::debug(Debug::Engine, "Actors restored successfully.");
+
+	if (getGameType() == GType_MONKEY4) {
+		PoolSound::getPool().restoreObjects(_savedState);
+		Debug::debug(Debug::Engine, "Pool sounds saved successfully.");
+
+		Layer::getPool().restoreObjects(_savedState);
+		Debug::debug(Debug::Engine, "Layers restored successfully.");
+	}
 
 	restoreGRIM();
-	Debug::debug(Debug::Engine, "Engine restored succesfully.");
+	Debug::debug(Debug::Engine, "Engine restored successfully.");
 
 	g_driver->restoreState(_savedState);
-	Debug::debug(Debug::Engine, "Renderer restored succesfully.");
+	Debug::debug(Debug::Engine, "Renderer restored successfully.");
 
-	g_imuse->restoreState(_savedState);
-	Debug::debug(Debug::Engine, "iMuse restored succesfully.");
+	g_sound->restoreState(_savedState);
+	Debug::debug(Debug::Engine, "iMuse restored successfully.");
 
 	g_movie->restoreState(_savedState);
-	Debug::debug(Debug::Engine, "Movie restored succesfully.");
+	Debug::debug(Debug::Engine, "Movie restored successfully.");
 
 	_iris->restoreState(_savedState);
-	Debug::debug(Debug::Engine, "Iris restored succesfully.");
+	Debug::debug(Debug::Engine, "Iris restored successfully.");
 
 	lua_Restore(_savedState);
-	Debug::debug(Debug::Engine, "Lua restored succesfully.");
+	Debug::debug(Debug::Engine, "Lua restored successfully.");
 
 	delete _savedState;
 
@@ -792,7 +944,8 @@ void GrimEngine::savegameRestore() {
 	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
 
 	LuaBase::instance()->postRestoreHandle();
-	g_imuse->pause(false);
+	if (g_imuse)
+		g_imuse->pause(false);
 	g_movie->pause(false);
 	debug("GrimEngine::savegameRestore() finished.");
 
@@ -855,7 +1008,7 @@ void GrimEngine::storeSaveGameImage(SaveGame *state) {
 	g_grim->setMode(_previousMode);
 	g_grim->updateDisplayScene();
 	g_driver->storeDisplay();
-	screenshot = g_driver->getScreenshot(width, height);
+	screenshot = g_driver->getScreenshot(width, height, false);
 	g_grim->setMode(mode);
 	state->beginSection('SIMG');
 	if (screenshot) {
@@ -876,11 +1029,14 @@ void GrimEngine::storeSaveGameImage(SaveGame *state) {
 void GrimEngine::savegameSave() {
 	debug("GrimEngine::savegameSave() started.");
 	_savegameSaveRequest = false;
-	char filename[200];
+	Common::String filename;
 	if (_savegameFileName.size() == 0) {
-		strcpy(filename, "grim.sav");
+		filename = "grim.sav";
 	} else {
-		strcpy(filename, _savegameFileName.c_str());
+		filename = _savegameFileName;
+	}
+	if (getGameType() == GType_MONKEY4 && filename.contains('/')) {
+		filename = Common::lastPathComponent(filename, '/');
 	}
 	_savedState = SaveGame::openForSaving(filename);
 	if (!_savedState) {
@@ -891,52 +1047,62 @@ void GrimEngine::savegameSave() {
 
 	storeSaveGameImage(_savedState);
 
-	g_imuse->pause(true);
+	if (g_imuse)
+		g_imuse->pause(true);
 	g_movie->pause(true);
 
 	savegameCallback();
 
 	Bitmap::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "Bitmaps saved succesfully.");
+	Debug::debug(Debug::Engine, "Bitmaps saved successfully.");
 
 	Font::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "Fonts saved succesfully.");
+	Debug::debug(Debug::Engine, "Fonts saved successfully.");
 
 	ObjectState::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "ObjectStates saved succesfully.");
+	Debug::debug(Debug::Engine, "ObjectStates saved successfully.");
 
 	Set::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "Sets saved succesfully.");
+	Debug::debug(Debug::Engine, "Sets saved successfully.");
 
 	TextObject::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "TextObjects saved succesfully.");
+	Debug::debug(Debug::Engine, "TextObjects saved successfully.");
 
 	PrimitiveObject::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "PrimitiveObjects saved succesfully.");
+	Debug::debug(Debug::Engine, "PrimitiveObjects saved successfully.");
 
 	Actor::getPool().saveObjects(_savedState);
-	Debug::debug(Debug::Engine, "Actors saved succesfully.");
+	Debug::debug(Debug::Engine, "Actors saved successfully.");
+
+	if (getGameType() == GType_MONKEY4) {
+		PoolSound::getPool().saveObjects(_savedState);
+		Debug::debug(Debug::Engine, "Pool sounds saved successfully.");
+
+		Layer::getPool().saveObjects(_savedState);
+		Debug::debug(Debug::Engine, "Layers saved successfully.");
+	}
 
 	saveGRIM();
-	Debug::debug(Debug::Engine, "Engine saved succesfully.");
+	Debug::debug(Debug::Engine, "Engine saved successfully.");
 
 	g_driver->saveState(_savedState);
-	Debug::debug(Debug::Engine, "Renderer saved succesfully.");
+	Debug::debug(Debug::Engine, "Renderer saved successfully.");
 
-	g_imuse->saveState(_savedState);
-	Debug::debug(Debug::Engine, "iMuse saved succesfully.");
+	g_sound->saveState(_savedState);
+	Debug::debug(Debug::Engine, "iMuse saved successfully.");
 
 	g_movie->saveState(_savedState);
-	Debug::debug(Debug::Engine, "Movie saved succesfully.");
+	Debug::debug(Debug::Engine, "Movie saved successfully.");
 
 	_iris->saveState(_savedState);
-	Debug::debug(Debug::Engine, "Iris saved succesfully.");
+	Debug::debug(Debug::Engine, "Iris saved successfully.");
 
 	lua_Save(_savedState);
 
 	delete _savedState;
 
-	g_imuse->pause(false);
+	if (g_imuse)
+		g_imuse->pause(false);
 	g_movie->pause(false);
 	debug("GrimEngine::savegameSave() finished.");
 
@@ -981,7 +1147,7 @@ Set *GrimEngine::findSet(const Common::String &name) {
 		if (s->getName() == name)
 			return s;
 	}
-	return NULL;
+	return nullptr;
 }
 
 void GrimEngine::setSetLock(const char *name, bool lockStatus) {
@@ -1006,8 +1172,8 @@ Set *GrimEngine::loadSet(const Common::String &name) {
 		}
 		Common::SeekableReadStream *stream;
 		stream = g_resourceloader->openNewStreamFile(filename.c_str());
-		if(!stream)
-			warning("Could not find scene file %s", name.c_str());
+		if (!stream)
+			error("Could not find scene file %s", name.c_str());
 
 		s = new Set(name, stream);
 		delete stream;
@@ -1024,6 +1190,11 @@ void GrimEngine::setSet(Set *scene) {
 	if (scene == _currSet)
 		return;
 
+	if (getGameType() == GType_MONKEY4) {
+		foreach (PoolSound *s, PoolSound::getPool()) {
+			s->stop();
+		}
+	}
 	// Stop the actors. This fixes bug #289 (https://github.com/residualvm/residualvm/issues/289)
 	// and it makes sense too, since when changing set the directions
 	// and coords change too.
@@ -1048,6 +1219,11 @@ void GrimEngine::setSet(Set *scene) {
 void GrimEngine::makeCurrentSetup(int num) {
 	int prevSetup = g_grim->getCurrSet()->getSetup();
 	if (prevSetup != num) {
+		foreach (Actor *a, Actor::getPool()) {
+			a->clearCleanBuffer();
+		}
+		g_driver->refreshBuffers();
+
 		getCurrSet()->setSetup(num);
 		getCurrSet()->setSoundParameters(20, 127);
 		cameraChangeHandle(prevSetup, num);
@@ -1077,15 +1253,15 @@ float GrimEngine::getPerSecond(float rate) const {
 	return rate * _frameTime / 1000;
 }
 
-void GrimEngine::immediatelyRemoveActor(Actor *a) {
-	_talkingActors.remove(a);
-	_activeActors.remove(a);
-}
-
 void GrimEngine::invalidateActiveActorsList() {
 	_buildActiveActorsList = true;
 }
 
+void GrimEngine::immediatelyRemoveActor(Actor *actor) {
+	_activeActors.remove(actor);
+	_talkingActors.remove(actor);
+}
+
 void GrimEngine::buildActiveActorsList() {
 	if (!_buildActiveActorsList) {
 		return;
@@ -1093,7 +1269,8 @@ void GrimEngine::buildActiveActorsList() {
 
 	_activeActors.clear();
 	foreach (Actor *a, Actor::getPool()) {
-		if (a->isInSet(_currSet->getName())) {
+		if (((_mode == NormalMode || _mode == DrawMode) && a->isInSet(_currSet->getName())) ||
+		    a->isInOverworld()) {
 			_activeActors.push_back(a);
 		}
 	}
@@ -1116,8 +1293,7 @@ bool GrimEngine::areActorsTalking() const {
 	return talking;
 }
 
-void GrimEngine::setMovieSubtitle(TextObject *to)
-{
+void GrimEngine::setMovieSubtitle(TextObject *to) {
 	if (_movieSubtitle != to) {
 		delete _movieSubtitle;
 		_movieSubtitle = to;
@@ -1133,10 +1309,6 @@ void GrimEngine::setMode(EngineMode mode) {
 	invalidateActiveActorsList();
 }
 
-const Common::String &GrimEngine::getSetName() const {
-	return _currSet->getName();
-}
-
 void GrimEngine::clearEventQueue() {
 	Common::Event event;
 	while (g_system->getEventManager()->pollEvent(event)) {
@@ -1153,4 +1325,27 @@ bool GrimEngine::hasFeature(EngineFeature f) const {
 		(f == kSupportsLoadingDuringRuntime);
 }
 
+void GrimEngine::openMainMenuDialog() {
+	Common::KeyState key(Common::KEYCODE_F1, Common::ASCII_F1);
+	handleControls(Common::EVENT_KEYDOWN, key);
+	handleControls(Common::EVENT_KEYUP, key);
+}
+
+void GrimEngine::pauseEngineIntern(bool pause) {
+	if (g_imuse)
+		g_imuse->pause(pause);
+	if (g_movie)
+		g_movie->pause(pause);
+
+	if (pause) {
+		_pauseStartTime = _system->getMillis();
+	} else {
+		_frameStart += _system->getMillis() - _pauseStartTime;
+	}
+}
+
+void GrimEngine::debugLua(const Common::String &str) {
+	lua_dostring(str.c_str());
+}
+
 } // end of namespace Grim
