@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -23,26 +23,17 @@
 #ifndef GRIM_LOCALIZE_H
 #define GRIM_LOCALIZE_H
 
-#include "common/array.h"
+#include "common/hash-str.h"
 
 namespace Grim {
 
 class Localizer {
 public:
-	Common::String localize(const char *str) const;
-
 	Localizer();
-	~Localizer();
-
-	struct LocaleEntry {
-		char *text;
-		char *translation;
-	};
 
+	Common::String localize(const char *str) const;
 private:
-	char *_data;
-
-	Common::Array<LocaleEntry> _entries;
+	Common::StringMap _entries;
 };
 
 extern Localizer *g_localizer;
