@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -104,11 +104,10 @@ void LuaObjects::pushObjects() const {
 }
 
 
-LuaBase *LuaBase::s_instance = NULL;
+LuaBase *LuaBase::s_instance = nullptr;
 
 LuaBase::LuaBase() :
-	_translationMode(0),
-	_frameTimeCollection(0) {
+		_translationMode(0), _frameTimeCollection(0) {
 	s_instance = this;
 
 	lua_iolibopen();
@@ -117,7 +116,7 @@ LuaBase::LuaBase() :
 }
 
 LuaBase::~LuaBase() {
-	s_instance = NULL;
+	s_instance = nullptr;
 
 	lua_removelibslists();
 	lua_close();
@@ -231,6 +230,8 @@ void LuaBase::registerLua() {
 	refTextObjectPan = lua_ref(true);
 	lua_pushstring("background");
 	refTextObjectBackground = lua_ref(true);
+	lua_pushstring("layer");
+	refTextObjectLayer = lua_ref(true);
 }
 
 struct luaL_reg baseOpcodes[] = {
@@ -249,8 +250,8 @@ void LuaBase::loadSystemScript() {
 }
 
 void LuaBase::boot() {
-	lua_pushnil();		// resumeSave
-	lua_pushnil();		// bootParam - not used in scripts
+	lua_pushnil();      // resumeSave
+	lua_pushnil();      // bootParam - not used in scripts
 	lua_call("BOOT");
 }
 
@@ -387,7 +388,7 @@ void LuaBase::dummyHandler() {
 }
 
 bool LuaBase::findCostume(lua_Object costumeObj, Actor *actor, Costume **costume) {
-	*costume = NULL;
+	*costume = nullptr;
 	if (lua_isnil(costumeObj))
 		return true;
 	if (lua_isnumber(costumeObj)) {
@@ -397,7 +398,7 @@ bool LuaBase::findCostume(lua_Object costumeObj, Actor *actor, Costume **costume
 	}
 	if (lua_isstring(costumeObj)) {
 		*costume = actor->findCostume(lua_getstring(costumeObj));
-		return *costume != 0;
+		return *costume != nullptr;
 	}
 
 	return false;
@@ -405,7 +406,7 @@ bool LuaBase::findCostume(lua_Object costumeObj, Actor *actor, Costume **costume
 
 Common::String LuaBase::parseMsgText(const char *msg, char *msgId) {
 	Common::String translation = g_localizer->localize(msg);
-	const char *secondSlash = NULL;
+	const char *secondSlash = nullptr;
 
 	if (msg[0] == '/' && msgId) {
 		secondSlash = strchr(msg + 1, '/');
@@ -426,7 +427,7 @@ Common::String LuaBase::parseMsgText(const char *msg, char *msgId) {
 	return translation;
 }
 
-void LuaBase::parseSayLineTable(lua_Object paramObj, bool *background, int *vol, int *pan, int *x, int *y) {
+void LuaBase::parseSayLineTable(lua_Object paramObj, bool *background, int *vol, int *pan, float *x, float *y) {
 	lua_Object tableObj;
 
 	lua_pushobject(paramObj);
@@ -434,7 +435,7 @@ void LuaBase::parseSayLineTable(lua_Object paramObj, bool *background, int *vol,
 	tableObj = lua_gettable();
 	if (lua_isnumber(tableObj)) {
 		if (x)
-			*x = (int)lua_getnumber(tableObj);
+			*x = lua_getnumber(tableObj);
 	}
 
 	lua_pushobject(paramObj);
@@ -442,7 +443,7 @@ void LuaBase::parseSayLineTable(lua_Object paramObj, bool *background, int *vol,
 	tableObj = lua_gettable();
 	if (lua_isnumber(tableObj)) {
 		if (y)
-			*y = (int)lua_getnumber(tableObj);
+			*y = lua_getnumber(tableObj);
 	}
 
 	lua_pushobject(paramObj);
@@ -480,7 +481,7 @@ void LuaBase::setTextObjectParams(TextObjectCommon *textObject, lua_Object table
 		if (lua_isnumber(keyObj)) {
 			float num = lua_getnumber(keyObj);
 			if (g_grim->getGameType() == GType_MONKEY4)
-				textObject->setX((int)(num * 640));
+				textObject->setX((int)(num * 320) + 320);
 			else
 				textObject->setX((int)num);
 		}
@@ -493,7 +494,7 @@ void LuaBase::setTextObjectParams(TextObjectCommon *textObject, lua_Object table
 		if (lua_isnumber(keyObj)) {
 			float num = lua_getnumber(keyObj);
 			if (g_grim->getGameType() == GType_MONKEY4)
-				textObject->setY((int)(num * 480));
+				textObject->setY((int)(240 - (num * 240)));
 			else
 				textObject->setY((int)num);
 		}
@@ -505,7 +506,7 @@ void LuaBase::setTextObjectParams(TextObjectCommon *textObject, lua_Object table
 	if (keyObj) {
 		if (g_grim->getGameType() == GType_MONKEY4 && lua_isstring(keyObj)) {
 			const char *str = lua_getstring(keyObj);
-			Font *font = 0;
+			Font *font = nullptr;
 			foreach (Font *f, Font::getPool()) {
 				if (f->getFilename() == str) {
 					font = f;
@@ -518,6 +519,10 @@ void LuaBase::setTextObjectParams(TextObjectCommon *textObject, lua_Object table
 			textObject->setFont(font);
 		} else if (lua_isuserdata(keyObj) && lua_tag(keyObj) == MKTAG('F','O','N','T')) {
 			textObject->setFont(getfont(keyObj));
+		} else if (g_grim->getGameType() == GType_MONKEY4 && !textObject->getFont() && g_grim->getGamePlatform() == Common::kPlatformPS2) {
+			// HACK:
+			warning("HACK: No default font set for PS2-version, just picking one for now");
+			textObject->setFont(*Font::getPool().begin());
 		}
 	}
 
@@ -603,6 +608,20 @@ void LuaBase::setTextObjectParams(TextObjectCommon *textObject, lua_Object table
 			textObject->setDuration((int)lua_getnumber(keyObj));
 		}
 	}
+
+	// FIXME: remove check once the major save version is updated
+	// currently it is needed for backward compatibility of old savegames
+	if (lua_getref(refTextObjectLayer) == LUA_NOOBJECT)
+		return;
+	lua_pushobject(tableObj);
+	lua_pushobject(lua_getref(refTextObjectLayer));
+	keyObj = lua_gettable();
+	if (keyObj) {
+		if (lua_isnumber(keyObj)) {
+			textObject->setLayer(lua_getnumber(keyObj));
+		}
+	}
+
 }
 
 void LuaBase::typeOverride() {
