@@ -8,19 +8,21 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
+#include "common/algorithm.h"
 #include "common/endian.h"
+#include "common/func.h"
 
 #include "engines/grim/debug.h"
 #include "engines/grim/grim.h"
@@ -30,32 +32,23 @@
 #include "engines/grim/gfx_base.h"
 #include "engines/grim/resource.h"
 #include "engines/grim/colormap.h"
+#include "engines/grim/sprite.h"
 
 namespace Grim {
 
-void Sprite::draw() const {
-	if (!_visible)
-		return;
-
-	_material->select();
-	g_driver->drawSprite(this);
-}
 
 /**
  * @class Model
  */
 Model::Model(const Common::String &filename, Common::SeekableReadStream *data, CMap *cmap, Model *parent) :
-		Object(), _parent(parent), _numMaterials(0), _numGeosets(0), _cmap(cmap) {
-	_fname = filename;
+		Object(), _parent(parent), _numMaterials(0), _numGeosets(0), _cmap(cmap), _fname(filename) {
 
-	if (g_grim->getGameType() == GType_MONKEY4) {
-		loadEMI(data);
-	} else if (data->readUint32BE() == MKTAG('L','D','O','M'))
-		loadBinary(data, cmap);
+	if (data->readUint32BE() == MKTAG('L','D','O','M'))
+		loadBinary(data);
 	else {
 		data->seek(0, SEEK_SET);
-		TextSplitter ts(data);
-		loadText(&ts, cmap);
+		TextSplitter ts(_fname, data);
+		loadText(&ts);
 	}
 
 	Math::Vector3d max;
@@ -65,6 +58,7 @@ Model::Model(const Common::String &filename, Common::SeekableReadStream *data, C
 	for (int i = 0; i < _numHierNodes; ++i) {
 		ModelNode &node = _rootHierNode[i];
 		if (node._mesh) {
+			g_driver->createMesh(node._mesh);
 			Mesh &mesh = *node._mesh;
 			Math::Vector3d p = mesh._matrix.getPosition();
 			float x = p.x();
@@ -107,39 +101,7 @@ Model::~Model() {
 	g_resourceloader->uncacheModel(this);
 }
 
-void Model::loadEMI(Common::SeekableReadStream *data) {
-	char name[64];
-
-	int nameLength = data->readUint32LE();
-	assert(nameLength < 64);
-
-	data->read(name, nameLength);
-
-	// skip over some unkown floats
-	data->seek(48, SEEK_CUR);
-
-	_numMaterials = data->readUint32LE();
-	_materials = new Material*[_numMaterials];
-	_materialNames = new char[_numMaterials][32];
-	for (int i = 0; i < _numMaterials; i++) {
-		nameLength = data->readUint32LE();
-		assert(nameLength < 32);
-
-		data->read(_materialNames[i], nameLength);
-		// I'm not sure what specialty mateials are, but they are handled differently.
-		if (memcmp(_materialNames[i], "specialty", 9) == 0) {
-			_materials[i] = 0;
-		} else {
-			loadMaterial(i, 0);
-		}
-		data->seek(4, SEEK_CUR);
-	}
-
-	data->seek(4, SEEK_CUR);
-
-
-}
-void Model::loadBinary(Common::SeekableReadStream *data, CMap *cmap) {
+void Model::loadBinary(Common::SeekableReadStream *data) {
 	char v3[4 * 3], f[4];
 	_numMaterials = data->readUint32LE();
 	_materials = new Material*[_numMaterials];
@@ -148,8 +110,8 @@ void Model::loadBinary(Common::SeekableReadStream *data, CMap *cmap) {
 	for (int i = 0; i < _numMaterials; i++) {
 		data->read(_materialNames[i], 32);
 		_materialsShared[i] = false;
-		_materials[i] = NULL;
-		loadMaterial(i, cmap);
+		_materials[i] = nullptr;
+		loadMaterial(i, _cmap);
 	}
 	data->seek(32, SEEK_CUR); // skip name
 	data->seek(4, SEEK_CUR);
@@ -167,10 +129,10 @@ void Model::loadBinary(Common::SeekableReadStream *data, CMap *cmap) {
 	_radius = get_float(f);
 	data->seek(36, SEEK_CUR);
 	data->read(v3, 3 * 4);
-	_insertOffset = Math::Vector3d::get_vector3d(v3);
+	_insertOffset = Math::Vector3d::getVector3d(v3);
 }
 
-void Model::loadText(TextSplitter *ts, CMap *cmap) {
+void Model::loadText(TextSplitter *ts) {
 	ts->expectString("section: header");
 	int major, minor;
 	ts->scanString("3do %d.%d", 2, &major, &minor);
@@ -183,11 +145,11 @@ void Model::loadText(TextSplitter *ts, CMap *cmap) {
 		char materialName[32];
 		int num;
 		_materialsShared[i] = false;
-		_materials[i] = NULL;
+		_materials[i] = nullptr;
 
 		ts->scanString("%d: %32s", 2, &num, materialName);
 		strcpy(_materialNames[num], materialName);
-		loadMaterial(num, cmap);
+		loadMaterial(num, _cmap);
 	}
 
 	ts->expectString("section: geometrydef");
@@ -215,34 +177,34 @@ void Model::loadText(TextSplitter *ts, CMap *cmap) {
 		_rootHierNode[num]._flags = (int)flags;
 		_rootHierNode[num]._type = (int)type;
 		if (mesh < 0)
-			_rootHierNode[num]._mesh = NULL;
+			_rootHierNode[num]._mesh = nullptr;
 		else
 			_rootHierNode[num]._mesh = &_geosets[0]._meshes[mesh];
 		if (parent >= 0) {
 			_rootHierNode[num]._parent = &_rootHierNode[parent];
 			_rootHierNode[num]._depth = _rootHierNode[parent]._depth + 1;
 		} else {
-			_rootHierNode[num]._parent = NULL;
+			_rootHierNode[num]._parent = nullptr;
 			_rootHierNode[num]._depth = 0;
 		}
 		if (child >= 0)
 			_rootHierNode[num]._child = &_rootHierNode[child];
 		else
-			_rootHierNode[num]._child = NULL;
+			_rootHierNode[num]._child = nullptr;
 		if (sibling >= 0)
 			_rootHierNode[num]._sibling = &_rootHierNode[sibling];
 		else
-			_rootHierNode[num]._sibling = NULL;
+			_rootHierNode[num]._sibling = nullptr;
 
 		_rootHierNode[num]._numChildren = numChildren;
 		_rootHierNode[num]._pos = Math::Vector3d(x, y, z);
-		_rootHierNode[num]._pitch = pitch;
-		_rootHierNode[num]._yaw = yaw;
-		_rootHierNode[num]._roll = roll;
+		_rootHierNode[num]._rot = Math::Quaternion::fromEuler(yaw, pitch, roll, Math::EO_ZXY);
+		_rootHierNode[num]._animRot = _rootHierNode[num]._rot;
+		_rootHierNode[num]._animPos = _rootHierNode[num]._pos;
 		_rootHierNode[num]._pivot = Math::Vector3d(pivotx, pivoty, pivotz);
 		_rootHierNode[num]._meshVisible = true;
 		_rootHierNode[num]._hierVisible = true;
-		_rootHierNode[num]._sprite = NULL;
+		_rootHierNode[num]._sprite = nullptr;
 		_rootHierNode[num]._initialized = true;
 	}
 
@@ -269,11 +231,11 @@ void Model::reload(CMap *cmap) {
 }
 
 void Model::loadMaterial(int index, CMap *cmap) {
-	Material *mat = NULL;
+	Material *mat = nullptr;
 	if (!_materialsShared[index]) {
 		mat = _materials[index];
 	}
-	_materials[index] = NULL;
+	_materials[index] = nullptr;
 	if (_parent) {
 		_materials[index] = _parent->findMaterial(_materialNames[index], cmap);
 		if (_materials[index]) {
@@ -284,7 +246,7 @@ void Model::loadMaterial(int index, CMap *cmap) {
 		if (mat && cmap->getFilename() == _cmap->getFilename()) {
 			_materials[index] = mat;
 		} else {
-			_materials[index] = g_resourceloader->loadMaterial(_materialNames[index], cmap);
+			_materials[index] = g_resourceloader->loadMaterial(_materialNames[index], cmap, false);
 		}
 		_materialsShared[index] = false;
 	}
@@ -302,7 +264,7 @@ Material *Model::findMaterial(const char *name, CMap *cmap) const {
 		}
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 /**
@@ -337,11 +299,23 @@ void Model::Geoset::changeMaterials(Material *materials[]) {
 /**
  * @class MeshFace
  */
+MeshFace::MeshFace() :
+		_material(nullptr), _type(0), _geo(0), _light(0), _tex(0),
+		_extraLight(0), _numVertices(0), _vertices(nullptr),
+		_texVertices(nullptr), _userData(nullptr) {
+}
+
 MeshFace::~MeshFace() {
 	delete[] _vertices;
 	delete[] _texVertices;
 }
 
+void MeshFace::stealData(MeshFace &other) {
+	*this = other;
+	other._vertices = nullptr;
+	other._texVertices = nullptr;
+}
+
 int MeshFace::loadBinary(Common::SeekableReadStream *data, Material *materials[]) {
 	char v3[4 * 3], f[4];
 	data->seek(4, SEEK_CUR);
@@ -358,37 +332,62 @@ int MeshFace::loadBinary(Common::SeekableReadStream *data, Material *materials[]
 	_extraLight = get_float(f);
 	data->seek(12, SEEK_CUR);
 	data->read(v3, 4 * 3);
-	_normal = Math::Vector3d::get_vector3d(v3);
+	_normal = Math::Vector3d::getVector3d(v3);
 
 	_vertices = new int[_numVertices];
-	for (int i = 0; i < _numVertices; i++)
+	for (int i = 0; i < _numVertices; i++) {
 		_vertices[i] = data->readUint32LE();
-	if (texPtr == 0)
-		_texVertices = NULL;
-	else {
-		_texVertices = new int[_numVertices];
-		for (int i = 0; i < _numVertices; i++)
-			_texVertices[i] = data->readUint32LE();
 	}
-	if (materialPtr == 0)
-		_material = 0;
-	else {
+
+	if (texPtr != 0) {
+		_texVertices = new int[_numVertices];
+		for (int i = 0; i < _numVertices; i++) {
+			_texVertices[i] = data->readUint32LE();
+		}
+	}
+
+	if (materialPtr != 0) {
 		materialPtr = data->readUint32LE();
 		_material = materials[materialPtr];
 	}
+
 	return materialPtr;
 }
 
+int MeshFace::loadText(TextSplitter *ts, Material *materials[], int offset) {
+	int readlen, materialid;
+
+	if (ts->isEof())
+		error("Expected face data, got EOF");
+
+	ts->scanStringAtOffsetNoNewLine(offset, "%d %x %d %d %d %f %d%n", 7, &materialid, &_type, &_geo, &_light, &_tex, &_extraLight, &_numVertices, &readlen);
+	readlen += offset;
+
+	assert(materialid != -1);
+	_material = materials[materialid];
+	_vertices = new int[_numVertices];
+	_texVertices = new int[_numVertices];
+	for (int i = 0; i < _numVertices; ++i) {
+		int readlen2;
+
+		ts->scanStringAtOffsetNoNewLine(readlen, " %d, %d%n", 2, &_vertices[i], &_texVertices[i], &readlen2);
+		readlen += readlen2;
+	}
+	ts->nextLine();
+
+	return materialid;
+}
+
 void MeshFace::changeMaterial(Material *material) {
 	_material = material;
 }
 
-void MeshFace::draw(float *vertices, float *vertNormals, float *textureVerts) const {
+void MeshFace::draw(const Mesh *mesh) const {
 	if (_light == 0 && !g_driver->isShadowModeActive())
 		g_driver->disableLights();
 
 	_material->select();
-	g_driver->drawModelFace(this, vertices, vertNormals, textureVerts);
+	g_driver->drawModelFace(mesh, this);
 
 	if (_light == 0 && !g_driver->isShadowModeActive())
 		g_driver->enableLights();
@@ -397,7 +396,19 @@ void MeshFace::draw(float *vertices, float *vertNormals, float *textureVerts) co
 /**
  * @class Mesh
  */
+Mesh::Mesh() :
+		_numFaces(0), _radius(0.0f), _shadow(0), _geometryMode(0),
+		_lightingMode(0), _textureMode(0), _numVertices(0), _materialid(nullptr),
+		_vertices(nullptr), _verticesI(nullptr), _vertNormals(nullptr),
+		_numTextureVerts(0), _textureVerts(nullptr), _faces(nullptr), _userData(nullptr) {
+	_name[0] = '\0';
+
+}
+
+
 Mesh::~Mesh() {
+	g_driver->destroyMesh(this);
+
 	delete[] _vertices;
 	delete[] _verticesI;
 	delete[] _vertNormals;
@@ -446,9 +457,10 @@ void Mesh::loadBinary(Common::SeekableReadStream *data, Material *materials[]) {
 	data->read(f, 4);
 	_radius = get_float(f);
 	data->seek(24, SEEK_CUR);
+	sortFaces();
 }
 
-void Mesh::loadText(TextSplitter *ts, Material* materials[]) {
+void Mesh::loadText(TextSplitter *ts, Material *materials[]) {
 	ts->scanString("name %32s", 1, _name);
 	ts->scanString("radius %f", 1, &_radius);
 
@@ -500,34 +512,9 @@ void Mesh::loadText(TextSplitter *ts, Material* materials[]) {
 	_faces = new MeshFace[_numFaces];
 	_materialid = new int[_numFaces];
 	for (int i = 0; i < _numFaces; i++) {
-		int num, materialid, geo, light, tex, verts;
-		unsigned int type;
-		float extralight;
-		int readlen;
-
-		if (ts->isEof())
-			error("Expected face data, got EOF");
-
-		ts->scanStringNoNewLine(" %d: %d %x %d %d %d %f %d%n", 8, &num, &materialid, &type, &geo, &light, &tex, &extralight, &verts, &readlen);
-
-		assert(materialid != -1);
-		_materialid[num] = materialid;
-		_faces[num]._material = materials[materialid];
-		_faces[num]._type = (int)type;
-		_faces[num]._geo = geo;
-		_faces[num]._light = light;
-		_faces[num]._tex = tex;
-		_faces[num]._extraLight = extralight;
-		_faces[num]._numVertices = verts;
-		_faces[num]._vertices = new int[verts];
-		_faces[num]._texVertices = new int[verts];
-		for (int j = 0; j < verts; j++) {
-			int readlen2;
-
-			ts->scanStringAtOffsetNoNewLine(readlen, " %d, %d%n", 2, &_faces[num]._vertices[j], &_faces[num]._texVertices[j], &readlen2);
-			readlen += readlen2;
-		}
-		ts->nextLine();
+		int num, readlen;
+		ts->scanStringNoNewLine(" %d:%n ", 1, &num, &readlen);
+		_materialid[num] = _faces[num].loadText(ts, materials, readlen);
 	}
 
 	ts->expectString("face normals");
@@ -535,8 +522,40 @@ void Mesh::loadText(TextSplitter *ts, Material* materials[]) {
 		int num;
 		float x, y, z;
 		ts->scanString(" %d: %f %f %f", 4, &num, &x, &y, &z);
-		_faces[num]._normal = Math::Vector3d(x, y, z);
+		_faces[num].setNormal(Math::Vector3d(x, y, z));
 	}
+	sortFaces();
+}
+
+void Mesh::sortFaces() {
+	if (_numFaces < 2)
+		return;
+
+	MeshFace *newFaces = new MeshFace[_numFaces];
+	int *newMaterialid = new int[_numFaces];
+	bool *copied = new bool[_numFaces];
+	for (int i = 0; i < _numFaces; ++i)
+		copied[i] = false;
+
+	for (int cur = 0, writeIdx = 0; cur < _numFaces; ++cur) {
+		if (copied[cur])
+			continue;
+
+		for (int other = cur; other < _numFaces; ++other) {
+			if (_faces[cur].getMaterial() == _faces[other].getMaterial() && !copied[other]) {
+				copied[other] = true;
+				newFaces[writeIdx].stealData(_faces[other]);
+				newMaterialid[writeIdx] = _materialid[other];
+				writeIdx++;
+			}
+		}
+	}
+
+	delete[] _faces;
+	_faces = newFaces;
+	delete[] _materialid;
+	_materialid = newMaterialid;
+	delete[] copied;
 }
 
 void Mesh::update() {
@@ -551,8 +570,7 @@ void Mesh::draw() const {
 	if (_lightingMode == 0)
 		g_driver->disableLights();
 
-	for (int i = 0; i < _numFaces; i++)
-		_faces[i].draw(_vertices, _vertNormals, _textureVerts);
+	g_driver->drawMesh(this);
 
 	if (_lightingMode == 0)
 		g_driver->enableLights();
@@ -560,7 +578,7 @@ void Mesh::draw() const {
 
 void Mesh::getBoundingBox(int *x1, int *y1, int *x2, int *y2) const {
 	int winX1, winY1, winX2, winY2;
-	g_driver->getBoundingBoxPos(this, &winX1, &winY1, &winX2, &winY2);
+	g_driver->getScreenBoundingBox(this, &winX1, &winY1, &winX2, &winY2);
 	if (winX1 != -1 && winY1 != -1 && winX2 != -1 && winY2 != -1) {
 		*x1 = MIN(*x1, winX1);
 		*y1 = MIN(*y1, winY1);
@@ -572,10 +590,17 @@ void Mesh::getBoundingBox(int *x1, int *y1, int *x2, int *y2) const {
 /**
  * @class ModelNode
  */
+ModelNode::ModelNode() :
+		_initialized(false), _needsUpdate(true), _mesh(nullptr), _flags(0), _type(0),
+		_depth(0), _numChildren(0), _parent(nullptr), _child(nullptr), _sprite(nullptr),
+		_sibling(nullptr), _meshVisible(false), _hierVisible(false) {
+	_name[0] = '\0';
+}
+
 ModelNode::~ModelNode() {
 	ModelNode *child = _child;
 	while (child) {
-		child->_parent = NULL;
+		child->_parent = nullptr;
 		child = child->_sibling;
 	}
 }
@@ -588,7 +613,7 @@ void ModelNode::loadBinary(Common::SeekableReadStream *data, ModelNode *hierNode
 	_type = data->readUint32LE();
 	int meshNum = data->readUint32LE();
 	if (meshNum < 0)
-		_mesh = NULL;
+		_mesh = nullptr;
 	else
 		_mesh = g->_meshes + meshNum;
 	_depth = data->readUint32LE();
@@ -597,37 +622,36 @@ void ModelNode::loadBinary(Common::SeekableReadStream *data, ModelNode *hierNode
 	int childPtr = data->readUint32LE();
 	int siblingPtr = data->readUint32LE();
 	data->read(v3, 4 * 3);
-	_pivot = Math::Vector3d::get_vector3d(v3);
+	_pivot = Math::Vector3d::getVector3d(v3);
 	data->read(v3, 4 * 3);
-	_pos = Math::Vector3d::get_vector3d(v3);
+	_pos = Math::Vector3d::getVector3d(v3);
 	data->read(f, 4);
-	_pitch = get_float(f);
+	float pitch = get_float(f);
 	data->read(f, 4);
-	_yaw = get_float(f);
+	float yaw = get_float(f);
 	data->read(f, 4);
-	_roll = get_float(f);
-	_animPos.set(0,0,0);
-	_animPitch = 0;
-	_animYaw = 0;
-	_animRoll = 0;
-	_sprite = NULL;
+	float roll = get_float(f);
+	_rot = Math::Quaternion::fromEuler(yaw, pitch, roll, Math::EO_ZXY);
+	_animRot = _rot;
+	_animPos = _pos;
+	_sprite = nullptr;
 
 	data->seek(48, SEEK_CUR);
 
 	if (parentPtr != 0)
 		_parent = hierNodes + data->readUint32LE();
 	else
-		_parent = NULL;
+		_parent = nullptr;
 
 	if (childPtr != 0)
 		_child = hierNodes + data->readUint32LE();
 	else
-		_child = NULL;
+		_child = nullptr;
 
 	if (siblingPtr != 0)
 		_sibling = hierNodes + data->readUint32LE();
 	else
-		_sibling = NULL;
+		_sibling = nullptr;
 
 	_meshVisible = true;
 	_hierVisible = true;
@@ -702,7 +726,7 @@ void ModelNode::removeChild(ModelNode *child) {
 		childPos = &(*childPos)->_sibling;
 	if (*childPos) {
 		*childPos = child->_sibling;
-		child->_parent = NULL;
+		child->_parent = nullptr;
 	}
 }
 
@@ -717,13 +741,8 @@ void ModelNode::update() {
 		return;
 
 	if (_hierVisible && _needsUpdate) {
-		Math::Vector3d animPos = _pos + _animPos;
-		Math::Angle animPitch = _pitch + _animPitch;
-		Math::Angle animYaw = _yaw + _animYaw;
-		Math::Angle animRoll = _roll + _animRoll;
-
-		_localMatrix.setPosition(animPos);
-		_localMatrix.buildFromPitchYawRoll(animPitch, animYaw, animRoll);
+		_localMatrix = _animRot.toMatrix();
+		_localMatrix.setPosition(_animPos);
 
 		_matrix = _matrix * _localMatrix;
 
@@ -752,9 +771,9 @@ void ModelNode::addSprite(Sprite *sprite) {
 	_sprite = sprite;
 }
 
-void ModelNode::removeSprite(Sprite *sprite) {
-	Sprite* curr = _sprite;
-	Sprite* prev = NULL;
+void ModelNode::removeSprite(const Sprite *sprite) {
+	Sprite *curr = _sprite;
+	Sprite *prev = nullptr;
 	while (curr) {
 		if (curr == sprite) {
 			if (prev)
@@ -768,16 +787,13 @@ void ModelNode::removeSprite(Sprite *sprite) {
 }
 
 void ModelNode::translateViewpoint() const {
-	Math::Vector3d animPos = _pos + _animPos;
-	Math::Angle animPitch = _pitch + _animPitch;
-	Math::Angle animYaw = _yaw + _animYaw;
-	Math::Angle animRoll = _roll + _animRoll;
 	g_driver->translateViewpointStart();
 
-	g_driver->translateViewpoint(animPos);
-	g_driver->rotateViewpoint(animYaw, Math::Vector3d(0, 0, 1));
-	g_driver->rotateViewpoint(animPitch, Math::Vector3d(1, 0, 0));
-	g_driver->rotateViewpoint(animRoll, Math::Vector3d(0, 1, 0));
+	g_driver->translateViewpoint(_animPos);
+
+	Math::Matrix4 rot = _animRot.toMatrix();
+	rot.transpose();
+	g_driver->rotateViewpoint(rot);
 }
 
 void ModelNode::translateViewpointBack() const {
