@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -161,7 +161,7 @@ void Blocky16::makeTablesInterpolation(int param) {
 	int32 b1, b2;
 	int32 value_table47_1_2, value_table47_1_1, value_table47_2_2, value_table47_2_1;
 	int32 tableSmallBig[64], tmp, s;
-	int8 *table47_1 = 0, *table47_2 = 0;
+	int8 *table47_1 = nullptr, *table47_2 = nullptr;
 	int32 *ptr_small_big;
 	byte *ptr;
 	int i, x, y;
@@ -276,7 +276,7 @@ void Blocky16::makeTablesInterpolation(int param) {
 						}
 					}
 				} else if ((b1 == 0 && b2 == 1) || (b2 == 0 && b1 == 1) ||
-				           (b1 == 3 && b2 != 2) || (b2 == 3 && b1 != 2)) {
+						   (b1 == 3 && b2 != 2) || (b2 == 3 && b1 != 2)) {
 					if (param > variable4) {
 						i = param - variable4;
 						while (i--) {
@@ -630,9 +630,12 @@ void Blocky16::init(int width, int height) {
 	// of using _frameSize.
 	int size = _blocksWidth * 8 * _blocksHeight * 8 * 2;
 	_offset = size - _frameSize;
-	_deltaSize = size * 3;
-	_deltaBuf = new byte[_deltaSize];
-	memset(_deltaBuf, 0, _deltaSize);
+	// workaround for read over buffer by increasing buffer
+	// 200 bytes is enough for smush anims:
+	// lol, byeruba, crushed, eldepot, heltrain, hostage
+	uint32 deltaSize = size * 3 + 200;
+	_deltaBuf = new byte[deltaSize];
+	memset(_deltaBuf, 0, deltaSize);
 	_deltaBufs[0] = _deltaBuf;
 	_deltaBufs[1] = _deltaBuf + _frameSize;
 	_curBuf = _deltaBuf + _frameSize * 2;
@@ -643,17 +646,29 @@ Blocky16::Blocky16() {
 	_tableSmall = new byte[32768];
 	memset(_tableBig, 0, 99328);
 	memset(_tableSmall, 0, 32768);
-	_deltaBuf = NULL;
+	_deltaBuf = nullptr;
+	_deltaBufs[0] = nullptr;
+	_deltaBufs[1] = nullptr;
+	_curBuf = nullptr;
+	_prevSeqNb = 0;
+	_lastTableWidth = 0;
+	_d_src = nullptr;
+	_paramPtr = nullptr;
+	_param6_7Ptr = nullptr;
+	_blocksHeight = _blocksWidth = 0;
+	_height = _width = 0;
+	_offset = _offset1 = _offset2 = 0;
+	_frameSize = 0;
+	_d_pitch = 0;
 }
 
 void Blocky16::deinit() {
 	_lastTableWidth = -1;
 	if (_deltaBuf) {
 		delete[] _deltaBuf;
-		_deltaSize = 0;
-		_deltaBuf = NULL;
-		_deltaBufs[0] = NULL;
-		_deltaBufs[1] = NULL;
+		_deltaBuf = nullptr;
+		_deltaBufs[0] = nullptr;
+		_deltaBufs[1] = nullptr;
 	}
 }
 
@@ -661,11 +676,11 @@ Blocky16::~Blocky16() {
 	deinit();
 	if (_tableBig) {
 		delete[] _tableBig;
-		_tableBig = NULL;
+		_tableBig = nullptr;
 	}
 	if (_tableSmall) {
 		delete[] _tableSmall;
-		_tableSmall = NULL;
+		_tableSmall = nullptr;
 	}
 }
 
@@ -759,7 +774,7 @@ void Blocky16::decode(byte *dst, const byte *src) {
 		_prevSeqNb = -1;
 	}
 
-	switch(src[18]) {
+	switch (src[18]) {
 	case 0:
 #if defined(SCUMM_BIG_ENDIAN)
 		for (int i = 0; i < _width * _height; i++) {
@@ -816,7 +831,7 @@ void Blocky16::decode(byte *dst, const byte *src) {
 	memcpy(dst, _curBuf, _frameSize);
 
 	if (seq_nb == _prevSeqNb + 1) {
-		byte *tmp_ptr = NULL;
+		byte *tmp_ptr = nullptr;
 		if (src[19] == 1) {
 			tmp_ptr = _curBuf;
 			_curBuf = _deltaBufs[1];
