@@ -1,19 +1,19 @@
 /* ResidualVM - A 3D game interpreter
  *
  * ResidualVM is the legal property of its developers, whose names
- * are too numerous to list here. Please refer to the COPYRIGHT
+ * are too numerous to list here. Please refer to the AUTHORS
  * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -34,6 +34,7 @@
 #include "engines/grim/bitmap.h"
 #include "engines/grim/font.h"
 #include "engines/grim/model.h"
+#include "engines/grim/sprite.h"
 #include "engines/grim/inputdialog.h"
 #include "engines/grim/debug.h"
 #include "engines/grim/emi/animationemi.h"
@@ -41,7 +42,9 @@
 #include "engines/grim/emi/modelemi.h"
 #include "engines/grim/emi/skeleton.h"
 #include "engines/grim/patchr.h"
+#include "engines/grim/md5check.h"
 #include "engines/grim/update/update.h"
+
 #include "common/algorithm.h"
 #include "common/zlib.h"
 #include "common/memstream.h"
@@ -50,7 +53,7 @@
 
 namespace Grim {
 
-ResourceLoader *g_resourceloader = NULL;
+ResourceLoader *g_resourceloader = nullptr;
 
 class LabListComperator {
 	const Common::String _labName;
@@ -70,10 +73,42 @@ public:
 ResourceLoader::ResourceLoader() {
 	_cacheDirty = false;
 	_cacheMemorySize = 0;
+	_localArchive = nullptr;
 
 	Lab *l;
 	Common::ArchiveMemberList files, updFiles;
 
+	//Load the update from the executable, if needed
+	const char *updateFilename = g_grim->getUpdateFilename();
+	if (updateFilename) {
+		Common::File *updStream = new Common::File();
+		if (updStream && updStream->open(updateFilename)) {
+			Common::Archive *update = loadUpdateArchive(updStream);
+			if (update)
+				SearchMan.add("update", update, 1);
+		} else
+			delete updStream;
+
+		// Check if the update has been correctly loaded
+		if (!SearchMan.hasArchive("update")) {
+			const char *errorMessage = nullptr;
+			if (g_grim->getGameType() == GType_GRIM) {
+				errorMessage =  "The original patch of Grim Fandango\n"
+								"is missing. Please download it from\n"
+								"http://www.residualvm.org/downloads/\n"
+								"and put it in the game data files directory";
+			} else if (g_grim->getGameType() == GType_MONKEY4) {
+				errorMessage =  "The original patch of Escape from Monkey Island is missing. \n"
+								"Please download it from http://www.residualvm.org/downloads/\n"
+								"and put it in the game data files directory.\n"
+								"Pay attention to download the correct version according to the game's language";
+			}
+
+			GUI::displayErrorDialog(errorMessage);
+			error("%s not found", updateFilename);
+		}
+	}
+
 	if (g_grim->getGameType() == GType_GRIM) {
 		if (g_grim->getGameFlags() & ADGF_DEMO) {
 			SearchMan.listMatchingMembers(files, "gfdemo01.lab");
@@ -81,20 +116,10 @@ ResourceLoader::ResourceLoader() {
 			SearchMan.listMatchingMembers(files, "sound001.lab");
 			SearchMan.listMatchingMembers(files, "voice001.lab");
 		} else {
-			//Load the update from the executable
-			Common::File *updStream = new Common::File();
-			if (updStream && updStream->open("gfupd101.exe")) {
-				Common::Archive *update = loadUpdateArchive(updStream);
-				if (update)
-					SearchMan.add("update", update, 1);
-			} else
-				delete updStream;
-
 			if (!SearchMan.hasFile("residualvm-grim-patch.lab"))
 				error("residualvm-grim-patch.lab not found");
 
 			SearchMan.listMatchingMembers(files, "residualvm-grim-patch.lab");
-			SearchMan.listMatchingMembers(files, "datausr.lab");
 			SearchMan.listMatchingMembers(files, "data005.lab");
 			SearchMan.listMatchingMembers(files, "data004.lab");
 			SearchMan.listMatchingMembers(files, "data003.lab");
@@ -112,68 +137,60 @@ ResourceLoader::ResourceLoader() {
 
 			//Check the presence of datausr.lab and if the user wants to load it.
 			//In this case put it in the top of the list
-			Common::ArchiveMemberList::iterator datausr_it = Common::find_if(files.begin(), files.end(), LabListComperator("datausr.lab"));
-			if (datausr_it != files.end()) {
-				if (ConfMan.getBool("datausr_load")) {
-					warning("Loading datausr.lab. Please note that the ResidualVM-team doesn't provide support for using such patches");
-					files.push_front(*datausr_it);
-				}
-				files.erase(datausr_it);
+			const char *datausr_name = "datausr.lab";
+			if (SearchMan.hasFile(datausr_name) && ConfMan.getBool("datausr_load")) {
+				warning("Loading datausr.lab. Please note that the ResidualVM-team doesn't provide support for using such patches");
+				files.push_front(SearchMan.getMember(datausr_name));
 			}
 		}
 	} else if (g_grim->getGameType() == GType_MONKEY4) {
-		if (g_grim->getGameFlags() == ADGF_DEMO) {
+		const char *emi_patches_filename = "residualvm-emi-patch.m4b";
+		if (!SearchMan.hasFile(emi_patches_filename))
+			error("%s not found", emi_patches_filename);
+
+		SearchMan.listMatchingMembers(files, emi_patches_filename);
+
+		// This is neccessary to speed up the launch of the mac version,
+		// we _COULD_ protect this with a platform check, but the file isn't
+		// really big anyhow...
+		Lab *localLab = new Lab();
+		if (localLab->open("local.m4b", true)) {
+			_localArchive = localLab;
+		} else {
+			delete localLab;
+		}
+
+		if (g_grim->getGameFlags() & ADGF_DEMO) {
 			SearchMan.listMatchingMembers(files, "i9n.lab");
 			SearchMan.listMatchingMembers(files, "lip.lab");
 			SearchMan.listMatchingMembers(files, "MagDemo.lab");
 			SearchMan.listMatchingMembers(files, "tile.lab");
 			SearchMan.listMatchingMembers(files, "voice.lab");
 		} else {
-			if (g_grim->getGamePlatform() == Common::kPlatformWindows) {
-				//Load the update from the executable
-				SearchMan.listMatchingMembers(updFiles, "MonkeyUpdate.exe");
-				SearchMan.listMatchingMembers(updFiles, "MonkeyUpdate_???.exe");
-				for (Common::ArchiveMemberList::const_iterator x = updFiles.begin(); x != updFiles.end(); ++x) {
-					Common::SeekableReadStream *updStream;
-					updStream = (*x)->createReadStream();
 
-					Common::Archive *update = loadUpdateArchive(updStream);
-					if (update)
-						SearchMan.add("update", update, 1);
-				}
-			}
-
-			SearchMan.listMatchingMembers(files, "patch.m4b");
+			//Keep i9n.m4b before patch.m4b for a better efficiency
+			//in decompressing from Monkey Update.exe
 			SearchMan.listMatchingMembers(files, "i9n.m4b");
+			SearchMan.listMatchingMembers(files, "patch.m4b");
 			SearchMan.listMatchingMembers(files, "art???.m4b");
 			SearchMan.listMatchingMembers(files, "lip.m4b");
 			SearchMan.listMatchingMembers(files, "local.m4b");
 			SearchMan.listMatchingMembers(files, "sfx.m4b");
 			SearchMan.listMatchingMembers(files, "voice???.m4b");
 			SearchMan.listMatchingMembers(files, "music?.m4b");
-		}
-	}
 
-	// Check if the update has correctly loaded
-	if (!(g_grim->getGameFlags() & ADGF_DEMO || SearchMan.hasArchive("update"))) {
-		const char *errorMessage = 0;
-		if (g_grim->getGameType() == GType_GRIM) {
-			errorMessage = 	"Unsupported version of Grim Fandango.\n"
-							"Please download the original patch from\n"
-							"http://www.residualvm.org/downloads/\n"
-							"and put it the game data files directory";
-			GUI::displayErrorDialog(errorMessage);
-			error("gfupd101.exe not found");
-		}
+			if (g_grim->getGamePlatform() == Common::kPlatformPS2) {
+				SearchMan.listMatchingMembers(files, "???.m4b");
+			}
 
-		//Don't force the update for MI4 for now
-		/*else if (g_grim->getGameType() == GType_MONKEY4)
-			errorMessage = 	"Unsupported version of Escape from Monkey Island.\n"
-							"Please download the original patch from\n"
-							"http://www.residualvm.org/downloads/\n"
-							"and put it the game data files directory.\n"
-							"Pay attention to download the correct version according to the game's language";
-		*/
+			//Check the presence of datausr.m4b and if the user wants to load it.
+			//In this case put it in the top of the list
+			const char *datausr_name = "datausr.m4b";
+			if (SearchMan.hasFile(datausr_name) && ConfMan.getBool("datausr_load")) {
+				warning("Loading datausr.m4b. Please note that the ResidualVM-team doesn't provide support for using such patches");
+				files.push_front(SearchMan.getMember(datausr_name));
+			}
+		}
 	}
 
 	if (files.empty())
@@ -218,6 +235,7 @@ ResourceLoader::~ResourceLoader() {
 	clearList(_colormaps);
 	clearList(_keyframeAnims);
 	clearList(_lipsyncs);
+	MD5Check::clear();
 }
 
 static int sortCallback(const void *entry1, const void *entry2) {
@@ -227,7 +245,7 @@ static int sortCallback(const void *entry1, const void *entry2) {
 Common::SeekableReadStream *ResourceLoader::getFileFromCache(const Common::String &filename) const {
 	ResourceLoader::ResourceCache *entry = getEntryFromCache(filename);
 	if (!entry)
-		return NULL;
+		return nullptr;
 
 	return new Common::MemoryReadStream(entry->resPtr, entry->len);
 
@@ -235,7 +253,7 @@ Common::SeekableReadStream *ResourceLoader::getFileFromCache(const Common::Strin
 
 ResourceLoader::ResourceCache *ResourceLoader::getEntryFromCache(const Common::String &filename) const {
 	if (_cache.empty())
-		return NULL;
+		return nullptr;
 
 	if (_cacheDirty) {
 		qsort(_cache.begin(), _cache.size(), sizeof(ResourceCache), sortCallback);
@@ -249,11 +267,13 @@ ResourceLoader::ResourceCache *ResourceLoader::getEntryFromCache(const Common::S
 }
 
 Common::SeekableReadStream *ResourceLoader::loadFile(const Common::String &filename) const {
-	Common::SeekableReadStream *rs = NULL;
-	if (SearchMan.hasFile(filename))
+	Common::SeekableReadStream *rs = nullptr;
+	if (_localArchive && _localArchive->hasFile(filename)) {
+		rs = _localArchive->createReadStreamForMember(filename);
+	} else if (SearchMan.hasFile(filename))
 		rs = SearchMan.createReadStreamForMember(filename);
 	else
-		return NULL;
+		return nullptr;
 
 	rs = wrapPatchedFile(rs, filename);
 	return rs;
@@ -261,14 +281,14 @@ Common::SeekableReadStream *ResourceLoader::loadFile(const Common::String &filen
 
 Common::SeekableReadStream *ResourceLoader::openNewStreamFile(Common::String fname, bool cache) const {
 	Common::SeekableReadStream *s;
-    fname.toLowercase();
+	fname.toLowercase();
 
 	if (cache) {
 		s = getFileFromCache(fname);
 		if (!s) {
 			s = loadFile(fname);
 			if (!s)
-				return NULL;
+				return nullptr;
 
 			uint32 size = s->size();
 			byte *buf = new byte[size];
@@ -308,7 +328,7 @@ CMap *ResourceLoader::loadColormap(const Common::String &filename) {
 	return result;
 }
 
-static Common::String fixFilename(const Common::String filename, bool append = true) {
+Common::String ResourceLoader::fixFilename(const Common::String &filename, bool append) {
 	Common::String fname(filename);
 	if (g_grim->getGameType() == GType_MONKEY4) {
 		int len = fname.size();
@@ -324,7 +344,7 @@ static Common::String fixFilename(const Common::String filename, bool append = t
 	return fname;
 }
 
-Costume *ResourceLoader::loadCostume(const Common::String &filename, Costume *prevCost) {
+Costume *ResourceLoader::loadCostume(const Common::String &filename, Actor *owner, Costume *prevCost) {
 	Common::String fname = fixFilename(filename);
 	fname.toLowercase();
 
@@ -334,13 +354,13 @@ Costume *ResourceLoader::loadCostume(const Common::String &filename, Costume *pr
 	}
 	Costume *result;
 	if (g_grim->getGameType() == GType_MONKEY4) {
-		result = new EMICostume(filename, prevCost);		
+		result = new EMICostume(filename, owner, prevCost);
 	} else {
-		result = new Costume(filename, prevCost);
+		result = new Costume(filename, owner, prevCost);
 	}
 	result->load(stream);
 	delete stream;
-	
+
 	return result;
 }
 
@@ -348,10 +368,11 @@ Font *ResourceLoader::loadFont(const Common::String &filename) {
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(filename.c_str(), true);
-	if(!stream)
+	if (!stream)
 		error("Could not find font file %s", filename.c_str());
 
-	Font *result = new Font(filename, stream);
+	Font *result = new Font();
+	result->load(filename, stream);
 	delete stream;
 
 	return result;
@@ -361,7 +382,7 @@ KeyframeAnim *ResourceLoader::loadKeyframe(const Common::String &filename) {
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(filename.c_str());
-	if(!stream)
+	if (!stream)
 		error("Could not find keyframe file %s", filename.c_str());
 
 	KeyframeAnim *result = new KeyframeAnim(filename, stream);
@@ -376,8 +397,8 @@ LipSync *ResourceLoader::loadLipSync(const Common::String &filename) {
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(filename.c_str());
-	if(!stream)
-		return NULL;
+	if (!stream)
+		return nullptr;
 
 	result = new LipSync(filename, stream);
 
@@ -386,23 +407,32 @@ LipSync *ResourceLoader::loadLipSync(const Common::String &filename) {
 		_lipsyncs.push_back(result);
 	else {
 		delete result;
-		result = NULL;
+		result = nullptr;
 	}
 	delete stream;
 
 	return result;
 }
 
-Material *ResourceLoader::loadMaterial(const Common::String &filename, CMap *c) {
+Material *ResourceLoader::loadMaterial(const Common::String &filename, CMap *c, bool clamp) {
 	Common::String fname = fixFilename(filename, false);
 	fname.toLowercase();
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(fname.c_str(), true);
-	if(!stream)
-		error("Could not find material %s", filename.c_str());
+	if (!stream && !filename.hasPrefix("specialty")) {
+		// FIXME: EMI demo references files that aren't included. Return a known material.
+		// This should be fixed in the data files instead.
+		if (g_grim->getGameType() == GType_MONKEY4 && g_grim->getGameFlags() & ADGF_DEMO) {
+			const Common::String replacement("fx/candle.sprb");
+			warning("Could not find material %s, using %s instead", filename.c_str(), replacement.c_str());
+			return loadMaterial(replacement, nullptr, clamp);
+		} else {
+			error("Could not find material %s", filename.c_str());
+		}
+	}
 
-	Material *result = new Material(fname, stream, c);
+	Material *result = new Material(fname, stream, c, clamp);
 	delete stream;
 
 	return result;
@@ -413,7 +443,7 @@ Model *ResourceLoader::loadModel(const Common::String &filename, CMap *c, Model
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(fname.c_str());
-	if(!stream)
+	if (!stream)
 		error("Could not find model %s", filename.c_str());
 
 	Model *result = new Model(filename, stream, c, parent);
@@ -423,17 +453,17 @@ Model *ResourceLoader::loadModel(const Common::String &filename, CMap *c, Model
 	return result;
 }
 
-EMIModel *ResourceLoader::loadModelEMI(const Common::String &filename, EMIModel *parent) {
+EMIModel *ResourceLoader::loadModelEMI(const Common::String &filename, EMICostume *costume) {
 	Common::String fname = fixFilename(filename);
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(fname.c_str());
-	if(!stream) {
+	if (!stream) {
 		warning("Could not find model %s", filename.c_str());
-		return NULL;
+		return nullptr;
 	}
 
-	EMIModel *result = new EMIModel(filename, stream, parent);
+	EMIModel *result = new EMIModel(filename, stream, costume);
 	_emiModels.push_back(result);
 	delete stream;
 
@@ -445,9 +475,9 @@ Skeleton *ResourceLoader::loadSkeleton(const Common::String &filename) {
 	Common::SeekableReadStream *stream;
 
 	stream = openNewStreamFile(fname.c_str(), true);
-	if(!stream) {
+	if (!stream) {
 		warning("Could not find skeleton %s", filename.c_str());
-		return NULL;
+		return nullptr;
 	}
 
 	Skeleton *result = new Skeleton(filename, stream);
@@ -456,17 +486,37 @@ Skeleton *ResourceLoader::loadSkeleton(const Common::String &filename) {
 	return result;
 }
 
+Sprite *ResourceLoader::loadSprite(const Common::String &filename, EMICostume *costume) {
+	assert(g_grim->getGameType() == GType_MONKEY4);
+	Common::SeekableReadStream *stream;
+
+	const Common::String fname = fixFilename(filename, true);
+
+	stream = openNewStreamFile(fname.c_str(), true);
+	if (!stream) {
+		warning("Could not find sprite %s", fname.c_str());
+		return nullptr;
+	}
+
+	Sprite *result = new Sprite();
+	result->loadBinary(stream, costume);
+	delete stream;
+
+	return result;
+}
+
 AnimationEmi *ResourceLoader::loadAnimationEmi(const Common::String &filename) {
 	Common::String fname = fixFilename(filename);
 	Common::SeekableReadStream *stream;
-	
+
 	stream = openNewStreamFile(fname.c_str(), true);
-	if(!stream) {
+	if (!stream) {
 		warning("Could not find animation %s", filename.c_str());
-		return NULL;
+		return nullptr;
 	}
-	
+
 	AnimationEmi *result = new AnimationEmi(filename, stream);
+	_emiAnims.push_back(result);
 	delete stream;
 
 	return result;
@@ -508,12 +558,16 @@ void ResourceLoader::uncacheLipSync(LipSync *s) {
 	_lipsyncs.remove(s);
 }
 
+void ResourceLoader::uncacheAnimationEmi(AnimationEmi *a) {
+	_emiAnims.remove(a);
+}
+
 ModelPtr ResourceLoader::getModel(const Common::String &fname, CMap *c) {
 	Common::String filename = fname;
 	filename.toLowercase();
 	for (Common::List<Model *>::const_iterator i = _models.begin(); i != _models.end(); ++i) {
 		Model *m = *i;
-		if (filename == m->_fname && *m->_cmap == *c) {
+		if (filename == m->getFilename() && *m->getCMap() == *c) {
 			return m;
 		}
 	}
@@ -560,4 +614,17 @@ LipSyncPtr ResourceLoader::getLipSync(const Common::String &fname) {
 	return loadLipSync(fname);
 }
 
+AnimationEmiPtr ResourceLoader::getAnimationEmi(const Common::String &fname) {
+	Common::String filename = fname;
+	filename.toLowercase();
+	for (Common::List<AnimationEmi *>::const_iterator i = _emiAnims.begin(); i != _emiAnims.end(); ++i) {
+		AnimationEmi *a = *i;
+		if (filename == a->getFilename()) {
+			return a;
+		}
+	}
+
+	return loadAnimationEmi(fname);
+}
+
 } // end of namespace Grim
