@@ -1,27 +1,25 @@
 /* ResidualVM - A 3D game interpreter
  *
  * ResidualVM is the legal property of its developers, whose names
- * are too numerous to list here. Please refer to the COPYRIGHT
+ * are too numerous to list here. Please refer to the AUTHORS
  * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
-#define FORBIDDEN_SYMBOL_EXCEPTION_printf
-
 #include "common/foreach.h"
 
 #include "engines/grim/debug.h"
@@ -35,26 +33,34 @@
 #include "engines/grim/gfx_base.h"
 
 #include "engines/grim/sound.h"
+#include "engines/grim/emi/sound/emisound.h"
+
+#include "math/frustum.h"
 
 namespace Grim {
 
 Set::Set(const Common::String &sceneName, Common::SeekableReadStream *data) :
-		PoolObject<Set, MKTAG('S', 'E', 'T', ' ')>(), _locked(false), _name(sceneName), _enableLights(false) {
+		_locked(false), _name(sceneName), _enableLights(false) {
 
 	char header[7];
 	data->read(header, 7);
 	data->seek(0, SEEK_SET);
 	if (memcmp(header, "section", 7) == 0) {
-		TextSplitter ts(data);
+		TextSplitter ts(_name, data);
 		loadText(ts);
 	} else {
 		loadBinary(data);
 	}
+	setupOverworldLights();
 }
 
 Set::Set() :
-	PoolObject<Set, MKTAG('S', 'E', 'T', ' ')>(), _cmaps(NULL) {
+		_cmaps(nullptr), _locked(false), _enableLights(false), _numSetups(0),
+		_numLights(0), _numSectors(0), _numObjectStates(0), _minVolume(0),
+		_maxVolume(0), _numCmaps(0), _numShadows(0), _currSetup(nullptr),
+		_setups(nullptr), _lights(nullptr), _sectors(nullptr), _shadows(nullptr) {
 
+	setupOverworldLights();
 }
 
 Set::~Set() {
@@ -71,9 +77,40 @@ Set::~Set() {
 			delete _sectors[i];
 		}
 		delete[] _sectors;
-		foreach (ObjectState *s, _states)
+		while (!_states.empty()) {
+			ObjectState *s = _states.front();
+			_states.pop_front();
 			delete s;
+		}
+		delete[] _shadows;
 	}
+	foreach (Light *l, _overworldLightsList) {
+		delete l;
+	}
+}
+
+void Set::setupOverworldLights() {
+	Light *l;
+
+	l = new Light();
+	l->_name = "Overworld Light 1";
+	l->_enabled = true;
+	l->_type = Light::Ambient;
+	l->_pos = Math::Vector3d(0, 0, 0);
+	l->_dir = Math::Vector3d(0, 0, 0);
+	l->_color = Color(255, 255, 255);
+	l->_intensity = 0.5f;
+	_overworldLightsList.push_back(l);
+
+	l = new Light();
+	l->_name = "Overworld Light 2";
+	l->_enabled = true;
+	l->_type = Light::Direct;
+	l->_pos = Math::Vector3d(0, 0, 0);
+	l->_dir = Math::Vector3d(0, 0, -1);
+	l->_color = Color(255, 255, 255);
+	l->_intensity = 0.6f;
+	_overworldLightsList.push_back(l);
 }
 
 void Set::loadText(TextSplitter &ts) {
@@ -106,10 +143,12 @@ void Set::loadText(TextSplitter &ts) {
 		_setups[i].load(this, i, ts);
 	_currSetup = _setups;
 
+	_numShadows = 0;
 	_numSectors = -1;
 	_numLights = -1;
-	_lights = NULL;
-	_sectors = NULL;
+	_lights = nullptr;
+	_sectors = nullptr;
+	_shadows = nullptr;
 
 	_minVolume = 0;
 	_maxVolume = 0;
@@ -129,7 +168,7 @@ void Set::loadText(TextSplitter &ts) {
 
 	// Calculate the number of sectors
 	ts.expectString("section: sectors");
-	if (ts.isEof())	// Sectors are optional, but section: doesn't seem to be
+	if (ts.isEof()) // Sectors are optional, but section: doesn't seem to be
 		return;
 
 	int sectorStart = ts.getLineNumber();
@@ -157,7 +196,7 @@ void Set::loadText(TextSplitter &ts) {
 
 void Set::loadBinary(Common::SeekableReadStream *data) {
 	// yes, an array of size 0
-	_cmaps = NULL;//new CMapPtr[0];
+	_cmaps = nullptr;//new CMapPtr[0];
 
 
 	_numSetups = data->readUint32LE();
@@ -168,8 +207,9 @@ void Set::loadBinary(Common::SeekableReadStream *data) {
 
 	_numSectors = 0;
 	_numLights = 0;
-	_lights = NULL;
-	_sectors = NULL;
+	_lights = nullptr;
+	_sectors = nullptr;
+	_shadows = nullptr;
 
 	_minVolume = 0;
 	_maxVolume = 0;
@@ -184,9 +224,6 @@ void Set::loadBinary(Common::SeekableReadStream *data) {
 		_lightsList.push_back(&_lights[i]);
 	}
 
-	// bypass light stuff for now
-	_numLights = 0;
-
 	_numSectors = data->readUint32LE();
 	// Allocate and fill an array of sector info
 	_sectors = new Sector*[_numSectors];
@@ -194,13 +231,25 @@ void Set::loadBinary(Common::SeekableReadStream *data) {
 		_sectors[i] = new Sector();
 		_sectors[i]->loadBinary(data);
 	}
+
+	_numShadows = data->readUint32LE();
+	_shadows = new SetShadow[_numShadows];
+
+	for (int i = 0; i < _numShadows; ++i) {
+		_shadows[i].loadBinary(data, this);
+	}
+
+	// Enable lights by default
+	_enableLights = true;
 }
 
 void Set::saveState(SaveGame *savedState) const {
 	savedState->writeString(_name);
-	savedState->writeLESint32(_numCmaps);
-	for (int i = 0; i < _numCmaps; ++i) {
-		savedState->writeString(_cmaps[i]->getFilename());
+	if (g_grim->getGameType() == GType_GRIM) {
+		savedState->writeLESint32(_numCmaps);
+		for (int i = 0; i < _numCmaps; ++i) {
+			savedState->writeString(_cmaps[i]->getFilename());
+		}
 	}
 	savedState->writeLEUint32((uint32)(_currSetup - _setups)); // current setup id
 	savedState->writeBool(_locked);
@@ -230,15 +279,23 @@ void Set::saveState(SaveGame *savedState) const {
 	for (int i = 0; i < _numLights; ++i) {
 		_lights[i].saveState(savedState);
 	}
+
+	//Shadows
+	savedState->writeLESint32(_numShadows);
+	for (int i = 0; i < _numShadows; ++i) {
+		_shadows[i].saveState(savedState);
+	}
 }
 
 bool Set::restoreState(SaveGame *savedState) {
 	_name = savedState->readString();
-	_numCmaps = savedState->readLESint32();
-	_cmaps = new CMapPtr[_numCmaps];
-	for (int i = 0; i < _numCmaps; ++i) {
-		Common::String str = savedState->readString();
-		_cmaps[i] = g_resourceloader->getColormap(str);
+	if (g_grim->getGameType() == GType_GRIM) {
+		_numCmaps = savedState->readLESint32();
+		_cmaps = new CMapPtr[_numCmaps];
+		for (int i = 0; i < _numCmaps; ++i) {
+			Common::String str = savedState->readString();
+			_cmaps[i] = g_resourceloader->getColormap(str);
+		}
 	}
 
 	int32 currSetupId = savedState->readLEUint32();
@@ -263,7 +320,7 @@ bool Set::restoreState(SaveGame *savedState) {
 		_setups[i].restoreState(savedState);
 	}
 
-    //Sectors
+	//Sectors
 	_numSectors = savedState->readLESint32();
 	if (_numSectors > 0) {
 		_sectors = new Sector*[_numSectors];
@@ -272,7 +329,7 @@ bool Set::restoreState(SaveGame *savedState) {
 			_sectors[i]->restoreState(savedState);
 		}
 	} else {
-		_sectors = NULL;
+		_sectors = nullptr;
 	}
 
 	_numLights = savedState->readLESint32();
@@ -283,6 +340,14 @@ bool Set::restoreState(SaveGame *savedState) {
 		_lightsList.push_back(&_lights[i]);
 	}
 
+	if (savedState->saveMinorVersion() >= 19) {
+		_numShadows = savedState->readLESint32();
+		_shadows = new SetShadow[_numShadows];
+		for (int i = 0; i < _numShadows; ++i) {
+			_shadows[i].restoreState(savedState);
+		}
+	}
+
 	return true;
 }
 
@@ -293,17 +358,10 @@ void Set::Setup::load(Set *set, int id, TextSplitter &ts) {
 	_name = buf;
 
 	ts.scanString(" background %256s", 1, buf);
-	_bkgndBm = Bitmap::create(buf);
-	if (!_bkgndBm) {
-		Debug::warning(Debug::Bitmaps | Debug::Sets,
-					   "Unable to load scene bitmap: %s\n", buf);
-	} else {
-		Debug::debug(Debug::Bitmaps | Debug::Sets,
-					 "Loaded scene bitmap: %s\n", buf);
-	}
+	_bkgndBm = loadBackground(buf);
 
 	// ZBuffer is optional
-	_bkgndZBm = NULL;
+	_bkgndZBm = nullptr;
 	if (ts.checkString("zbuffer")) {
 		ts.scanString(" zbuffer %256s", 1, buf);
 		// Don't even try to load if it's the "none" bitmap
@@ -346,21 +404,24 @@ void Set::Setup::loadBinary(Common::SeekableReadStream *data) {
 	int fNameLen = 0;
 	fNameLen = data->readUint32LE();
 
-	char* fileName = new char[fNameLen];
-	data->read(fileName,fNameLen);
+	char *fileName = new char[fNameLen];
+	data->read(fileName, fNameLen);
 
-	_bkgndZBm = NULL;
-	_bkgndBm = Bitmap::create(fileName);
+	_bkgndZBm = nullptr;
+	_bkgndBm = loadBackground(fileName);
 
+	char v[sizeof(float) * 4];
+	data->read(v, sizeof(float) * 3);
+	_pos = Math::Vector3d::getVector3d(v);
 
-	data->read(_pos.getData(), 12);
+	data->read(v, sizeof(float) * 4);
+	Math::Quaternion q(get_float(v), get_float(v + 4), get_float(v + 8), get_float(v + 12));
+	q.toMatrix(_rot);
 
-	data->read(_interest.getData(), 12);
-
-	data->read(&_roll, 4);
-	data->read(&_fov, 4);
-	data->read(&_nclip, 4);
-	data->read(&_fclip, 4);
+	data->read(v, sizeof(float) * 3);
+	_fov   = get_float(v);
+	_nclip = get_float(v + 4);
+	_fclip = get_float(v + 8);
 
 	delete[] fileName;
 }
@@ -384,8 +445,17 @@ void Set::Setup::saveState(SaveGame *savedState) const {
 	}
 
 	savedState->writeVector3d(_pos);
-	savedState->writeVector3d(_interest);
-	savedState->writeFloat(_roll);
+	if (g_grim->getGameType() == GType_MONKEY4) {
+		// Get the rotation matrix as a quaternion and write it out
+		Math::Quaternion q(_rot);
+		savedState->writeFloat(q.x());
+		savedState->writeFloat(q.y());
+		savedState->writeFloat(q.z());
+		savedState->writeFloat(q.w());
+	} else {
+		savedState->writeVector3d(_interest);
+		savedState->writeFloat(_roll);
+	}
 	savedState->writeFloat(_fov);
 	savedState->writeFloat(_nclip);
 	savedState->writeFloat(_fclip);
@@ -398,8 +468,17 @@ bool Set::Setup::restoreState(SaveGame *savedState) {
 	_bkgndZBm = Bitmap::getPool().getObject(savedState->readLESint32());
 
 	_pos      = savedState->readVector3d();
-	_interest = savedState->readVector3d();
-	_roll     = savedState->readFloat();
+	if (g_grim->getGameType() == GType_MONKEY4) {
+		float x = savedState->readFloat();
+		float y = savedState->readFloat();
+		float z = savedState->readFloat();
+		float w = savedState->readFloat();
+		Math::Quaternion q(x, y, z, w);
+		_rot = q.toMatrix();
+	} else {
+		_interest = savedState->readVector3d();
+		_roll     = savedState->readFloat();
+	}
 	_fov      = savedState->readFloat();
 	_nclip    = savedState->readFloat();
 	_fclip    = savedState->readFloat();
@@ -407,6 +486,58 @@ bool Set::Setup::restoreState(SaveGame *savedState) {
 	return true;
 }
 
+Light::Light() : _intensity(0.0f), _umbraangle(0.0f), _penumbraangle(0.0f),
+		_falloffNear(0.0f), _falloffFar(0.0f), _enabled(false), _id(0) {
+}
+
+void Set::Setup::getRotation(float *x, float *y, float *z) {
+	Math::Angle aX, aY, aZ;
+	if (g_grim->getGameType() == GType_MONKEY4)
+		_rot.getEuler(&aX, &aY, &aZ, Math::EO_ZYX);
+	else
+		_rot.getEuler(&aX, &aY, &aZ, Math::EO_ZXY);
+
+	if (x != nullptr)
+		*x = aX.getDegrees();
+	if (y != nullptr)
+		*y = aY.getDegrees();
+	if (z != nullptr)
+		*z = aZ.getDegrees();
+}
+
+void Set::Setup::setPitch(Math::Angle pitch) {
+	Math::Angle oldYaw, oldRoll;
+	if (g_grim->getGameType() == GType_MONKEY4) {
+		_rot.getEuler(&oldRoll, &oldYaw, nullptr, Math::EO_ZYX);
+		_rot.buildFromEuler(oldRoll, oldYaw, pitch, Math::EO_ZYX);
+	} else {
+		_rot.getEuler(&oldYaw, nullptr, &oldRoll, Math::EO_ZXY);
+		_rot.buildFromEuler(oldYaw, pitch, oldRoll, Math::EO_ZXY);
+	}
+}
+
+void Set::Setup::setYaw(Math::Angle yaw) {
+	Math::Angle oldPitch, oldRoll;
+	if (g_grim->getGameType() == GType_MONKEY4) {
+		_rot.getEuler(&oldRoll, nullptr, &oldPitch, Math::EO_ZYX);
+		_rot.buildFromEuler(oldRoll, yaw, oldPitch, Math::EO_ZYX);
+	} else {
+		_rot.getEuler(nullptr, &oldPitch, &oldRoll, Math::EO_ZXY);
+		_rot.buildFromEuler(yaw, oldPitch, oldRoll, Math::EO_ZXY);
+	}
+}
+
+void Set::Setup::setRoll(Math::Angle roll) {
+	Math::Angle oldPitch, oldYaw;
+	if (g_grim->getGameType() == GType_MONKEY4) {
+		_rot.getEuler(nullptr, &oldYaw, &oldPitch, Math::EO_ZYX);
+		_rot.buildFromEuler(roll, oldYaw, oldPitch, Math::EO_ZYX);
+	} else {
+		_rot.getEuler(&oldYaw, &oldPitch, nullptr, Math::EO_ZXY);
+		_rot.buildFromEuler(oldYaw, oldPitch, roll, Math::EO_ZXY);
+	}
+}
+
 void Light::load(TextSplitter &ts) {
 	char buf[256];
 
@@ -420,7 +551,16 @@ void Light::load(TextSplitter &ts) {
 	_name = buf;
 
 	ts.scanString(" type %256s", 1, buf);
-	_type = buf;
+	Common::String type = buf;
+	if (type == "spot") {
+		_type = Spot;
+	} else if (type == "omni") {
+		_type = Omni;
+	} else if (type == "direct") {
+		_type = Direct;
+	} else {
+		error("Light::load() Unknown type of light: %s", buf);
+	}
 
 	ts.scanString(" position %f %f %f", 3, &_pos.x(), &_pos.y(), &_pos.z());
 	ts.scanString(" direction %f %f %f", 3, &_dir.x(), &_dir.y(), &_dir.z());
@@ -438,8 +578,45 @@ void Light::load(TextSplitter &ts) {
 }
 
 void Light::loadBinary(Common::SeekableReadStream *data) {
-	// skip lights for now
-	data->seek(100, SEEK_CUR);
+	char name[32];
+	data->read(name, 32);
+	_name = name;
+
+	char v[sizeof(float)*4];
+	data->read(v, sizeof(float) * 3);
+	_pos = Math::Vector3d::getVector3d(v);
+
+	Math::Quaternion quat;
+	data->read(v, sizeof(float) * 4);
+	quat = Math::Quaternion::getQuaternion(v);
+
+	_dir.set(0, 0, -1);
+	Math::Matrix4 rot = quat.toMatrix();
+	rot.transform(&_dir, false);
+
+	// This relies on the order of the LightType enum.
+	_type = (LightType)data->readSint32LE();
+
+	data->read(v, sizeof(float));
+	_intensity = get_float(v);
+
+	int j = data->readSint32LE();
+	// This always seems to be 0
+	if (j != 0) {
+		warning("Light::loadBinary j != 0");
+	}
+
+	_color.getRed() = data->readSint32LE();
+	_color.getGreen() = data->readSint32LE();
+	_color.getBlue() = data->readSint32LE();
+
+	data->read(v, sizeof(float) * 4);
+	_falloffNear = get_float(v);
+	_falloffFar = get_float(v + 4);
+	_umbraangle = get_float(v + 8);
+	_penumbraangle = get_float(v + 12);
+
+	_enabled = true;
 }
 
 void Light::saveState(SaveGame *savedState) const {
@@ -448,7 +625,7 @@ void Light::saveState(SaveGame *savedState) const {
 	savedState->writeBool(_enabled);
 
 	//type
-	savedState->writeString(_type);
+	savedState->writeLEUint32(_type);
 
 	savedState->writeVector3d(_pos);
 	savedState->writeVector3d(_dir);
@@ -458,12 +635,39 @@ void Light::saveState(SaveGame *savedState) const {
 	savedState->writeFloat(_intensity);
 	savedState->writeFloat(_umbraangle);
 	savedState->writeFloat(_penumbraangle);
+
+	savedState->writeFloat(_falloffNear);
+	savedState->writeFloat(_falloffFar);
 }
 
 bool Light::restoreState(SaveGame *savedState) {
 	_name = savedState->readString();
 	_enabled = savedState->readBool();
-	_type = savedState->readString();
+	if (savedState->saveMinorVersion() > 7) {
+		if (savedState->saveMinorVersion() >= 12) {
+			_type = (LightType)savedState->readLEUint32();
+		} else {
+			int type = savedState->readLEUint32();
+			if (type == 1) {
+				_type = Spot;
+			} else if (type == 2) {
+				_type = Direct;
+			} else if (type == 3) {
+				_type = Omni;
+			} else if (type == 4) {
+				_type = Ambient;
+			}
+		}
+	} else {
+		Common::String type = savedState->readString();
+		if (type == "spot") {
+			_type = Spot;
+		} else if (type == "omni") {
+			_type = Omni;
+		} else if (type == "direct") {
+			_type = Direct;
+		}
+	}
 
 	_pos           = savedState->readVector3d();
 	_dir           = savedState->readVector3d();
@@ -474,11 +678,80 @@ bool Light::restoreState(SaveGame *savedState) {
 	_umbraangle    = savedState->readFloat();
 	_penumbraangle = savedState->readFloat();
 
+	if (savedState->saveMinorVersion() >= 20) {
+		_falloffNear = savedState->readFloat();
+		_falloffFar = savedState->readFloat();
+	}
+
 	return true;
 }
 
+SetShadow::SetShadow() : _numSectors(0) {
+}
+
+void SetShadow::loadBinary(Common::SeekableReadStream *data, Set *set) {
+	uint32 nameLen = data->readUint32LE();
+	char *name = new char[nameLen];
+	data->read(name, nameLen);
+	_name = Common::String(name);
+
+	int lightNameLen = data->readSint32LE();
+	char *lightName = new char[lightNameLen];
+	data->read(lightName, lightNameLen);
+
+	char v[sizeof(float) * 3];
+	data->read(v, sizeof(float) * 3);
+	_shadowPoint = Math::Vector3d::getVector3d(v);
+
+	if (lightNameLen > 0) {
+		for (Common::List<Light *>::const_iterator it = set->getLights(false).begin(); it != set->getLights(false).end(); ++it) {
+			if ((*it)->_name.equals(lightName)) {
+				_shadowPoint = (*it)->_pos;
+				break;
+			}
+		}
+	}
+
+	int numSectors = data->readSint32LE();
+	for (int i = 0; i < numSectors; ++i) {
+		uint32 sectorNameLen = data->readUint32LE();
+		char *sectorName = new char[sectorNameLen];
+		data->read(sectorName, sectorNameLen);
+		_sectorNames.push_back(sectorName);
+		delete[] sectorName;
+	}
+
+	data->skip(4); // Unknown
+	_color._vals[0] = (byte)data->readSint32LE();
+	_color._vals[1] = (byte)data->readSint32LE();
+	_color._vals[2] = (byte)data->readSint32LE();
+	delete[] name;
+}
+
+void SetShadow::saveState(SaveGame *savedState) const {
+	savedState->writeString(_name);
+	savedState->writeVector3d(_shadowPoint);
+	savedState->writeLESint32(_numSectors);
+	savedState->writeLEUint32(_sectorNames.size());
+	for (Common::List<Common::String>::const_iterator it = _sectorNames.begin(); it != _sectorNames.end(); ++it) {
+		savedState->writeString(*it);
+	}
+	savedState->writeColor(_color);
+}
+
+void SetShadow::restoreState(SaveGame *savedState) {
+	_name = savedState->readString();
+	_shadowPoint = savedState->readVector3d();
+	_numSectors = savedState->readLESint32();
+	uint numSectors = savedState->readLEUint32();
+	for (uint i = 0; i < numSectors; ++i) {
+		_sectorNames.push_back(savedState->readString());
+	}
+	_color = savedState->readColor();
+}
+
 void Set::Setup::setupCamera() const {
-	// Ignore nclip_ and fclip_ for now.  This fixes:
+	// Ignore nclip_ and fclip_ for now in Grim.  This fixes:
 	// (a) Nothing was being displayed in the Land of the Living
 	// diner because lr.set set nclip to 0.
 	// (b) The zbuffers for setups with different nclip or
@@ -486,8 +759,13 @@ void Set::Setup::setupCamera() const {
 	// are important at some point, we'll need to modify the
 	// zbuffer transformation in bitmap.cpp to take nclip_ and
 	// fclip_ into account.
-	g_driver->setupCamera(_fov, 0.01f, 3276.8f, _roll);
-	g_driver->positionCamera(_pos, _interest);
+	if (g_grim->getGameType() == GType_GRIM) {
+		g_driver->setupCameraFrustum(_fov, 0.01f, 3276.8f);
+		g_driver->positionCamera(_pos, _interest, _roll);
+	} else {
+		g_driver->setupCameraFrustum(_fov, _nclip, _fclip);
+		g_driver->positionCamera(_pos, _rot);
+	}
 }
 
 class Sorter {
@@ -509,7 +787,13 @@ public:
 	Math::Vector3d _pos;
 };
 
-void Set::setupLights(const Math::Vector3d &pos) {
+void Set::setupLights(const Math::Vector3d &pos, bool inOverworld) {
+	if (g_grim->getGameType() == GType_MONKEY4 && !g_driver->supportsShaders()) {
+		// If shaders are not available, we do lighting in software for EMI.
+		g_driver->disableLights();
+		return;
+	}
+
 	if (!_enableLights) {
 		g_driver->disableLights();
 		return;
@@ -517,10 +801,11 @@ void Set::setupLights(const Math::Vector3d &pos) {
 
 	// Sort the ligths from the nearest to the farthest to the pos.
 	Sorter sorter(pos);
-	Common::sort(_lightsList.begin(), _lightsList.end(), sorter);
+	Common::List<Light *>* lightsList = inOverworld ? &_overworldLightsList : &_lightsList;
+	Common::sort(lightsList->begin(), lightsList->end(), sorter);
 
 	int count = 0;
-	foreach (Light *l, _lightsList) {
+	foreach (Light *l, *lightsList) {
 		if (l->_enabled) {
 			g_driver->setupLight(l, count);
 			++count;
@@ -555,6 +840,29 @@ void Set::setSetup(int num) {
 	}
 	_currSetup = _setups + num;
 	g_grim->flagRefreshShadowMask(true);
+	if (g_emiSound) {
+		g_emiSound->updateSoundPositions();
+	}
+}
+
+Bitmap::Ptr Set::loadBackground(const char *fileName) {
+	Bitmap::Ptr bg = Bitmap::create(fileName);
+	if (!bg) {
+		Debug::warning(Debug::Bitmaps | Debug::Sets,
+					   "Unable to load scene bitmap: %s, loading dfltroom instead", fileName);
+		if (g_grim->getGameType() == GType_MONKEY4) {
+			bg = Bitmap::create("dfltroom.til");
+		} else {
+			bg = Bitmap::create("dfltroom.bm");
+		}
+		if (!bg) {
+			Debug::error(Debug::Bitmaps | Debug::Sets, "Unable to load dfltroom");
+		}
+	} else {
+		Debug::debug(Debug::Bitmaps | Debug::Sets,
+					 "Loaded scene bitmap: %s", fileName);
+	}
+	return bg;
 }
 
 void Set::drawBackground() const {
@@ -577,17 +885,42 @@ void Set::drawBitmaps(ObjectState::Position stage) {
 	}
 }
 
+void Set::setupCamera() {
+	_currSetup->setupCamera();
+	_frustum.setup(g_driver->getProjection() * g_driver->getModelView());
+}
+
 Sector *Set::findPointSector(const Math::Vector3d &p, Sector::SectorType type) {
 	for (int i = 0; i < _numSectors; i++) {
 		Sector *sector = _sectors[i];
 		if (sector && (sector->getType() & type) && sector->isVisible() && sector->isPointInSector(p))
 			return sector;
 	}
-	return NULL;
+	return nullptr;
+}
+
+int Set::findSectorSortOrder(const Math::Vector3d &p, Sector::SectorType type) {
+	int setup = getSetup();
+	int sortOrder = 0;
+	float minDist = 0.01f;
+
+	for (int i = 0; i < _numSectors; i++) {
+		Sector *sector = _sectors[i];
+		if (!sector || (sector->getType() & type) == 0 || !sector->isVisible() || setup >= sector->getNumSortplanes())
+			continue;
+
+		Math::Vector3d closestPt = sector->getClosestPoint(p);
+		float thisDist = (closestPt - p).getMagnitude();
+		if (thisDist < minDist) {
+			minDist = thisDist;
+			sortOrder = sector->getSortplane(setup);
+		}
+	}
+	return sortOrder;
 }
 
 void Set::findClosestSector(const Math::Vector3d &p, Sector **sect, Math::Vector3d *closestPoint) {
-	Sector *resultSect = NULL;
+	Sector *resultSect = nullptr;
 	Math::Vector3d resultPt = p;
 	float minDist = 0.0;
 
@@ -675,7 +1008,17 @@ void Set::setSoundPosition(const char *soundName, const Math::Vector3d &pos) {
 }
 
 void Set::setSoundPosition(const char *soundName, const Math::Vector3d &pos, int minVol, int maxVol) {
+	int newBalance, newVolume;
+	calculateSoundPosition(pos, minVol, maxVol, newVolume, newBalance);
+	g_sound->setVolume(soundName, newVolume);
+	g_sound->setPan(soundName, newBalance);
+}
+
+void Set::calculateSoundPosition(const Math::Vector3d &pos, int minVol, int maxVol, int &volume, int &balance) {
 	// TODO: The volume and pan needs to be updated when the setup changes.
+	// Note: This is only used in Grim. See SoundTrack::updatePosition for the corresponding implementation in EMI.
+
+	/* distance calculation */
 	Math::Vector3d cameraPos = _currSetup->_pos;
 	Math::Vector3d vector = pos - cameraPos;
 	float distance = vector.getMagnitude();
@@ -685,50 +1028,62 @@ void Set::setSoundPosition(const char *soundName, const Math::Vector3d &pos, int
 	newVolume += minVol;
 	if (newVolume > _maxVolume)
 		newVolume = _maxVolume;
-	g_sound->setVolume(soundName, newVolume);
+	volume = newVolume;
+	float angle;
 
-	Math::Vector3d cameraVector =_currSetup->_interest - _currSetup->_pos;
-	Math::Vector3d up(0,0,1);
+	Math::Vector3d cameraVector = _currSetup->_interest - _currSetup->_pos;
+	Math::Vector3d up(0, 0, 1);
 	Math::Vector3d right;
 	cameraVector.normalize();
 	float roll = -_currSetup->_roll * LOCAL_PI / 180.f;
 	float cosr = cos(roll);
 	// Rotate the up vector by roll.
 	up = up * cosr + Math::Vector3d::crossProduct(cameraVector, up) * sin(roll) +
-		cameraVector * Math::Vector3d::dotProduct(cameraVector, up) * (1 - cosr);
+			cameraVector * Math::Vector3d::dotProduct(cameraVector, up) * (1 - cosr);
 	right = Math::Vector3d::crossProduct(cameraVector, up);
 	right.normalize();
-	float angle = atan2(Math::Vector3d::dotProduct(vector, right),
+	angle = atan2(Math::Vector3d::dotProduct(vector, right),
 						Math::Vector3d::dotProduct(vector, cameraVector));
+
 	float pan = sin(angle);
-	g_sound->setPan(soundName, (int)((pan + 1.f) / 2.f * 127.f + 0.5f));
+	balance = (int)((pan + 1.f) / 2.f * 127.f + 0.5f);
 }
 
 Sector *Set::getSectorBase(int id) {
 	if ((_numSectors >= 0) && (id < _numSectors))
 		return _sectors[id];
 	else
-		return NULL;
+		return nullptr;
 }
 
-Sector *Set::getSector(const Common::String &name) {
+Sector *Set::getSectorByName(const Common::String &name) {
 	for (int i = 0; i < _numSectors; i++) {
 		Sector *sector = _sectors[i];
-		if (strstr(sector->getName(), name.c_str())) {
+		if (sector->getName() == name) {
 			return sector;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
-Sector *Set::getSector(const Common::String &name, const Math::Vector3d &pos) {
+Sector *Set::getSectorBySubstring(const Common::String &str) {
 	for (int i = 0; i < _numSectors; i++) {
 		Sector *sector = _sectors[i];
-		if (strstr(sector->getName(), name.c_str()) && sector->isPointInSector(pos)) {
+		if (strstr(sector->getName().c_str(), str.c_str())) {
 			return sector;
 		}
 	}
-	return NULL;
+	return nullptr;
+}
+
+Sector *Set::getSectorBySubstring(const Common::String &str, const Math::Vector3d &pos) {
+	for (int i = 0; i < _numSectors; i++) {
+		Sector *sector = _sectors[i];
+		if (strstr(sector->getName().c_str(), str.c_str()) && sector->isPointInSector(pos)) {
+			return sector;
+		}
+	}
+	return nullptr;
 }
 
 void Set::setSoundParameters(int minVolume, int maxVolume) {
@@ -770,7 +1125,7 @@ ObjectState *Set::findState(const Common::String &filename) {
 			return *i;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 void Set::moveObjectStateToFront(const ObjectState::Ptr &s) {
@@ -786,4 +1141,17 @@ void Set::moveObjectStateToBack(const ObjectState::Ptr &s) {
 	_states.push_back(s);
 }
 
+SetShadow *Set::getShadow(int i) {
+	return &_shadows[i];
+}
+
+SetShadow *Set::getShadowByName(const Common::String &name) {
+	for (int i = 0; i < _numShadows; ++i) {
+		SetShadow *shadow = &_shadows[i];
+		if (shadow->_name.equalsIgnoreCase(name))
+			return shadow;
+	}
+	return nullptr;
+}
+
 } // end of namespace Grim
