@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -27,13 +27,16 @@
 #include "common/file.h"
 #include "common/util.h"
 #include "common/textconsole.h"
+#include "common/translation.h"
 
 #include "gui/debugger.h"
+#include "gui/error.h"
 
 #include "engines/engine.h"
 
 #include "engines/myst3/console.h"
 #include "engines/myst3/database.h"
+#include "engines/myst3/effects.h"
 #include "engines/myst3/myst3.h"
 #include "engines/myst3/nodecube.h"
 #include "engines/myst3/nodeframe.h"
@@ -43,10 +46,13 @@
 #include "engines/myst3/script.h"
 #include "engines/myst3/menu.h"
 #include "engines/myst3/sound.h"
+#include "engines/myst3/ambient.h"
+#include "engines/myst3/transition.h"
+
+#include "image/jpeg.h"
 
-#include "graphics/decoders/jpeg.h"
 #include "graphics/conversion.h"
-#include "graphics/pixelbuffer.h"
+#include "graphics/yuv_to_rgb.h"
 
 #include "math/vector2d.h"
 
@@ -66,10 +72,12 @@ Myst3Engine::Myst3Engine(OSystem *syst, const Myst3GameDescription *version) :
 		_db(0), _console(0), _scriptEngine(0),
 		_state(0), _node(0), _scene(0), _archiveNode(0),
 		_cursor(0), _inventory(0), _gfx(0), _menu(0),
-		_rnd(0), _sound(0),
+		_rnd(0), _sound(0), _ambient(0),
 		_inputSpacePressed(false), _inputEnterPressed(false),
 		_inputEscapePressed(false), _inputTildePressed(false),
-		_menuAction(0), _projectorBackground(0) {
+		_menuAction(0), _projectorBackground(0),
+		_shakeEffect(0), _rotationEffect(0), _backgroundSoundScriptLastRoomId(0),
+		_transition(0) {
 	DebugMan.addDebugChannel(kDebugVariable, "Variable", "Track Variable Accesses");
 	DebugMan.addDebugChannel(kDebugSaveLoad, "SaveLoad", "Track Save/Load Function");
 	DebugMan.addDebugChannel(kDebugScript, "Script", "Track Script Execution");
@@ -103,6 +111,7 @@ Myst3Engine::Myst3Engine(OSystem *syst, const Myst3GameDescription *version) :
 	SearchMan.addSubDirectoryMatching(gameDataDir, "MYST3BIN/M3DATA/TEXT/PAL");
 
 	settingsInitDefaults();
+	syncSoundSettings();
 }
 
 Myst3Engine::~Myst3Engine() {
@@ -121,21 +130,41 @@ Myst3Engine::~Myst3Engine() {
 	delete _state;
 	delete _rnd;
 	delete _sound;
+	delete _ambient;
 	delete _gfx;
 }
 
 bool Myst3Engine::hasFeature(EngineFeature f) const {
+	// The TinyGL renderer does not support arbitrary resolutions for now
+	bool softRenderer = ConfMan.getBool("soft_renderer");
+
 	return
 		(f == kSupportsRTL) ||
-		(f == kSupportsLoadingDuringRuntime);
+		(f == kSupportsLoadingDuringRuntime) ||
+		(f == kSupportsArbitraryResolutions && !softRenderer);
 }
 
 Common::Error Myst3Engine::run() {
-	const int w = 640;
-	const int h = 480;
+	if (!checkDatafiles()) {
+		// An error message has already been displayed
+		return Common::kUserCanceled;
+	}
 
-	_gfx = new Renderer(_system);
+	bool softRenderer = ConfMan.getBool("soft_renderer");
+
+	if (softRenderer) {
+		_gfx = CreateGfxTinyGL(_system);
+	} else {
+#if defined(USE_GLES2) || defined(USE_OPENGL_SHADERS)
+		_gfx = CreateGfxOpenGLShader(_system);
+#elif defined(USE_OPENGL)
+		_gfx = CreateGfxOpenGL(_system);
+#else
+		_gfx = CreateGfxTinyGL(_system);
+#endif
+	}
 	_sound = new Sound(this);
+	_ambient = new Ambient(this);
 	_rnd = new Common::RandomSource("sprint");
 	_console = new Console(this);
 	_scriptEngine = new Script(this);
@@ -145,17 +174,15 @@ Common::Error Myst3Engine::run() {
 	_menu = new Menu(this);
 	_archiveNode = new Archive();
 
-	_system->setupScreen(w, h, false, true);
 	_system->showMouse(false);
-	_system->lockMouse(true);
 
 	openArchives();
 
+	_gfx->init();
+
 	_cursor = new Cursor(this);
 	_inventory = new Inventory(this);
 
-	_gfx->init();
-
 	// Init the font
 	Graphics::Surface *font = loadTexture(1206);
 	_gfx->initFont(font);
@@ -186,13 +213,15 @@ Common::Error Myst3Engine::run() {
 	unloadNode();
 
 	_archiveNode->close();
+	_gfx->freeFont();
 
+	// Make sure the mouse is unlocked
 	_system->lockMouse(false);
 
 	return Common::kNoError;
 }
 
-void Myst3Engine::addArchive(const Common::String &file, bool mandatory) {
+bool Myst3Engine::addArchive(const Common::String &file, bool mandatory) {
 	Archive *archive = new Archive();
 	bool opened = archive->open(file.c_str(), 0);
 
@@ -203,6 +232,8 @@ void Myst3Engine::addArchive(const Common::String &file, bool mandatory) {
 		if (mandatory)
 			error("Unable to open archive %s", file.c_str());
 	}
+
+	return opened;
 }
 
 void Myst3Engine::openArchives() {
@@ -237,6 +268,7 @@ void Myst3Engine::openArchives() {
 		menuLanguage = "POLISH";
 		break;
 	case Common::EN_ANY:
+	case Common::RU_RUS:
 	default:
 		menuLanguage = "ENGLISH";
 		break;
@@ -265,8 +297,7 @@ void Myst3Engine::openArchives() {
 			break;
 		}
 	} else {
-		if (getDefaultLanguage() == Common::EN_ANY
-				|| ConfMan.getInt("text_language")) {
+		if (isMonolingual() || ConfMan.getInt("text_language")) {
 			textLanguage = menuLanguage;
 		} else {
 			textLanguage = "ENGLISHjp";
@@ -277,9 +308,9 @@ void Myst3Engine::openArchives() {
 	addArchive("OVER101.m3o", false);
 	addArchive(textLanguage + ".m3t", true);
 
-	if (getExecutableVersion()->flags & kFlagDVD
-			|| getDefaultLanguage() != Common::EN_ANY)
-		addArchive(menuLanguage + ".m3u", true);
+	if (getExecutableVersion()->flags & kFlagDVD || !isMonolingual())
+		if (!addArchive("language.m3u", false))
+			addArchive(menuLanguage + ".m3u", true);
 
 	addArchive("RSRC.m3r", true);
 }
@@ -287,9 +318,33 @@ void Myst3Engine::openArchives() {
 void Myst3Engine::closeArchives() {
 	for (uint i = 0; i < _archivesCommon.size(); i++)
 		delete _archivesCommon[i];
+
 	_archivesCommon.clear();
 }
 
+bool Myst3Engine::checkDatafiles() {
+#ifndef USE_SAFEDISC
+	if (getExecutableVersion()->safeDiskKey) {
+		static const char *safediscMessage =
+				_("This version of Myst III is encrypted with a copy-protection\n"
+				"preventing ResidualVM from reading required data.\n"
+				"Please replace your 'M3.exe' file with the one from the official update\n"
+				"corresponding to your game's language and redetect the game.\n"
+				"These updates don't contain the copy-protection and can be downloaded from\n"
+				"http://www.residualvm.org/downloads/");
+		warning("%s", safediscMessage);
+		GUI::displayErrorDialog(safediscMessage);
+		return false;
+	}
+#endif // USE_SAFEDISC
+	return true;
+}
+
+bool Myst3Engine::isMonolingual() const {
+	return getDefaultLanguage() == Common::EN_ANY
+			|| getDefaultLanguage() == Common::RU_RUS;
+}
+
 HotSpot *Myst3Engine::getHoveredHotspot(NodePtr nodeData, uint16 var) {
 	_state->setHotspotHovered(false);
 	_state->setHotspotActiveRect(0);
@@ -298,12 +353,9 @@ HotSpot *Myst3Engine::getHoveredHotspot(NodePtr nodeData, uint16 var) {
 		float pitch, heading;
 		_cursor->getDirection(pitch, heading);
 
-		Common::Point mouse = Common::Point((int16)heading, (int16)pitch);
-
 		for (uint j = 0; j < nodeData->hotspots.size(); j++) {
-			int32 hitRect = nodeData->hotspots[j].isPointInRectsCube(mouse);
-			if (hitRect >= 0
-					&& nodeData->hotspots[j].isEnabled(_state, var)) {
+			int32 hitRect = nodeData->hotspots[j].isPointInRectsCube(pitch, heading);
+			if (hitRect >= 0 && nodeData->hotspots[j].isEnabled(_state, var)) {
 				if (nodeData->hotspots[j].rects.size() > 1) {
 					_state->setHotspotHovered(true);
 					_state->setHotspotActiveRect(hitRect);
@@ -313,24 +365,15 @@ HotSpot *Myst3Engine::getHoveredHotspot(NodePtr nodeData, uint16 var) {
 		}
 	} else {
 		Common::Point mouse = _cursor->getPosition();
-		Common::Point scaledMouse;
 
-		if (_state->getViewType() == kMenu)  {
-			scaledMouse = Common::Point(
-					mouse.x * Renderer::kOriginalWidth / _system->getWidth(),
-					CLIP<uint>(mouse.y * Renderer::kOriginalHeight / _system->getHeight(),
-							0, Renderer::kOriginalHeight));
-		} else {
-			scaledMouse = Common::Point(
-					mouse.x * Renderer::kOriginalWidth / _system->getWidth(),
-					CLIP<uint>(mouse.y * Renderer::kOriginalHeight / _system->getHeight()
-							- Renderer::kTopBorderHeight, 0, Renderer::kFrameHeight));
+		if (_state->getViewType() == kFrame)  {
+			mouse.y -= Renderer::kTopBorderHeight;
+			mouse.y = CLIP<uint>(mouse.y, 0, Renderer::kFrameHeight);
 		}
 
 		for (uint j = 0; j < nodeData->hotspots.size(); j++) {
-			int32 hitRect = nodeData->hotspots[j].isPointInRectsFrame(_state, scaledMouse);
-			if (hitRect >= 0
-					&& nodeData->hotspots[j].isEnabled(_state, var)) {
+			int32 hitRect = nodeData->hotspots[j].isPointInRectsFrame(_state, mouse);
+			if (hitRect >= 0 && nodeData->hotspots[j].isEnabled(_state, var)) {
 				if (nodeData->hotspots[j].rects.size() > 1) {
 					_state->setHotspotHovered(true);
 					_state->setHotspotActiveRect(hitRect);
@@ -346,7 +389,10 @@ HotSpot *Myst3Engine::getHoveredHotspot(NodePtr nodeData, uint16 var) {
 void Myst3Engine::updateCursor() {
 	NodePtr nodeData = _db->getNodeData(_state->getLocationNode(), _state->getLocationRoom());
 
+	_state->setHotspotIgnoreClick(true);
 	HotSpot *hovered = getHoveredHotspot(nodeData);
+	_state->setHotspotIgnoreClick(false);
+
 	uint16 hoveredInventory = _inventory->hoveredItem();
 
 	if (hovered) {
@@ -361,6 +407,7 @@ void Myst3Engine::updateCursor() {
 void Myst3Engine::processInput(bool lookOnly) {
 	// Process events
 	Common::Event event;
+
 	while (getEventManager()->pollEvent(event)) {
 		if (event.type == Common::EVENT_MOUSEMOVE) {
 			if (_state->getViewType() == kCube
@@ -368,11 +415,12 @@ void Myst3Engine::processInput(bool lookOnly) {
 				_scene->updateCamera(event.relMouse);
 			}
 
-			_cursor->updatePosition(event.relMouse);
+			_cursor->updatePosition(event.mouse);
 
 		} else if (event.type == Common::EVENT_LBUTTONDOWN) {
 			// Skip the event when in look only mode
-			if (lookOnly) continue;
+			if (lookOnly)
+				continue;
 
 			uint16 hoveredInventory = _inventory->hoveredItem();
 			if (isInventoryVisible() && hoveredInventory > 0) {
@@ -392,11 +440,14 @@ void Myst3Engine::processInput(bool lookOnly) {
 			_sound->playEffect(697, 5);
 		} else if (event.type == Common::EVENT_RBUTTONDOWN) {
 			// Skip the event when in look only mode
-			if (lookOnly) continue;
+			if (lookOnly)
+				continue;
 			// Nothing to do if not in cube view
-			if (_state->getViewType() != kCube) continue;
+			if (_state->getViewType() != kCube)
+				continue;
 			// Don't unlock if the cursor is transparent
-			if (!_state->getCursorTransparency()) continue;
+			if (!_state->getCursorTransparency())
+				continue;
 
 			bool cursorLocked = _cursor->isPositionLocked();
 			_cursor->lockPosition(!cursorLocked);
@@ -432,7 +483,7 @@ void Myst3Engine::processInput(bool lookOnly) {
 					_system->lockMouse(false);
 					_console->attach();
 					_console->onFrame();
-					_system->lockMouse(true);
+					_system->lockMouse(_cursor->isPositionLocked());
 				}
 				break;
 			default:
@@ -460,7 +511,7 @@ void Myst3Engine::processInput(bool lookOnly) {
 	}
 }
 
-void Myst3Engine::drawFrame() {
+void Myst3Engine::drawFrame(bool noSwap) {
 	_sound->update();
 	_gfx->clear();
 
@@ -468,9 +519,25 @@ void Myst3Engine::drawFrame() {
 		float pitch = _state->getLookAtPitch();
 		float heading = _state->getLookAtHeading();
 		float fov = _state->getLookAtFOV();
+
+		// Apply the rotation effect
+		if (_rotationEffect) {
+			_rotationEffect->update();
+
+			heading += _rotationEffect->getHeadingOffset();
+			_state->lookAt(pitch, heading);
+		}
+
+		// Apply the shake effect
+		if (_shakeEffect) {
+			_shakeEffect->update();
+			pitch += _shakeEffect->getPitchOffset();
+			heading += _shakeEffect->getHeadingOffset();
+		}
+
 		_gfx->setupCameraPerspective(pitch, heading, fov);
 	} else {
-		_gfx->setupCameraOrtho2D();
+		_gfx->setupCameraOrtho2D(false);
 	}
 
 	if (_node) {
@@ -478,7 +545,7 @@ void Myst3Engine::drawFrame() {
 		_node->draw();
 	}
 
-	for (uint i = 0; i < _movies.size(); i++) {
+	for (int i = _movies.size() - 1; i >= 0 ; i--) {
 		_movies[i]->update();
 		_movies[i]->draw();
 	}
@@ -492,7 +559,7 @@ void Myst3Engine::drawFrame() {
 	}
 
 	if (_state->getViewType() == kCube) {
-		_gfx->setupCameraOrtho2D();
+		_gfx->setupCameraOrtho2D(false);
 	}
 
 	if (_state->getViewType() != kMenu) {
@@ -509,7 +576,7 @@ void Myst3Engine::drawFrame() {
 		_inventory->draw();
 
 	// Draw overlay 2D movies
-	for (uint i = 0; i < _movies.size(); i++) {
+	for (int i = _movies.size() - 1; i >= 0 ; i--) {
 		_movies[i]->drawOverlay();
 	}
 
@@ -522,12 +589,18 @@ void Myst3Engine::drawFrame() {
 		_node->drawOverlay();
 	}
 
+	// The cursor is drawn unscaled
+	_gfx->setupCameraOrtho2D(true);
 	if (_cursor->isVisible())
 		_cursor->draw();
 
-	_system->updateScreen();
-	_system->delayMillis(10);
-	_state->updateFrameCounters();
+	_gfx->flipBuffer();
+
+	if (!noSwap) {
+		_system->updateScreen();
+		_system->delayMillis(10);
+		_state->updateFrameCounters();
+	}
 }
 
 bool Myst3Engine::isInventoryVisible() {
@@ -540,7 +613,20 @@ bool Myst3Engine::isInventoryVisible() {
 	return true;
 }
 
-void Myst3Engine::goToNode(uint16 nodeID, uint transition) {
+void Myst3Engine::setupTransition() {
+	delete _transition;
+	_transition = new Transition(this);
+}
+
+void Myst3Engine::drawTransition(TransitionType transitionType) {
+	if (_transition) {
+		_transition->draw(transitionType);
+		delete _transition;
+		_transition = 0;
+	}
+}
+
+void Myst3Engine::goToNode(uint16 nodeID, TransitionType transitionType) {
 	uint16 node = _state->getLocationNextNode();
 	if (node == 0)
 		node = nodeID;
@@ -548,7 +634,10 @@ void Myst3Engine::goToNode(uint16 nodeID, uint transition) {
 	uint16 room = _state->getLocationNextRoom();
 	uint16 age = _state->getLocationNextAge();
 
-	if (_state->getViewType() == kCube) {
+	setupTransition();
+
+	ViewType sourceViewType = _state->getViewType();
+	if (sourceViewType == kCube) {
 		// The lookat direction in the next node should be
 		// the direction of the mouse cursor
 		float pitch, heading;
@@ -561,6 +650,12 @@ void Myst3Engine::goToNode(uint16 nodeID, uint transition) {
 	_state->setLocationNextNode(0);
 	_state->setLocationNextRoom(0);
 	_state->setLocationNextAge(0);
+
+	if (_state->getAmbiantPreviousFadeOutDelay() > 0) {
+		_ambient->playCurrentNode(100, _state->getAmbiantPreviousFadeOutDelay());
+	}
+
+	drawTransition(transitionType);
 }
 
 void Myst3Engine::loadNode(uint16 nodeID, uint32 roomID, uint32 ageID) {
@@ -598,10 +693,15 @@ void Myst3Engine::loadNode(uint16 nodeID, uint32 roomID, uint32 ageID) {
 
 	runNodeInitScripts();
 
+	// These effects can only be created after running the node init scripts
+	_shakeEffect = ShakeEffect::create(this);
+	_rotationEffect = RotationEffect::create(this);
+
 	// WORKAROUND: In Narayan, the scripts in node NACH 9 test on var 39
 	// without first reinitializing it leading to Saavedro not always giving
 	// Releeshan to the player when he is trapped between both shields.
-	if (nodeID == 9 && roomID == 801) _state->setVar(39, 0);
+	if (nodeID == 9 && roomID == 801)
+		_state->setVar(39, 0);
 }
 
 void Myst3Engine::unloadNode() {
@@ -614,10 +714,18 @@ void Myst3Engine::unloadNode() {
 	// Remove all sunspots
 	for (uint i = 0; i < _sunspots.size(); i++)
 		delete _sunspots[i];
+
 	_sunspots.clear();
 
+	// Clean up the effects
+	delete _shakeEffect;
+	_shakeEffect = nullptr;
+	_state->setShakeEffectAmpl(0);
+	delete _rotationEffect;
+	_rotationEffect = nullptr;
+
 	delete _node;
-	_node = 0;
+	_node = nullptr;
 }
 
 void Myst3Engine::runNodeInitScripts() {
@@ -642,10 +750,15 @@ void Myst3Engine::runNodeInitScripts() {
 			_scriptEngine->run(&nodeData->scripts[j].script);
 		}
 	}
+
+	// Mark the node as a reachable zip destination
+	_state->markNodeAsVisited(
+			_state->getLocationNode(),
+			_state->getLocationRoom());
 }
 
 void Myst3Engine::runNodeBackgroundScripts() {
-	NodePtr nodeDataRoom = _db->getNodeData(32675, _state->getLocationRoom());
+	NodePtr nodeDataRoom = _db->getNodeData(32765, _state->getLocationRoom());
 
 	if (nodeDataRoom) {
 		for (uint j = 0; j < nodeDataRoom->hotspots.size(); j++) {
@@ -711,6 +824,9 @@ void Myst3Engine::runScriptsFromNode(uint16 nodeID, uint32 roomID, uint32 ageID)
 }
 
 void Myst3Engine::runBackgroundSoundScriptsFromNode(uint16 nodeID, uint32 roomID, uint32 ageID) {
+	if (_state->getSoundScriptsSuspended())
+		return;
+
 	if (roomID == 0)
 		roomID = _state->getLocationRoom();
 
@@ -721,6 +837,24 @@ void Myst3Engine::runBackgroundSoundScriptsFromNode(uint16 nodeID, uint32 roomID
 
 	if (!nodeData) return;
 
+	// Stop previous music when changing room
+	if (_backgroundSoundScriptLastRoomId != roomID) {
+		if (_backgroundSoundScriptLastRoomId != 0
+				&& _backgroundSoundScriptLastRoomId != 901
+				&& _backgroundSoundScriptLastRoomId != 902
+				&& roomID != 0 && roomID != 901 && roomID != 902) {
+			_sound->stopMusic(_state->getSoundScriptFadeOutDelay());
+
+			if (nodeData->backgroundSoundScripts.size() != 0) {
+				_state->setSoundScriptsPaused(1);
+				_state->setSoundScriptsTimer(0);
+			}
+		}
+
+		_backgroundSoundScriptLastRoomId = roomID;
+	}
+
+
 	for (uint j = 0; j < nodeData->backgroundSoundScripts.size(); j++) {
 		if (_state->evaluate(nodeData->backgroundSoundScripts[j].condition)) {
 			if (!_scriptEngine->run(&nodeData->backgroundSoundScripts[j].script))
@@ -729,6 +863,25 @@ void Myst3Engine::runBackgroundSoundScriptsFromNode(uint16 nodeID, uint32 roomID
 	}
 }
 
+void Myst3Engine::runAmbientScripts(uint32 node) {
+	uint32 room = _ambient->_scriptRoom;
+	uint32 age = _ambient->_scriptAge;
+
+	if (room == 0)
+		room = _state->getLocationRoom();
+
+	if (age == 0)
+		age = _state->getLocationAge();
+
+	NodePtr nodeData = _db->getNodeData(node, room, age);
+
+	if (!nodeData) return;
+
+	for (uint j = 0; j < nodeData->soundScripts.size(); j++)
+		if (_state->evaluate(nodeData->soundScripts[j].condition))
+			_scriptEngine->run(&nodeData->soundScripts[j].script);
+}
+
 void Myst3Engine::loadMovie(uint16 id, uint16 condition, bool resetCond, bool loop) {
 	ScriptedMovie *movie;
 
@@ -815,6 +968,45 @@ void Myst3Engine::loadMovie(uint16 id, uint16 condition, bool resetCond, bool lo
 		_state->setMovieForce2d(0);
 	}
 
+	if (_state->getMovieVolume1()) {
+		movie->setVolume(_state->getMovieVolume1());
+		_state->setMovieVolume1(0);
+	} else {
+		movie->setVolume(_state->getMovieVolume2());
+	}
+
+	if (_state->getMovieVolumeVar()) {
+		movie->setVolumeVar(_state->getMovieVolumeVar());
+		_state->setMovieVolumeVar(0);
+	}
+
+	if (_state->getMovieSoundHeading()) {
+		movie->setSoundHeading(_state->getMovieSoundHeading());
+		_state->setMovieSoundHeading(0);
+	}
+
+	if (_state->getMoviePanningStrenght()) {
+		movie->setSoundAttenuation(_state->getMoviePanningStrenght());
+		_state->setMoviePanningStrenght(0);
+	}
+
+	if (_state->getMovieAdditiveBlending()) {
+		movie->setAdditiveBlending(true);
+		_state->setMovieAdditiveBlending(0);
+	}
+
+	if (_state->getMovieTransparency()) {
+		movie->setTransparency(_state->getMovieTransparency());
+		_state->setMovieTransparency(0);
+	} else {
+		movie->setTransparency(100);
+	}
+
+	if (_state->getMovieTransparencyVar()) {
+		movie->setTransparencyVar(_state->getMovieTransparencyVar());
+		_state->setMovieTransparencyVar(0);
+	}
+
 	_movies.push_back(movie);
 }
 
@@ -836,12 +1028,22 @@ void Myst3Engine::playSimpleMovie(uint16 id, bool fullframe) {
 		_state->setMovieEndFrame(0);
 	}
 
+	if (_state->getMovieVolume1()) {
+		movie.setVolume(_state->getMovieVolume1());
+		_state->setMovieVolume1(0);
+	} else {
+		movie.setVolume(_state->getMovieVolume2());
+	}
+
 	if (fullframe) {
 		movie.setForce2d(_state->getViewType() == kCube);
+		movie.setForceOpaque(true);
 		movie.setPosU(0);
 		movie.setPosV(0);
 	}
 
+	movie.playStartupSound();
+
 	_drawables.push_back(&movie);
 
 	bool skip = false;
@@ -854,7 +1056,7 @@ void Myst3Engine::playSimpleMovie(uint16 id, bool fullframe) {
 				if (_state->getViewType() == kCube)
 					_scene->updateCamera(event.relMouse);
 
-				_cursor->updatePosition(event.relMouse);
+				_cursor->updatePosition(event.mouse);
 
 			} else if (event.type == Common::EVENT_KEYDOWN) {
 				if (event.kbd.keycode == Common::KEYCODE_SPACE
@@ -866,6 +1068,13 @@ void Myst3Engine::playSimpleMovie(uint16 id, bool fullframe) {
 	}
 
 	_drawables.pop_back();
+
+	// Reset the movie script so that the next movie will not try to run them
+	// when the user has skipped this one before the script is triggered.
+	_state->setMovieScriptStartFrame(0);
+	_state->setMovieScript(0);
+	_state->setMovieAmbiantScriptStartFrame(0);
+	_state->setMovieAmbiantScript(0);
 }
 
 void Myst3Engine::removeMovie(uint16 id) {
@@ -902,10 +1111,15 @@ void Myst3Engine::addSpotItem(uint16 id, uint16 condition, bool fade) {
 	_node->loadSpotItem(id, condition, fade);
 }
 
-void Myst3Engine::addMenuSpotItem(uint16 id, uint16 condition, const Common::Rect &rect) {
+SpotItemFace *Myst3Engine::addMenuSpotItem(uint16 id, uint16 condition, const Common::Rect &rect) {
 	assert(_node);
 
-	_node->loadMenuSpotItem(id, condition, rect);
+	SpotItemFace *face = _node->loadMenuSpotItem(condition, rect);
+
+	if (id == 1)
+		_menu->setSaveLoadSpotItem(face);
+
+	return face;
 }
 
 void Myst3Engine::loadNodeSubtitles(uint32 id) {
@@ -938,7 +1152,7 @@ const DirectorySubEntry *Myst3Engine::getFileDescription(const char* room, uint3
 }
 
 Graphics::Surface *Myst3Engine::loadTexture(uint16 id) {
-	const DirectorySubEntry *desc = getFileDescription("GLOB", id, 0, DirectorySubEntry::kCursor);
+	const DirectorySubEntry *desc = getFileDescription("GLOB", id, 0, DirectorySubEntry::kRawData);
 
 	if (!desc)
 		error("Texture %d does not exist", id);
@@ -946,7 +1160,7 @@ Graphics::Surface *Myst3Engine::loadTexture(uint16 id) {
 	Common::MemoryReadStream *data = desc->getData();
 
 	uint32 magic = data->readUint32LE();
-	if (magic != 0x2E544558)
+	if (magic != MKTAG('.', 'T', 'E', 'X'))
 		error("Wrong texture format %d", id);
 
 	data->readUint32LE(); // unk 1
@@ -956,27 +1170,29 @@ Graphics::Surface *Myst3Engine::loadTexture(uint16 id) {
 	data->readUint32LE(); // unk 3
 
 	Graphics::Surface *s = new Graphics::Surface();
-	s->create(width, height, Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24));
+	s->create(width, height, Graphics::PixelFormat(4, 8, 8, 8, 8, 8, 16, 24, 0));
 
-	data->read(s->pixels, height * s->pitch);
+	data->read(s->getPixels(), height * s->pitch);
 	delete data;
 
 	// ARGB => RGBA
-	uint32 *p = (uint32 *)s->pixels;
-	for (uint i = 0; i < width * height; i++) {
-		uint8 a = (*p >> 0) & 0xFF;
-		uint8 r = (*p >> 24) & 0xFF;
-		uint8 g = (*p >> 16) & 0xFF;
-		uint8 b = (*p >>  8) & 0xFF;
-
-		*p = (a << 24) | (r << 16) | (g << 8) | b;
-
-		p++;
-	}
+	s->convertToInPlace(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24));
 
 	return s;
 }
 
+Graphics::Surface *Myst3Engine::decodeJpeg(const DirectorySubEntry *jpegDesc) {
+	Common::MemoryReadStream *jpegStream = jpegDesc->getData();
+
+	Image::JPEGDecoder jpeg;
+	if (!jpeg.loadStream(*jpegStream))
+		error("Could not decode Myst III JPEG");
+	delete jpegStream;
+
+	const Graphics::Surface *bitmap = jpeg.getSurface();
+	return bitmap->convertTo(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24));
+}
+
 int16 Myst3Engine::openDialog(uint16 id) {
 	Dialog dialog(this, id);
 
@@ -1006,6 +1222,10 @@ void Myst3Engine::dragSymbol(uint16 var, uint16 id) {
 
 	while (inputValidatePressed() && !shouldQuit()) {
 		processInput(true);
+
+		HotSpot *hovered = getHoveredHotspot(nodeData, var);
+		drag.setFrame(hovered ? 2 : 1);
+
 		drawFrame();
 	}
 
@@ -1013,8 +1233,11 @@ void Myst3Engine::dragSymbol(uint16 var, uint16 id) {
 	_drawables.pop_back();
 
 	HotSpot *hovered = getHoveredHotspot(nodeData, var);
-	if (hovered)
+	if (hovered) {
+		_cursor->setVisible(false);
 		_scriptEngine->run(&hovered->script);
+		_cursor->setVisible(true);
+	}
 }
 
 void Myst3Engine::dragItem(uint16 statusVar, uint16 movie, uint16 frame, uint16 hoverFrame, uint16 itemVar) {
@@ -1041,7 +1264,9 @@ void Myst3Engine::dragItem(uint16 statusVar, uint16 movie, uint16 frame, uint16
 
 	HotSpot *hovered = getHoveredHotspot(nodeData, itemVar);
 	if (hovered) {
+		_cursor->setVisible(false);
 		_scriptEngine->run(&hovered->script);
+		_cursor->setVisible(true);
 	} else {
 		_state->setVar(statusVar, 1);
 		_state->setVar(itemVar, 0);
@@ -1063,7 +1288,11 @@ Common::Error Myst3Engine::loadGameState(int slot) {
 		_state->setMenuSavedRoom(0);
 		_state->setMenuSavedNode(0);
 
-		goToNode(0, 1);
+		_sound->stopMusic(15);
+		_state->setSoundScriptsSuspended(0);
+		_sound->playEffect(696, 60);
+
+		goToNode(0, kTransitionNone);
 		return Common::kNoError;
 	}
 
@@ -1080,6 +1309,15 @@ void Myst3Engine::animateDirectionChange(float targetPitch, float targetHeading,
 	float pitchDistance = targetPitch - startPitch;
 	float headingDistance = targetHeading - startHeading;
 
+	// Make sure to use the shortest direction
+	while (ABS(headingDistance) > 180) {
+		if (headingDistance > 0) {
+			headingDistance -= 360;
+		} else {
+			headingDistance += 360;
+		}
+	}
+
 	// Compute animation duration in frames
 	float numFrames;
 	if (scriptFrames) {
@@ -1165,8 +1403,8 @@ void Myst3Engine::playMovieGoToNode(uint16 movie, uint16 node) {
 		float startPitch, startHeading;
 		getMovieLookAt(movie, true, startPitch, startHeading);
 		animateDirectionChange(startPitch, startHeading, 0);
-		_state->setCameraSkipAnimation(0);
 	}
+	_state->setCameraSkipAnimation(0);
 
 	loadNode(node, room, age);
 
@@ -1188,8 +1426,8 @@ void Myst3Engine::playMovieFullFrame(uint16 movie) {
 		float startPitch, startHeading;
 		getMovieLookAt(movie, true, startPitch, startHeading);
 		animateDirectionChange(startPitch, startHeading, 0);
-		_state->setCameraSkipAnimation(0);
 	}
+	_state->setCameraSkipAnimation(0);
 
 	playSimpleMovie(movie, true);
 
@@ -1198,6 +1436,8 @@ void Myst3Engine::playMovieFullFrame(uint16 movie) {
 		getMovieLookAt(movie, false, endPitch, endHeading);
 		_state->lookAt(endPitch, endHeading);
 	}
+
+	setupTransition();
 }
 
 bool Myst3Engine::inputValidatePressed() {
@@ -1259,8 +1499,9 @@ SunSpot Myst3Engine::computeSunspotsIntensity(float pitch, float heading) {
 			result.radius = distance;
 			result.color = s->color;
 			result.intensity = s->intensity;
+			result.variableIntensity = s->variableIntensity;
 
-			if (s->variableIntensity) {
+			if (result.variableIntensity) {
 				result.radius = value * distance / 100;
 			}
 		}
@@ -1340,11 +1581,26 @@ void Myst3Engine::settingsApplyFromVars() {
 	ConfMan.setBool("zip_mode", _state->getZipModeEnabled());
 	ConfMan.setBool("subtitles", _state->getSubtitlesEnabled());
 
+	// Mouse speed may have changed, refresh it
+	_scene->updateMouseSpeed();
+
 	// The language changed, reload the correct archives
 	if (_state->getLanguageText() != oldTextLanguage) {
 		closeArchives();
 		openArchives();
 	}
+
+	syncSoundSettings();
+}
+
+void Myst3Engine::syncSoundSettings() {
+	Engine::syncSoundSettings();
+
+	uint soundOverall = ConfMan.getInt("overall_volume");
+	uint soundVolumeMusic = ConfMan.getInt("music_volume");
+
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, soundOverall);
+	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, soundVolumeMusic * soundOverall / 256);
 }
 
 } // end of namespace Myst3
