@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -28,14 +28,14 @@
 #include "engines/myst3/inventory.h"
 #include "engines/myst3/puzzles.h"
 #include "engines/myst3/sound.h"
+#include "engines/myst3/ambient.h"
 
 #include "common/events.h"
 
 namespace Myst3 {
 
 Script::Script(Myst3Engine *vm):
-	_vm(vm) {
-
+		_vm(vm) {
 	_puzzles = new Puzzles(_vm);
 
 #define OP_0(op, x) _commands.push_back(Command(op, &Script::x, #x, 0))
@@ -45,6 +45,11 @@ Script::Script(Myst3Engine *vm):
 #define OP_4(op, x, type1, type2, type3, type4) _commands.push_back(Command(op, &Script::x, #x, 4, type1, type2, type3, type4))
 #define OP_5(op, x, type1, type2, type3, type4, type5) _commands.push_back(Command(op, &Script::x, #x, 5, type1, type2, type3, type4, type5))
 
+	// TODO: Implement these remaining opcodes
+	// 5: I'm pretty sure it's useless
+	// 144: drawTransition
+	// 247: quit
+
 	OP_0(  0, badOpcode																					);
 	OP_1(  4, nodeCubeInit, 				kEvalValue													);
 	OP_5(  6, nodeCubeInitIndex, 			kVar, 		kEvalValue,	kEvalValue,	kEvalValue,	kEvalValue	);
@@ -71,6 +76,10 @@ Script::Script(Myst3Engine *vm):
 	OP_0( 28, movieRemoveAll																			);
 	OP_1( 29, movieSetLooping,				kValue														);
 	OP_1( 30, movieSetNotLooping,			kValue														);
+	OP_1( 31, waterEffectSetSpeed,			kValue														);
+	OP_1( 32, waterEffectSetAttenuation,	kValue														);
+	OP_2( 33, waterEffectSetWave,			kValue,		kValue											);
+	OP_2( 34, shakeEffectSet,				kEvalValue,	kEvalValue										);
 	OP_2( 35, sunspotAdd,					kValue,		kValue											);
 	OP_3( 36, sunspotAddIntensity,			kValue,		kValue,		kValue								);
 	OP_4( 37, sunspotAddVarIntensity,		kValue,		kValue,		kValue,		kVar					);
@@ -174,6 +183,7 @@ Script::Script(Myst3Engine *vm):
 	OP_2(139, goToRoomNode,					kValue,		kValue											);
 	OP_1(140, zipToNode,					kValue														);
 	OP_2(141, zipToRoomNode,				kValue,		kValue											);
+	OP_0(144, drawTransition																			);
 	OP_0(145, reloadNode																				);
 	OP_0(146, redrawFrame																				);
 	OP_1(147, moviePlay, 					kEvalValue													);
@@ -182,20 +192,21 @@ Script::Script(Myst3Engine *vm):
 	OP_1(150, moviePlayFullFrameTrans,		kEvalValue													);
 	OP_2(151, moviePlayChangeNode,			kEvalValue,	kEvalValue										);
 	OP_2(152, moviePlayChangeNodeTrans,		kEvalValue,	kEvalValue										);
-	OP_2(153, lootAt,						kValue, 	kValue											);
-	OP_3(154, lootAtInXFrames,				kValue, 	kValue,		kValue								);
-	OP_1(155, lootAtMovieStart,				kEvalValue													);
-	OP_2(156, lootAtMovieStartInXFrames,	kEvalValue,	kValue											);
+	OP_2(153, lookAt,						kValue, 	kValue											);
+	OP_3(154, lookAtInXFrames,				kValue, 	kValue,		kValue								);
+	OP_1(155, lookAtMovieStart,				kEvalValue													);
+	OP_2(156, lookAtMovieStartInXFrames,	kEvalValue,	kValue											);
 	OP_4(157, cameraLimitMovement,			kValue,		kValue,		kValue,		kValue					);
 	OP_0(158, cameraFreeMovement																		);
 	OP_2(159, cameraLookAt,					kValue,		kValue											);
 	OP_1(160, cameraLookAtVar,				kVar														);
 	OP_1(161, cameraGetLookAt,				kVar														);
-	OP_1(162, lootAtMovieStartImmediate,	kEvalValue													);
+	OP_1(162, lookAtMovieStartImmediate,	kEvalValue													);
 	OP_1(163, cameraSetFOV,					kEvalValue													);
 	OP_1(164, changeNode,					kValue														);
 	OP_2(165, changeNodeRoom,				kValue,		kValue											);
 	OP_3(166, changeNodeRoomAge,			kValue,		kValue,		kValue								);
+	OP_0(168, uselessOpcode																				);
 	OP_1(169, drawXFrames,					kValue														);
 	OP_1(171, drawWhileCond,				kCondition													);
 	OP_1(172, whileStart,					kCondition													);
@@ -221,13 +232,44 @@ Script::Script(Myst3Engine *vm):
 	OP_2(195, runPuzzle2,					kValue,		kValue											);
 	OP_3(196, runPuzzle3,					kValue,		kValue,		kValue								);
 	OP_4(197, runPuzzle4,					kValue,		kValue,		kValue,		kValue					);
+	OP_3(198, ambientLoadNode,				kValue,		kValue,		kValue								);
+	OP_1(199, ambientReloadCurrentNode,		kEvalValue													);
+	OP_2(200, ambientPlayCurrentNode,		kValue,		kValue											);
+	OP_0(201, ambientApply																				);
+	OP_1(202, ambientApplyWithFadeDelay,	kEvalValue													);
+	OP_0(203, soundPlayBadClick																			);
+	OP_5(204, soundPlayBlocking,			kEvalValue,	kEvalValue,	kEvalValue,	kEvalValue,	kValue		);
 	OP_1(205, soundPlay,					kEvalValue													);
 	OP_2(206, soundPlayVolume,				kEvalValue,	kEvalValue										);
 	OP_3(207, soundPlayVolumeDirection,		kEvalValue,	kEvalValue,	kEvalValue							);
 	OP_4(208, soundPlayVolumeDirectionAtt,	kEvalValue,	kEvalValue,	kEvalValue,	kEvalValue				);
+	OP_1(209, soundStopEffect,				kEvalValue													);
+	OP_2(210, soundFadeOutEffect,			kEvalValue,	kEvalValue										);
+	OP_1(212, soundPlayLooping,				kEvalValue													);
+	OP_5(213, soundPlayFadeInOut,			kEvalValue,	kEvalValue,	kEvalValue,	kEvalValue,	kEvalValue	);
+	OP_5(214, soundChooseNext,				kVar,		kValue,		kValue,		kEvalValue,	kEvalValue	);
+	OP_5(215, soundRandomizeNext,			kVar,		kValue,		kValue,		kEvalValue,	kEvalValue	);
+	OP_5(216, soundChooseNextAfterOther,	kVar,		kValue,		kValue,		kEvalValue,	kEvalValue	); // Seven args
+	OP_5(217, soundRandomizeNextAfterOther,	kVar,		kValue,		kValue,		kEvalValue,	kEvalValue	); // Seven args
+	OP_1(218, ambientSetFadeOutDelay,		kValue														);
+	OP_2(219, ambientAddSound1,				kEvalValue,	kEvalValue										);
+	OP_3(220, ambientAddSound2,				kEvalValue,	kEvalValue,	kValue								);
+	OP_3(222, ambientAddSound3,				kEvalValue,	kEvalValue,	kValue								);
+	OP_4(223, ambientAddSound4,				kEvalValue,	kEvalValue,	kValue,		kValue					);
+	OP_3(224, ambientAddSound5,				kEvalValue,	kEvalValue,	kEvalValue							);
+	OP_2(225, ambientSetCue1,				kValue,		kEvalValue										);
+	OP_3(226, ambientSetCue2,				kValue,		kEvalValue,	kValue								);
+	OP_4(227, ambientSetCue3,				kValue,		kEvalValue,	kValue, kValue						);
+	OP_2(228, ambientSetCue4,				kValue,		kEvalValue										);
+	OP_1(229, runAmbientScriptNode,			kEvalValue													);
+	OP_4(230, runAmbientScriptNodeRoomAge,	kEvalValue,	kEvalValue,	kEvalValue,	kEvalValue				);
 	OP_1(231, runSoundScriptNode,			kEvalValue													);
 	OP_2(232, runSoundScriptNodeRoom,		kEvalValue,	kEvalValue										);
 	OP_3(233, runSoundScriptNodeRoomAge,	kEvalValue,	kEvalValue,	kEvalValue							);
+	OP_1(234, soundStopMusic,				kEvalValue													);
+	OP_1(235, movieSetStartupSound,			kEvalValue													);
+	OP_2(236, movieSetStartupSoundVolume,	kEvalValue,	kEvalValue										);
+	OP_3(237, movieSetStartupSoundVolumeH,	kEvalValue,	kEvalValue,	kEvalValue							);
 	OP_0(239, drawOneFrame																				);
 	OP_0(240, cursorHide																				);
 	OP_0(241, cursorShow																				);
@@ -250,7 +292,7 @@ Script::~Script() {
 }
 
 bool Script::run(const Common::Array<Opcode> *script) {
-	debugC(kDebugScript, "Script start %p", (void *) script);
+	debugC(kDebugScript, "Script start %p", (const void *) script);
 
 	Context c;
 	c.result = true;
@@ -267,7 +309,7 @@ bool Script::run(const Common::Array<Opcode> *script) {
 		c.op++;
 	}
 
-	debugC(kDebugScript, "Script stop %p ", (void *) script);
+	debugC(kDebugScript, "Script stop %p ", (const void *) script);
 
 	return c.result;
 }
@@ -287,7 +329,12 @@ void Script::runOp(Context &c, const Opcode &op) {
 	if (cmd.op != 0)
 		(this->*(cmd.proc))(c, op);
 	else
-		warning("Trying to run invalid opcode %d", op.op);
+		debugC(kDebugScript, "Trying to run invalid opcode %d", op.op);
+}
+
+void Script::runSingleOp(const Opcode &op) {
+	Context c;
+	runOp(c, op);
 }
 
 const Common::String Script::describeCommand(uint16 op) {
@@ -342,12 +389,21 @@ void Script::badOpcode(Context &c, const Opcode &cmd) {
 	error("Trying to run invalid opcode %d", cmd.op);
 }
 
+void Script::uselessOpcode(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Useless opcode", cmd.op);
+
+	// List of useless opcodes
+
+	// 167 and 168 form a pair. 168 resets what 167 sets up.
+	// Since 167 is never used, 168 is marked as useless.
+}
+
+
 void Script::nodeCubeInit(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Node cube init %d", cmd.op, cmd.args[0]);
 
 	uint16 nodeId = _vm->_state->valueOrVarValue(cmd.args[0]);
 	_vm->loadNodeCubeFaces(nodeId);
-	// TODO: Load rects
 }
 
 void Script::nodeCubeInitIndex(Context &c, const Opcode &cmd) {
@@ -363,7 +419,6 @@ void Script::nodeCubeInitIndex(Context &c, const Opcode &cmd) {
 
 	uint16 nodeId = _vm->_state->valueOrVarValue(value);
 	_vm->loadNodeCubeFaces(nodeId);
-	// TODO: Load rects
 }
 
 void Script::nodeFrameInit(Context &c, const Opcode &cmd) {
@@ -371,7 +426,6 @@ void Script::nodeFrameInit(Context &c, const Opcode &cmd) {
 
 	uint16 nodeId = _vm->_state->valueOrVarValue(cmd.args[0]);
 	_vm->loadNodeFrame(nodeId);
-	// TODO: Load rects
 }
 
 void Script::nodeFrameInitCond(Context &c, const Opcode &cmd) {
@@ -386,7 +440,6 @@ void Script::nodeFrameInitCond(Context &c, const Opcode &cmd) {
 
 	uint16 nodeId = _vm->_state->valueOrVarValue(value);
 	_vm->loadNodeFrame(nodeId);
-	// TODO: Load rects
 }
 
 void Script::nodeFrameInitIndex(Context &c, const Opcode &cmd) {
@@ -402,7 +455,6 @@ void Script::nodeFrameInitIndex(Context &c, const Opcode &cmd) {
 
 	uint16 nodeId = _vm->_state->valueOrVarValue(value);
 	_vm->loadNodeFrame(nodeId);
-	// TODO: Load rects
 }
 
 void Script::nodeMenuInit(Context &c, const Opcode &cmd) {
@@ -410,7 +462,6 @@ void Script::nodeMenuInit(Context &c, const Opcode &cmd) {
 
 	uint16 nodeId = _vm->_state->valueOrVarValue(cmd.args[0]);
 	_vm->loadNodeMenu(nodeId);
-	// TODO: Load rects
 }
 
 void Script::stopWholeScript(Context &c, const Opcode &cmd) {
@@ -586,6 +637,39 @@ void Script::movieSetNotLooping(Context &c, const Opcode &cmd) {
 	_vm->setMovieLooping(cmd.args[0], false);
 }
 
+void Script::waterEffectSetSpeed(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set water effect speed to %d",
+			cmd.op, cmd.args[0]);
+
+	_vm->_state->setWaterEffectSpeed(cmd.args[0]);
+}
+
+void Script::waterEffectSetAttenuation(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set water effect attenuation to %d",
+			cmd.op, cmd.args[0]);
+
+	_vm->_state->setWaterEffectAttenuation(cmd.args[0]);
+}
+
+void Script::waterEffectSetWave(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set water effect frequency to %d and amplitude to %d",
+			cmd.op, cmd.args[0], cmd.args[1]);
+
+	_vm->_state->setWaterEffectFrequency(cmd.args[0]);
+	_vm->_state->setWaterEffectAmpl(cmd.args[1]);
+}
+
+void Script::shakeEffectSet(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set shake effect amplitude to %d and period to %d",
+			cmd.op, cmd.args[0], cmd.args[1]);
+
+	uint16 ampl = _vm->_state->valueOrVarValue(cmd.args[0]);
+	uint16 period = _vm->_state->valueOrVarValue(cmd.args[1]);
+
+	_vm->_state->setShakeEffectAmpl(ampl);
+	_vm->_state->setShakeEffectFramePeriod(period);
+}
+
 void Script::sunspotAdd(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Add sunspot: pitch %d heading %d", cmd.op, cmd.args[0], cmd.args[1]);
 
@@ -1284,8 +1368,7 @@ void Script::goToElse(Context &c) {
 	// Go to next command until an else statement is met
 	do {
 		c.op++;
-	} while (c.op != c.script->end()
-			&& c.op->op != 104);
+	} while (c.op != c.script->end() && c.op->op != 104);
 }
 
 void Script::ifCondition(Context &c, const Opcode &cmd) {
@@ -1412,7 +1495,7 @@ void Script::ifVarNotInRange(Context &c, const Opcode &cmd) {
 			cmd.op, cmd.args[0], cmd.args[1], cmd.args[2]);
 
 	int32 value = _vm->_state->getVar(cmd.args[0]);
-	if(value < cmd.args[1] && value > cmd.args[2])
+	if(value < cmd.args[1] || value > cmd.args[2])
 		return;
 
 	goToElse(c);
@@ -1614,7 +1697,7 @@ void Script::leverDrag(Context &c, const Opcode &cmd) {
 		_vm->_state->setDragEnded(!mousePressed);
 
 		if (_vm->_state->getDragLeverSpeed()) {
-			warning("Interaction with var 58 is missing in opcode 127.");
+			debugC(kDebugScript, "Interaction with var 58 is missing in opcode 127.");
 			return;
 		}
 
@@ -1677,7 +1760,7 @@ void Script::leverDragPositions(Context &c, const Opcode &cmd) {
 		_vm->_state->setDragEnded(!mousePressed);
 
 		if (_vm->_state->getDragLeverSpeed()) {
-			warning("Interaction with var 58 is missing in opcode 132.");
+			debugC(kDebugScript, "Interaction with var 58 is missing in opcode 132.");
 			return;
 		}
 
@@ -1812,19 +1895,19 @@ void Script::chooseNextNode(Context &c, const Opcode &cmd) {
 void Script::goToNodeTransition(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Go to node %d with transition %d", cmd.op, cmd.args[0], cmd.args[1]);
 
-	_vm->goToNode(cmd.args[0], cmd.args[1]);
+	_vm->goToNode(cmd.args[0], static_cast<TransitionType>(cmd.args[1]));
 }
 
 void Script::goToNodeTrans2(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Go to node %d", cmd.op, cmd.args[0]);
 
-	_vm->goToNode(cmd.args[0], 2);
+	_vm->goToNode(cmd.args[0], kTransitionNone);
 }
 
 void Script::goToNodeTrans1(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Go to node %d", cmd.op, cmd.args[0]);
 
-	_vm->goToNode(cmd.args[0], 1);
+	_vm->goToNode(cmd.args[0], kTransitionFade);
 }
 
 void Script::goToRoomNode(Context &c, const Opcode &cmd) {
@@ -1833,13 +1916,13 @@ void Script::goToRoomNode(Context &c, const Opcode &cmd) {
 	_vm->_state->setLocationNextRoom(cmd.args[0]);
 	_vm->_state->setLocationNextNode(cmd.args[1]);
 
-	_vm->goToNode(0, 1);
+	_vm->goToNode(0, kTransitionFade);
 }
 
 void Script::zipToNode(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Zip to node %d", cmd.op, cmd.args[0]);
 
-	_vm->goToNode(cmd.args[0], 3);
+	_vm->goToNode(cmd.args[0], kTransitionZip);
 }
 
 void Script::zipToRoomNode(Context &c, const Opcode &cmd) {
@@ -1848,7 +1931,13 @@ void Script::zipToRoomNode(Context &c, const Opcode &cmd) {
 	_vm->_state->setLocationNextRoom(cmd.args[0]);
 	_vm->_state->setLocationNextNode(cmd.args[1]);
 
-	_vm->goToNode(0, 3);
+	_vm->goToNode(0, kTransitionZip);
+}
+
+void Script::drawTransition(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Draw transition", cmd.op);
+
+	_vm->drawTransition(kTransitionFade);
 }
 
 void Script::reloadNode(Context &c, const Opcode &cmd) {
@@ -1915,7 +2004,7 @@ void Script::cameraGetLookAt(Context &c, const Opcode &cmd) {
 	_vm->_state->setVar(cmd.args[0] + 1, (int32)heading);
 }
 
-void Script::lootAtMovieStartImmediate(Context &c, const Opcode &cmd) {
+void Script::lookAtMovieStartImmediate(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Look at movie %d start", cmd.op, cmd.args[0]);
 
 	uint16 movieId = _vm->_state->valueOrVarValue(cmd.args[0]);
@@ -1965,8 +2054,7 @@ void Script::drawXFrames(Context &c, const Opcode &cmd) {
 void Script::drawWhileCond(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: While condition %d, draw", cmd.op, cmd.args[0]);
 
-	// TODO: Skippable with Escape
-	while (_vm->_state->evaluate(cmd.args[0])) {
+	while (_vm->_state->evaluate(cmd.args[0]) && !_vm->inputEscapePressed()) {
 		_vm->processInput(true);
 		_vm->drawFrame();
 	}
@@ -1980,8 +2068,7 @@ void Script::whileStart(Context &c, const Opcode &cmd) {
 		// Condition is false, go to the next opcode after the end of the while loop
 		do {
 			c.op++;
-		} while (c.op != c.script->end()
-				&& c.op->op != 173);
+		} while (c.op != c.script->end() && c.op->op != 173);
 	}
 
 	_vm->processInput(true);
@@ -2052,7 +2139,7 @@ void Script::moviePlayFullFrameTrans(Context &c, const Opcode &cmd) {
 	_vm->playMovieFullFrame(movieId);
 	_vm->_cursor->setVisible(true);
 
-	// TODO: Transition
+	_vm->drawTransition(kTransitionFade);
 }
 
 void Script::moviePlayChangeNode(Context &c, const Opcode &cmd) {
@@ -2074,22 +2161,22 @@ void Script::moviePlayChangeNodeTrans(Context &c, const Opcode &cmd) {
 	_vm->playMovieGoToNode(movieId, nodeId);
 	_vm->_cursor->setVisible(true);
 
-	// TODO: Transition
+	_vm->drawTransition(kTransitionFade);
 }
 
-void Script::lootAt(Context &c, const Opcode &cmd) {
+void Script::lookAt(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Look at %d, %d", cmd.op, cmd.args[0], cmd.args[1]);
 
 	_vm->animateDirectionChange(cmd.args[0], cmd.args[1], 0);
 }
 
-void Script::lootAtInXFrames(Context &c, const Opcode &cmd) {
+void Script::lookAtInXFrames(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Look at %d, %d in %d frames", cmd.op, cmd.args[0], cmd.args[1], cmd.args[2]);
 
 	_vm->animateDirectionChange(cmd.args[0], cmd.args[1], cmd.args[2]);
 }
 
-void Script::lootAtMovieStart(Context &c, const Opcode &cmd) {
+void Script::lookAtMovieStart(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Look at movie %d start", cmd.op, cmd.args[0]);
 
 	uint16 movieId = _vm->_state->valueOrVarValue(cmd.args[0]);
@@ -2099,7 +2186,7 @@ void Script::lootAtMovieStart(Context &c, const Opcode &cmd) {
 	_vm->animateDirectionChange(startPitch, startHeading, 0);
 }
 
-void Script::lootAtMovieStartInXFrames(Context &c, const Opcode &cmd) {
+void Script::lookAtMovieStartInXFrames(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Look at movie %d start in %d frames", cmd.op, cmd.args[0], cmd.args[1]);
 
 	uint16 movieId = _vm->_state->valueOrVarValue(cmd.args[0]);
@@ -2154,8 +2241,8 @@ void Script::runScriptForVarDrawFramesHelper(uint16 var, int32 startValue, int32
 		bool positiveDirection = endValue > startValue;
 
 		while (1) {
-			if ((positiveDirection && (currentValue >= endValue))
-					|| (!positiveDirection && (currentValue <= endValue)))
+			if ((positiveDirection && (currentValue > endValue))
+					|| (!positiveDirection && (currentValue < endValue)))
 				break;
 
 			_vm->_state->setVar(var, currentValue);
@@ -2309,6 +2396,63 @@ void Script::runPuzzle4(Context &c, const Opcode &cmd) {
 	_puzzles->run(cmd.args[0], cmd.args[1], cmd.args[2], cmd.args[3]);
 }
 
+void Script::ambientLoadNode(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Load ambient sounds from node %d %d %d", cmd.op, cmd.args[0], cmd.args[1], cmd.args[2]);
+
+	_vm->_ambient->loadNode(cmd.args[2], cmd.args[1], cmd.args[0]);
+}
+
+void Script::ambientReloadCurrentNode(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Reload ambient sounds from current node with fade out delay : %d", cmd.op, cmd.args[0]);
+
+	_vm->_ambient->loadNode(0, 0, 0);
+	_vm->_ambient->applySounds(_vm->_state->valueOrVarValue(cmd.args[0]));
+}
+
+void Script::ambientPlayCurrentNode(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Play ambient sounds from current node %d %d", cmd.op, cmd.args[0], cmd.args[1]);
+
+	_vm->_ambient->playCurrentNode(cmd.args[0], cmd.args[1]);
+}
+
+void Script::ambientApply(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Apply loadad ambient sounds", cmd.op);
+
+	_vm->_ambient->applySounds(1);
+}
+
+void Script::ambientApplyWithFadeDelay(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Apply loadad ambient sounds with fade out delay : %d", cmd.op, cmd.args[0]);
+
+	_vm->_ambient->applySounds(_vm->_state->valueOrVarValue(cmd.args[0]));
+}
+
+void Script::soundPlayBadClick(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Play bad click sound", cmd.op);
+
+	_vm->_sound->playEffect(697, 5);
+}
+
+void Script::soundPlayBlocking(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Play skippable sound %d", cmd.op, cmd.args[0]);
+
+	int16 soundId = cmd.args[0];
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 heading = _vm->_state->valueOrVarValue(cmd.args[2]);
+	int32 att = _vm->_state->valueOrVarValue(cmd.args[3]);
+	bool nonBlocking = _vm->_state->valueOrVarValue(cmd.args[4]);
+	_vm->_sound->playEffect(soundId, volume, heading, att);
+
+	if (nonBlocking || !_vm->_sound->isPlaying(soundId)) {
+		return;
+	}
+
+	while (_vm->_sound->isPlaying(soundId) && !_vm->inputEscapePressed()) {
+		_vm->processInput(true);
+		_vm->drawFrame();
+	}
+}
+
 void Script::soundPlay(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Play sound %d", cmd.op, cmd.args[0]);
 
@@ -2341,6 +2485,219 @@ void Script::soundPlayVolumeDirectionAtt(Context &c, const Opcode &cmd) {
 	_vm->_sound->playEffect(cmd.args[0], volume, heading, att);
 }
 
+void Script::soundStopEffect(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Stop sound effect %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+
+	_vm->_sound->stopEffect(id, 0);
+}
+
+void Script::soundFadeOutEffect(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Stop sound effect %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 fadeDuration = _vm->_state->valueOrVarValue(cmd.args[1]);
+
+	_vm->_sound->playEffect(id, fadeDuration);
+}
+
+void Script::soundPlayLooping(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Play sound effect looping %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+
+	_vm->_sound->playEffectLooping(id, 100);
+}
+
+void Script::soundPlayFadeInOut(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Play sound effect fade in fade out %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 fadeInDuration = _vm->_state->valueOrVarValue(cmd.args[2]);
+
+	int32 playDuration;
+	if (cmd.args[3] == -1) {
+		playDuration = 108000;
+	} else {
+		playDuration = _vm->_state->valueOrVarValue(cmd.args[3]);
+	}
+
+	int32 fadeOutDuration = _vm->_state->valueOrVarValue(cmd.args[4]);
+
+	_vm->_sound->playEffectFadeInOut(id, volume, 0, 0, fadeInDuration, playDuration, fadeOutDuration);
+}
+
+void Script::soundChooseNext(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Setup next sound with control var %d", cmd.op, cmd.args[0]);
+
+	int16 controlVar = cmd.args[0];
+	int16 startSoundId = cmd.args[1];
+	int16 soundCount = cmd.args[2];
+	int32 soundMinDelay = _vm->_state->valueOrVarValue(cmd.args[3]);
+	int32 soundMaxDelay = _vm->_state->valueOrVarValue(cmd.args[4]);
+
+	_vm->_sound->setupNextSound(kNext, controlVar, startSoundId, soundCount, soundMinDelay, soundMaxDelay);
+}
+
+void Script::soundRandomizeNext(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Setup next sound with control var %d", cmd.op, cmd.args[0]);
+
+	int16 controlVar = cmd.args[0];
+	int16 startSoundId = cmd.args[1];
+	int16 soundCount = cmd.args[2];
+	int32 soundMinDelay = _vm->_state->valueOrVarValue(cmd.args[3]);
+	int32 soundMaxDelay = _vm->_state->valueOrVarValue(cmd.args[4]);
+
+	_vm->_sound->setupNextSound(kRandom, controlVar, startSoundId, soundCount, soundMinDelay, soundMaxDelay);
+}
+
+void Script::soundChooseNextAfterOther(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Setup next sound with control var %d", cmd.op, cmd.args[0]);
+
+	int16 controlVar = cmd.args[0];
+	int16 startSoundId = cmd.args[1];
+	int16 soundCount = cmd.args[2];
+	int32 soundMinDelay = _vm->_state->valueOrVarValue(cmd.args[3]);
+	int32 soundMaxDelay = _vm->_state->valueOrVarValue(cmd.args[4]);
+
+	int32 controlSoundId = _vm->_state->valueOrVarValue(cmd.args[5]);
+	int32 controlSoundMaxPosition = _vm->_state->valueOrVarValue(cmd.args[6]);
+
+	_vm->_sound->setupNextSound(kNextIfOtherStarting, controlVar, startSoundId, soundCount, soundMinDelay, soundMaxDelay, controlSoundId, controlSoundMaxPosition);
+}
+
+void Script::soundRandomizeNextAfterOther(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Setup next sound with control var %d", cmd.op, cmd.args[0]);
+
+	int16 controlVar = cmd.args[0];
+	int16 startSoundId = cmd.args[1];
+	int16 soundCount = cmd.args[2];
+	int32 soundMinDelay = _vm->_state->valueOrVarValue(cmd.args[3]);
+	int32 soundMaxDelay = _vm->_state->valueOrVarValue(cmd.args[4]);
+
+	int32 controlSoundId = _vm->_state->valueOrVarValue(cmd.args[5]);
+	int32 controlSoundMaxPosition = _vm->_state->valueOrVarValue(cmd.args[6]);
+
+	_vm->_sound->setupNextSound(kRandomIfOtherStarting, controlVar, startSoundId, soundCount, soundMinDelay, soundMaxDelay, controlSoundId, controlSoundMaxPosition);
+}
+
+void Script::ambientSetFadeOutDelay(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set fade out delay : %d", cmd.op, cmd.args[0]);
+
+	_vm->_state->setAmbiantPreviousFadeOutDelay(cmd.args[0]);
+}
+
+void Script::ambientAddSound1(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Add ambient sound %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+
+	_vm->_ambient->addSound(id, volume, 0, 0, 0, 0);
+}
+
+void Script::ambientAddSound2(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Add ambient sound %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 fadeOutDelay = cmd.args[2];
+
+	_vm->_ambient->addSound(id, volume, 0, 0, 0, fadeOutDelay);
+}
+
+void Script::ambientAddSound3(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Add ambient sound %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 heading = cmd.args[2];
+
+	_vm->_ambient->addSound(id, volume, heading, 85, 0, 0);
+}
+
+void Script::ambientAddSound4(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Add ambient sound %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 heading = cmd.args[2];
+	int32 angle = cmd.args[3];
+
+	_vm->_ambient->addSound(id, volume, heading, angle, 0, 0);
+}
+
+void Script::ambientAddSound5(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Add ambient sound %d", cmd.op, cmd.args[0]);
+
+	int32 id = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 u1 = _vm->_state->valueOrVarValue(cmd.args[2]);
+
+	_vm->_ambient->addSound(id, volume, 0, 0, u1, 0);
+}
+
+void Script::ambientSetCue1(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set ambient cue %d", cmd.op, cmd.args[0]);
+
+	int32 id = cmd.args[0];
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+
+	_vm->_ambient->setCueSheet(id, volume, 0, 0);
+}
+
+void Script::ambientSetCue2(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set ambient cue %d", cmd.op, cmd.args[0]);
+
+	int32 id = cmd.args[0];
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 heading = cmd.args[2];
+
+	_vm->_ambient->setCueSheet(id, volume, heading, 85);
+}
+
+void Script::ambientSetCue3(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set ambient cue %d", cmd.op, cmd.args[0]);
+
+	int32 id = cmd.args[0];
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 heading = cmd.args[2];
+	int32 angle = cmd.args[3];
+
+	_vm->_ambient->setCueSheet(id, volume, heading, angle);
+}
+
+void Script::ambientSetCue4(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set ambient cue %d", cmd.op, cmd.args[0]);
+
+	int32 id = cmd.args[0];
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+
+	_vm->_ambient->setCueSheet(id, volume, 32766, 85);
+}
+
+void Script::runAmbientScriptNode(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Run ambient script for node %d",
+			cmd.op, cmd.args[0]);
+
+	int32 node = _vm->_state->valueOrVarValue(cmd.args[0]);
+	_vm->runAmbientScripts(node);
+}
+
+void Script::runAmbientScriptNodeRoomAge(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Run sound script for node %d, room %d, age %d",
+			cmd.op, cmd.args[2], cmd.args[1], cmd.args[0]);
+
+	int32 node = _vm->_state->valueOrVarValue(cmd.args[2]);
+	_vm->_ambient->_scriptRoom = _vm->_state->valueOrVarValue(cmd.args[1]);
+	_vm->_ambient->_scriptAge = _vm->_state->valueOrVarValue(cmd.args[0]);
+
+	_vm->runAmbientScripts(node);
+	_vm->_ambient->scaleVolume(_vm->_state->valueOrVarValue(cmd.args[3]));
+}
+
 void Script::runSoundScriptNode(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Run sound script for node %d",
 			cmd.op, cmd.args[0]);
@@ -2368,6 +2725,50 @@ void Script::runSoundScriptNodeRoomAge(Context &c, const Opcode &cmd) {
 	_vm->runBackgroundSoundScriptsFromNode(node, room, age);
 }
 
+void Script::soundStopMusic(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Stop music", cmd.op);
+
+	int32 fadeOutDuration = _vm->_state->valueOrVarValue(cmd.args[0]);
+
+	_vm->_sound->stopMusic(fadeOutDuration);
+}
+
+void Script::movieSetStartupSound(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set movie startup sound %d", cmd.op, cmd.args[0]);
+
+	int32 soundId = _vm->_state->valueOrVarValue(cmd.args[0]);
+
+	_vm->_state->setMovieStartSoundId(soundId);
+	_vm->_state->setMovieStartSoundVolume(100);
+	_vm->_state->setMovieStartSoundHeading(0);
+	_vm->_state->setMovieStartSoundAttenuation(0);
+}
+
+void Script::movieSetStartupSoundVolume(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set movie startup sound %d", cmd.op, cmd.args[0]);
+
+	int32 soundId = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+
+	_vm->_state->setMovieStartSoundId(soundId);
+	_vm->_state->setMovieStartSoundVolume(volume);
+	_vm->_state->setMovieStartSoundHeading(0);
+	_vm->_state->setMovieStartSoundAttenuation(0);
+}
+
+void Script::movieSetStartupSoundVolumeH(Context &c, const Opcode &cmd) {
+	debugC(kDebugScript, "Opcode %d: Set movie startup sound %d", cmd.op, cmd.args[0]);
+
+	int32 soundId = _vm->_state->valueOrVarValue(cmd.args[0]);
+	int32 volume = _vm->_state->valueOrVarValue(cmd.args[1]);
+	int32 heading = _vm->_state->valueOrVarValue(cmd.args[2]);
+
+	_vm->_state->setMovieStartSoundId(soundId);
+	_vm->_state->setMovieStartSoundVolume(volume);
+	_vm->_state->setMovieStartSoundHeading(heading);
+	_vm->_state->setMovieStartSoundAttenuation(0);
+}
+
 void Script::drawOneFrame(Context &c, const Opcode &cmd) {
 	debugC(kDebugScript, "Opcode %d: Draw one frame", cmd.op);
 
@@ -2420,4 +2821,4 @@ void Script::newGame(Context &c, const Opcode &cmd) {
 	_vm->_inventory->reset();
 }
 
-} /* namespace Myst3 */
+} // End of namespace Myst3
