@@ -218,15 +218,12 @@ bool ThemeParser::parserCallback_cursor(ParserNode *node) {
 		return true;
 	}
 
-	int spotx, spoty, scale;
+	int spotx, spoty;
 
 	if (!parseIntegerKey(node->values["hotspot"], 2, &spotx, &spoty))
 		return parserError("Error parsing cursor Hot Spot coordinates.");
 
-	if (!parseIntegerKey(node->values["scale"], 1, &scale))
-		return parserError("Error parsing cursor scale.");
-
-	if (!_theme->createCursor(node->values["file"], spotx, spoty, scale))
+	if (!_theme->createCursor(node->values["file"], spotx, spoty))
 		return parserError("Error creating Bitmap Cursor.");
 
 	return true;
@@ -338,11 +335,15 @@ bool ThemeParser::parserCallback_drawstep(ParserNode *node) {
 
 	drawstep->drawingCall = getDrawingFunctionCallback(functionName);
 
-	if (drawstep->drawingCall == 0)
+	if (drawstep->drawingCall == 0) {
+		delete drawstep;
 		return parserError(functionName + " is not a valid drawing function name");
+	}
 
-	if (!parseDrawStep(node, drawstep, true))
+	if (!parseDrawStep(node, drawstep, true)) {
+		delete drawstep;
 		return false;
+	}
 
 	_theme->addDrawStep(getParentNode(node)->values["id"], *drawstep);
 	delete drawstep;
@@ -551,11 +552,11 @@ bool ThemeParser::parseDrawStep(ParserNode *stepNode, Graphics::DrawStep *drawst
 		else
 			return parserError("'" + stepNode->values["fill"] + "' is not a valid fill mode for a shape.");
 	}
-	
+
 	if (stepNode->values.contains("padding")) {
 		val = stepNode->values["padding"];
 		int pr, pt, pl, pb;
-		if (parseIntegerKey(val, 4, &pl, &pt, &pr, &pb)) 
+		if (parseIntegerKey(val, 4, &pl, &pt, &pr, &pb))
 			drawstep->padding.left = pl,
 			drawstep->padding.top = pt,
 			drawstep->padding.right = pr,
@@ -694,7 +695,7 @@ bool ThemeParser::parserCallback_layout(ParserNode *node) {
 			return false;
 	}
 
-	Common::parseBool(node->values["center"], center);
+	(void)Common::parseBool(node->values["center"], center);
 
 	if (node->values["type"] == "vertical")
 		_theme->getEvaluator()->addLayout(GUI::ThemeLayout::kLayoutVertical, spacing, center);
